<?xml version="1.0" encoding="UTF-8" ?>

<!--  dit2fo.xsl 1.0; 25 June 2001
 | DITA topic to PDF; "chapter"-level view, demo application
 |
 | Copyright IBM Corporation, 2001
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +======================================
 | 
 |Bugs:
 | FO processors are anything but interoperable at the time of this
 | release of the DITA FO demo (October 2001). There are possibly
 | other things to work on as well for this FO implementation, but
 | we know about these:
 |
 |  - this *demo application* has not been fully localization-enabled
 |  - something causes a page overflow error with XEP
 |  - column numbers do not total correctly for FOP (collapses some tables
 |    to the left, although others do appear to work okay)
 |  - "inherit" as an indent value does not work for FOP
 |  - white-space-collapse="false" is provided as an FOP equivalent to
 |    white-space="pre" but doesn't appear to work
 |  - leader-patter="use-content" is ignored by FOP
 |  - FOP starts a blank initial page; AH does not. layout defs may be the cause
 |  - Antenna House requires changing the "dflt-ext" variable to ".jpg"
 |    (and you must have equivalent graphics in the JPEG format)
 |  - the new semtable element currently spreads evenly across the number of
 |    defined entries; it has a @relativecols attribute to use for future
 |    methods of passing relative widths to its processor (possibly "* * 3*")
 |  - can't get left indent into "lq" so using surround box instead for now
 |
 *-->

<!DOCTYPE xsl:stylesheet [
<!-- entities for use in the generated output (must produce correctly in HTML) -->
  <!ENTITY gt            "&gt;"> 
  <!ENTITY lt            "&lt;"> 
  <!ENTITY rbl           "&#160;">
  <!ENTITY nbsp          "&#160;">
  <!ENTITY quot          "&#34;">
  <!ENTITY quotedblleft  "&#x201C;">
  <!ENTITY quotedblright "&#x201D;">
  <!ENTITY bullet        "&#x2022;"><!--check these two for better assignments -->
  <!ENTITY middot        "&#x2023;">

  <!ENTITY section       "&#xA7;">
  <!ENTITY endash        "&#x2013;">
  <!ENTITY emdash        "&#x2014;">

  <!ENTITY copyr         "&#xA9;">
  <!ENTITY trademark     "&#x2122;">
  <!ENTITY registered    "&#xAE;">

]>

<xsl:stylesheet version="1.0"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">


<!-- stylesheet imports -->


<xsl:output method="xml"
 version="1.0" indent="no"/>

<!-- SETTINGS: -->
<xsl:variable name="trace">no</xsl:variable> <!--set string to 'yes' to turn on trace -->

<!-- set up common attributes for all page definitions -->
<xsl:attribute-set name="common-grid">
  <xsl:attribute name="page-width">51pc</xsl:attribute> <!-- A4: 210mm -->
  <xsl:attribute name="page-height">66pc</xsl:attribute> <!-- A4: 297mm -->
  <xsl:attribute name="margin-top">3pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">3pc</xsl:attribute>
  <xsl:attribute name="margin-left">6pc</xsl:attribute>
  <xsl:attribute name="margin-right">6pc</xsl:attribute>
</xsl:attribute-set>

<!-- CONTROL PARAMETERS: -->
<!-- offset -->
<xsl:param name="basic-start-indent">72pt</xsl:param>
<xsl:param name="basic-end-indent">24pt</xsl:param>
<xsl:param name="basic-first-indent">84pt</xsl:param>


<!-- GLOBALS: -->
<xsl:variable name="img-path"></xsl:variable> <!-- this will be specific to your delivery tree! -->
<xsl:variable name="dflt-ext">.gif</xsl:variable> <!-- this sets whatever your preferred extension is for your graphics -->
                                                  <!-- For Antenna House, override to ".jpg" -->



<!-- TBD: move localizable rules/defs to external, language-controlled imports -->
<!-- ITEMS FOR TRANSLATION/GLOBALIZATION -->

<!-- various labels (not author controlled) -->
<xsl:variable name="deftxt-nota">Note</xsl:variable>
<xsl:variable name="deftxt-fig">Figure</xsl:variable>
<xsl:variable name="deftxt-tbl">Table</xsl:variable>
<xsl:variable name="deftxt-art">Artwork</xsl:variable>
<xsl:variable name="deftxt-abstract">Abstract:</xsl:variable>
<xsl:variable name="deftxt-docnum">Document Number:</xsl:variable>
<xsl:variable name="deftxt-linktop">Return to Top</xsl:variable>
<xsl:variable name="deftxt-nextlink">Next topic</xsl:variable>
<xsl:variable name="deftxt-prevlink">Previous topic</xsl:variable>

<!-- peril notice labels -->
<xsl:variable name="deftxt-attention">Attention:</xsl:variable>
<xsl:variable name="deftxt-caution">Caution:</xsl:variable>
<xsl:variable name="deftxt-danger">Danger! </xsl:variable>

<!-- default title text for section level generated sections -->
<xsl:variable name="deftxt-toc">Contents</xsl:variable>
<xsl:variable name="deftxt-ptoc">Local topics</xsl:variable><!-- differentiated for tracing -->
<xsl:variable name="deftxt-relsect">Related Information</xsl:variable>
<xsl:variable name="deftxt-rellinks">Link Relations</xsl:variable>
<xsl:variable name="deftxt-endnotes">End Notes</xsl:variable>

<!-- default title text for reftopic special sections -->
<xsl:variable name="deftxt-refsyn">Syntax</xsl:variable>
<xsl:variable name="deftxt-refxmp">Examples</xsl:variable>

<!-- default title text for task special sections -->
<xsl:variable name="deftxt-prereq">Prerequisites</xsl:variable>
<xsl:variable name="deftxt-results">Results</xsl:variable>
<xsl:variable name="deftxt-optional">Optional: </xsl:variable>


<!-- Localizable rules -->

<xsl:template match="date|*[contains(@spec,' topic.date ')]" priority="1">
  <!-- apply lexical checking and locale-specific rendering for this content -->
  US-<xsl:apply-templates/>
</xsl:template>

<xsl:template match="time|*[contains(@spec,' topic.time ')]" priority="1">
  <!-- apply lexical checking and locale-specific rendering for this content -->
  US-$<xsl:apply-templates/>
</xsl:template>

<xsl:template match="currency|*[contains(@spec,' topic.currency ')]" priority="1">
  <!-- apply lexical checking and locale-specific rendering for this content -->
  US-<xsl:apply-templates/>
</xsl:template>


<!-- General Procs -->

<!-- null out some things whose content will be built elsewhere or excluded -->
<xsl:template match="figcap|figdesc"/>
<xsl:template match="tdesc"/>

<!-- footnotes can be collected as endnotes or cast as popups -->
<xsl:template match="fn">
<fo:footnote>
  <fo:inline baseline-shift="super" font-size="75%" font-family="Helvetica">
    <xsl:number count="//fn"    format="1" />
  </fo:inline>
  <fo:footnote-body>
    <!--fo:block space-after="0pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="1in"/>
    </fo:block-->
    <fo:block xsl:use-attribute-sets="footnote">
      <fo:inline baseline-shift="super" font-size="75%">
         <xsl:number level="any" count="//fn"  format="1 "/>
      </fo:inline>
      <xsl:apply-templates />
    </fo:block>
  </fo:footnote-body>
</fo:footnote>
</xsl:template>




<!-- ==== START OF DITA COMPLIANT TRANSFORMS ==== -->

<!-- For this stylesheet, the document element is 
     presumed to be either a topic or a specialized topic. -->



<!-- TYPED TOPIC MAIN STRUCTURES -->

<!-- root rule... just fall through; no side effects for new delivery contexts -->

<xsl:template match="/">
  <xsl:comment>Sample DITA to XSL-FO application by IBM Corp., 2001</xsl:comment>
  <xsl:apply-templates/>
</xsl:template>



<!-- these template rules generate the outer-level shell for a standalone delivery context -->

<!-- h1 (& setup, in the case of standalone) -->
<xsl:template match="/*[contains(@spec,' topic.topic ')]">
<!-- this is an "h1div" context for standalone documents -->
  <xsl:call-template name="chapter-setup"/>
</xsl:template>

<xsl:template match="/*/*[contains(@spec,' topic.title ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at h1 heading context FOR STANDALONE TOPICS!}</fo:block></xsl:if>
  <xsl:call-template name="h1heading"/>
</xsl:template>

<!-- h2 -->
<xsl:template name="h2div" match="/*/*[contains(@spec, 'topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at "name='h2div'"}</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="/*/*/*[contains(@spec,' topic.title ')]">
  <xsl:call-template name="h2heading"/>
</xsl:template>

<!-- h3 -->
<xsl:template name="h3div" match="/*/*/*[contains(@spec, 'topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at "name='h3div'"}</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="/*/*/*/*[contains(@spec,' topic.title ')]">
  <xsl:call-template name="h3heading"/>
</xsl:template>

<!-- h4 -->
<xsl:template name="h4div" match="/*/*/*/*[contains(@spec, 'topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at "name='h4div'"}</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="/*/*/*/*/*[contains(@spec,' topic.title ')]">
  <xsl:call-template  name="h4heading"/>
</xsl:template>

<!-- h5 -->
<xsl:template name="h5div" match="/*/*/*/*/*[contains(@spec,' topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at "name='h5div'"}</fo:block></xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="/*/*/*/*/*/*[contains(@spec, 'topic.title ')]">
  <xsl:call-template  name="h5heading"/>
</xsl:template>

<!-- h6 -->
<xsl:template name="h6div" match="/*/*/*/*/*/*[contains(@spec,' topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at "name='h6div'"}</fo:block></xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="/*/*/*/*/*/*/*[contains(@spec, 'topic.title ')]">
  <xsl:call-template  name="h6heading"/>
</xsl:template>

<!-- h7 --> <!-- no absolute root now... -->
<xsl:template name="h7div" match="*/*/*/*/*/*/*[contains(@spec,' topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at "name='h7div'" and all child divs from there}</fo:block></xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*/*/*/*/*/*/*/*[contains(@spec, 'topic.title ')]">
  <xsl:call-template  name="h7heading"/>
</xsl:template>






<!-- titles get level appropriate treatment, but are still processed for contained markup -->

<!-- 1st level heading -->
<xsl:template name="h1heading">
<xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='h1heading'"}</fo:block></xsl:if>
  <xsl:if test="@id"><fo:inline id="{@id}"></fo:inline></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle1" id="{generate-id()}" line-height="100%">
    <fo:block><fo:leader color="black" leader-pattern="rule" rule-thickness="3pt" leader-length="100%"/></fo:block>
    <fo:block>
      <fo:marker marker-class-name="chaptitle">
<xsl:variable name="title-txt">
        <xsl:call-template name="get-title"/>
</xsl:variable>
        <xsl:value-of select="$title-txt"/>
      </fo:marker>
    </fo:block>
  </fo:block>
  <!--xsl:call-template name="gen-body-qatoc"/--> <!-- only one of these will usually "take" -->
  <!--xsl:call-template name="gen-sect-qatoc"/-->
  <!--xsl:call-template name="gen-ptoc"/-->
</xsl:template>

<!-- 2nd level heading -->
<xsl:template name="h2heading">
<xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='h2heading'"}</fo:block></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle2" id="{generate-id()}">
    <fo:block space-after="3pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="100%"/>
    </fo:block>
    <xsl:call-template name="get-title"/>
  </fo:block>
</xsl:template>

<!-- 3rd level heading -->
<xsl:template name="h3heading">
<xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='h3heading'"}</fo:block></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle3" id="{generate-id()}">
    <xsl:call-template name="get-title"/>
  </fo:block>
</xsl:template>

<!-- 4th level heading -->
<xsl:template name="h4heading">
<xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='h4heading'"}</fo:block></xsl:if>
  <fo:block xsl:use-attribute-sets="topictitle4" id="{generate-id()}">
    <xsl:call-template name="get-title"/>
  </fo:block>
</xsl:template>

<!-- 5th level heading -->
<xsl:template name="h5heading">
  <fo:block xsl:use-attribute-sets="topictitle5" id="{generate-id()}">
    <xsl:call-template name="get-title"/><xsl:text>: </xsl:text>
  </fo:block>
</xsl:template>

<!-- 6th level heading -->
<xsl:template name="h6heading">
  <fo:block xsl:use-attribute-sets="topictitle6" id="{generate-id()}">
    <xsl:call-template name="get-title"/><xsl:text>: </xsl:text>
  </fo:block>
</xsl:template>


<!-- 7th level heading -->
<xsl:template name="h7heading">
  <fo:block  xsl:use-attribute-sets="topictitle6" font-style="italic">
    <xsl:call-template name="get-title"/><xsl:text>: </xsl:text>
  </fo:block>
</xsl:template>







<xsl:template name="chapter-setup">
<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">


  <fo:layout-master-set>

    <fo:simple-page-master master-name="cover"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="36pt"      margin-right="36pt">
      <fo:region-body margin-top="120pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="leftPage"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="72pt"      margin-right="60pt">
      <fo:region-body margin-bottom="36pt" margin-top="24pt" />
      <fo:region-before extent="24pt"/>
      <fo:region-after  extent="36pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="rightPage"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="60pt"      margin-right="72pt">
      <fo:region-body margin-bottom="36pt" margin-top="24pt" />
      <fo:region-before extent="24pt"/>
      <fo:region-after  extent="36pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="common-page"
      page-height="792pt"     page-width="612pt"
      margin-top="36pt"       margin-bottom="36pt"
      margin-left="60pt"      margin-right="60pt">
      <fo:region-body margin-bottom="36pt" margin-top="24pt" />
      <fo:region-before extent="24pt"/>
      <fo:region-after  extent="36pt"/>
    </fo:simple-page-master>

  </fo:layout-master-set>

  <!--xsl:call-template name="gen-cover-page"-->

  <!-- initiate content processing within basic page "shell" -->
  <fo:page-sequence master-name="common-page">

  <!-- header -->
  <fo:static-content flow-name="xsl-region-before" font-size="9pt" font-family="Helvetica">
    <fo:block linefeed-treatment="ignore" text-align-last="justify">
      <fo:inline text-align="start">
        <xsl:text>Stub for header content</xsl:text>
      </fo:inline>
      <fo:leader rule-style="use-content" leader-length.maximum="100%" leader-length.optimum="60%" leader-length.minimum="10pt" leader-pattern="use-content">&nbsp;</fo:leader>
      <fo:inline text-align="end" font-weight="bold">
        <xsl:text>right side content</xsl:text>
      </fo:inline>
    </fo:block>
    <!--fo:block><fo:leader leader-pattern="rule" leader-length="100%"/></fo:block--> 
  </fo:static-content>


  <!-- footer -->
  <fo:static-content flow-name="xsl-region-after">
    <fo:block linefeed-treatment="ignore" text-align-last="justify">
      <fo:inline text-align="start" font-size="10pt" font-family="Helvetica">
        <xsl:value-of select="//*/*[contains(@spec, ' topic.title ')]"/>
      </fo:inline>
      <!--fo:leader rule-style="use-content" leader-length.maximum="100%" leader-length.optimum="60%" leader-length.minimum="10pt" leader-pattern="use-content">&nbsp;</fo:leader-->
      <fo:inline text-align="end" font-size="10pt" font-weight="bold"  font-family="Helvetica">
        <xsl:text>Page </xsl:text><fo:page-number/>
      </fo:inline>
    </fo:block>
  </fo:static-content>

    <!-- body -->
    <fo:flow flow-name="xsl-region-body" >
      <fo:block line-height="12pt" font-size="10pt"
                font-family="Helvetica"
                id="page1-1">

         <xsl:apply-templates />
      </fo:block>
   </fo:flow>

  </fo:page-sequence>

</fo:root>
</xsl:template>


<xsl:template name="gen-cover-page">
  <!-- generate an "outside-front cover" page -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block text-align="center">
        <fo:block space-after.optimum="40pt"
                  space-before.optimum="40pt"
                  line-height="80pt" font-size="32pt"
                  font-family="Helvetica">
          <xsl:value-of select="/topic/title"/>
        </fo:block>
        <fo:block line-height="50pt" font-size="24pt" font-family="Helvetica">
           <fo:block font-style="italic">
             <xsl:value-of select="*[contains(@spec, ' topic.title ')]"/>
           </fo:block>
        </fo:block>
      </fo:block>
    </fo:flow>
  </fo:page-sequence>

  <!-- generate an "inside front cover" page (left side) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block xsl:use-attribute-sets="p" color="purple" text-align="center">[TBD: this page intentionally left blank.]</fo:block>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>





<!-- the following named templates generate inline content for the delivery context -->


<!-- named templates for labels and titles related to topic structures -->

<xsl:template name="get-title"><!-- get fully-processed title content by whatever mechanism -->
  <xsl:choose>
   <xsl:when test="@classif">
     <xsl:value-of select="//classdef[@classname=current()/@classif]/title"/>
   </xsl:when>
   <xsl:when test="@title">
     <xsl:value-of select="@title"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:apply-templates/> <!-- select="*[contains(@spec,' topic.title ')]"/-->
   </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="sect-heading">
  <xsl:param name="deftitle" select="."/> <!-- get param by reference -->
  <xsl:variable name="heading">
     <xsl:choose>
      <xsl:when test="@classif">
        <xsl:value-of select="//classdef[@classname=current()/@classif]/title"/>
      </xsl:when>
      <xsl:when test="@title">
        <xsl:value-of select="@title"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="title"/>
      </xsl:otherwise>
     </xsl:choose>
  </xsl:variable>

  <!-- based on graceful defaults, build an appropriate section-level heading -->
  <xsl:if test="$trace='yes'"><fo:block color="purple">{arrived at "name='sect-heading'"}</fo:block></xsl:if>
  <xsl:choose>
    <xsl:when test="not($heading='')">
      <xsl:if test="normalize-space($heading)=''">
        <!-- hack: a title with whitespace ALWAYS overrides as null -->
        <xsl:comment>no heading</xsl:comment>
      </xsl:if>
      <fo:block xsl:use-attribute-sets="section.title">
        <xsl:call-template name="proc-ing"/> <!-- this places a "Information Navigation Graphic" if pertinent -->
        <xsl:value-of select="$heading"/>
      </fo:block>
    </xsl:when>
    <xsl:when test="$deftitle">
      <fo:block xsl:use-attribute-sets="section.title">
        <xsl:value-of select="$deftitle"/>
      </fo:block>
    </xsl:when>
    <xsl:otherwise><!-- no output --></xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="proc-ing">
  <xsl:if test="$do-place-ing = 'yes'"> <!-- set in a global variable, as with label placement, etc. -->
    <fo:external-graphic src="tip-ing.jpg"/> <!-- this should be an xsl:choose with the approved list and a selection method-->
    <!-- add any other required positioning controls, if needed, but must be valid in the location
         from which the call to this template was made -->
    &nbsp;
  </xsl:if>
</xsl:template>




<!-- template rules not sensitive to nesting depth -->

<xsl:template match="*[contains(@spec,' topic.body ')]">
  <fo:block>
    <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>



<!-- Pass through things whose content will be used out of apply=templates sequence -->
<!-- (this includes titles of all section-like exhibits) -->


<xsl:template match="*[contains(@spec,' topic.desc ')]"/>
<!--xsl:template match="*[contains(@spec,' topic.title ')]"/-->
<xsl:template match="*[contains(@spec,' topic.section ')]/*[contains(@spec,' topic.title ')]" priority="100"/>
<xsl:template match="*[contains(@spec,' topic.fig ')]/*[contains(@spec,' topic.title ')]" priority="100"/>
<xsl:template match="*[contains(@spec,' topic.table ')]/*[contains(@spec,' topic.title ')]" priority="100"/>



  <!-- == TOPIC UNIQUE SUBSTRUCTURES == -->


<xsl:template match="*[contains(@spec,' topic.section ')]">
  <!-- place locally defined IDs for use by xref -->
  <xsl:if test="@id"><fo:inline id="{@id}"> </fo:inline></xsl:if>
  <xsl:call-template name="sect-heading">
     <xsl:with-param name="deftitle"></xsl:with-param>
  </xsl:call-template>
  <fo:block><!-- margin-top="15pt" line-height="12pt"-->
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>



<xsl:template match="*[contains(@spec,' topic.xmp ')]">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <xsl:apply-templates/>
</xsl:template>



  <!-- == REFTOPIC UNIQUE SUBSTRUCTURES == -->
  <!-- when a specialized rule is imbedded with the topic defaults, there will be
       a priority conflict unless you set the priority of the contained specialied rule
       so that it overrides the topic fallback -->

<xsl:template match="*[contains(@spec,' reftopic.refsect ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.refsyn ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.refxmp ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <xsl:apply-templates />
</xsl:template>

<!-- dl~derived models -->

<xsl:template match="*[contains(@spec,'reftopic.property ')]" priority="1">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.properties ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.proptype ')]" priority="1">        
 <fo:block xsl:use-attribute-sets="dt">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.propvalue ')]" priority="1">
 <fo:block xsl:use-attribute-sets="dd">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>


<!-- parameter lists (specialized DL within a refsect or refbody)  -->
<xsl:template match="*[contains(@spec,' reftopic.parml ')]" priority="1">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.plentry ')]" priority="1">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.pt ')]" priority="1">
 <fo:block xsl:use-attribute-sets="dt" id="pl_{.}">
  <xsl:apply-templates/>
 </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.pd ')]" priority="1">
 <fo:block xsl:use-attribute-sets="dd">
  <xsl:apply-templates/>
 </fo:block>
</xsl:template>



  <!-- == TASK UNIQUE SUBSTRUCTURES == -->

<xsl:template match="*[contains(@spec,' task.steps ')]" priority="1">
<fo:list-block provisional-distance-between-starts="15mm"
               provisional-label-separation="5mm">
  <xsl:apply-templates/>
</fo:list-block>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.step ')]" priority="1">
<xsl:variable name="list-level">1</xsl:variable>
<fo:list-item >
  <fo:list-item-label  end-indent="label-end()">
    <xsl:attribute name="start-indent">
    <xsl:choose>
      <xsl:when test="$list-level=1">
        <xsl:value-of select="$basic-start-indent" />
      </xsl:when>
    <xsl:otherwise>+5mm</xsl:otherwise><!-- inherit? -->
    </xsl:choose>
    </xsl:attribute>
    <fo:block ><xsl:number format="1."/></fo:block>
  </fo:list-item-label>
  <fo:list-item-body  end-indent="{$basic-end-indent}" text-align="justify" >
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          <xsl:value-of select="$basic-start-indent"/> + 12pt</xsl:when>
        <xsl:otherwise>inherit + 24pt</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block>
      <!-- optreq support implemented here -->
      <xsl:if test="@importance='optional' or @optreq='opt'">
         <fo:inline font-weight="bold">
          <xsl:value-of select="$deftxt-optional" />
         </fo:inline>
      </xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
</fo:list-item>
</xsl:template>



<xsl:template match="*[contains(@spec,' task.cmd ')]" priority="1">
  <fo:inline><xsl:apply-templates/></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.context ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template> 

<xsl:template match="*[contains(@spec,' task.prereq ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template> 

<xsl:template match="*[contains(@spec,' task.postreq ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template> 

<xsl:template match="*[contains(@spec,' task.result ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template> 

<xsl:template match="*[contains(@spec,' task.substeps ')]" priority="1">
<fo:list-block provisional-distance-between-starts="15mm"
               provisional-label-separation="5mm">
  <!-- apply preferred "alpha roman" output style here, not decimal numbers -->
  <xsl:apply-templates/>
</fo:list-block>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.substep ')]" priority="1">
<xsl:variable name="list-level">2</xsl:variable><!-- explicit nested level -->
<fo:list-item >
  <fo:list-item-label  end-indent="label-end()">
    <xsl:attribute name="start-indent">
    <xsl:choose>
      <xsl:when test="$list-level=1">
        <xsl:value-of select="$basic-start-indent" />
      </xsl:when>
    <xsl:otherwise>+5mm</xsl:otherwise><!-- inherit? -->
    </xsl:choose>
    </xsl:attribute>
    <fo:block ><xsl:number format="1."/></fo:block>
  </fo:list-item-label>
  <fo:list-item-body start-indent="body-start()" end-indent="{$basic-end-indent}" text-align="justify" >
    <fo:block>
      <!-- optreq support implemented here -->
      <xsl:if test="@importance='optional' or @optreq='opt'">
         <fo:inline font-weight="bold">
          <xsl:value-of select="$deftxt-optional" />
         </fo:inline>
      </xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
</fo:list-item>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.choices ')]" priority="1">
  <fo:list-block provisional-distance-between-starts="15mm"  provisional-label-separation="5mm" xsl:use-attribute-sets="p">
    <xsl:apply-templates/>
  </fo:list-block>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.choice ')]" priority="1">
  <xsl:variable name="list-level">1</xsl:variable><!-- preset for now -->
  <fo:list-item>
    <fo:list-item-label end-indent="label-end()">
      <xsl:attribute name="start-indent">
        <xsl:choose>
          <xsl:when test="$list-level=1">
            <xsl:value-of select="$basic-start-indent"/></xsl:when>
          <xsl:otherwise>inherit + 12pt</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <fo:block>
        <fo:inline font-family="Symbol"><!--&#183;--></fo:inline>
      </fo:block>
    </fo:list-item-label>
  <fo:list-item-body start-indent="body-start()" text-align="justify" end-indent="{$basic-end-indent}">
    <fo:block>
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
  </fo:list-item>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.ifcond ')]" priority="1">
  <xsl:variable name="list-level">2</xsl:variable><!-- preset for now -->
  <fo:list-item>
    <fo:list-item-label end-indent="label-end()">
  <!--
      <xsl:attribute name="start-indent">
        <xsl:choose>
          <xsl:when test="$list-level=1">
            <xsl:value-of select="$basic-start-indent"/></xsl:when>
          <xsl:otherwise>inherit + 12pt</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
  -->
      <fo:block>
        <fo:inline font-family="Symbol"><!--&#183;--></fo:inline>
      </fo:block>
    </fo:list-item-label>
  <fo:list-item-body start-indent="body-start()" text-align="justify" end-indent="{$basic-end-indent}">
    <fo:block>
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
  </fo:list-item>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.stepresult ')]" priority="1">
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template>

<!-- these are derived from "liblock"-->
<xsl:template match="*[contains(@spec,' task.info ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.tutorialinfo ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.taskxmp ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template>


<!-- ========= regular body content |*[contains(@spec,' topic.')]============== -->


<!-- list item block like li/lq but without presentation (indent) -->
<xsl:template match="*[contains(@spec,' topic.liblock ')]">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <fo:block xsl:use-attribute-sets="p"><xsl:apply-templates /></fo:block>
</xsl:template>


<!-- newly added "XHTML compatibility subset" extras -->

<xsl:template match="*[contains(@spec,' topic.b ')]">
<fo:inline font-weight="bold"><xsl:apply-templates /></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.i ')]">
<fo:inline font-style="italic"><xsl:apply-templates /></fo:inline>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.strong ')]">
<fo:inline font-weight="bold"><xsl:apply-templates /></fo:inline>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.em ')]">
<fo:inline xsl:use-attribute-sets="em"><xsl:apply-templates /></fo:inline>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.tt ')]">
<fo:inline xsl:use-attribute-sets="tt"><xsl:apply-templates /></fo:inline>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.code ')]">
<fo:inline xsl:use-attribute-sets="tt"><xsl:apply-templates /></fo:inline>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.samp ')]">
<fo:inline xsl:use-attribute-sets="tt"><xsl:apply-templates /></fo:inline>
</xsl:template> 

<!-- others? dfn, kbd, kdw? -->


<!-- TYPED CONTENT (not likely to be further specialized at this granularity) -->


<xsl:template match="*[contains(@spec,' topic.synph ')]">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.comment ')]">
  <fo:inline color="purple" font-style="italic"><xsl:apply-templates/></fo:inline>
</xsl:template>


<!-- syntax diagram markup language -->

<xsl:template match="*[contains(@spec,' topic.syntax ')]">
  <fo:block xsl:use-attribute-sets="program">
   <xsl:if test="@id"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
    <xsl:apply-templates />
  </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.kbd ')]">
  <fo:inline xsl:use-attribute-sets="tt" font-weight="bold"><xsl:apply-templates /></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.kwd ')]">
  <fo:inline font-weight="bold"><xsl:apply-templates /></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.var ')]">
  <fo:inline font-style="italic"><xsl:apply-templates /></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.flag ')]">
  <fo:inline font-weight="bold"><xsl:apply-templates /></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.parm ')]">
  <fo:inline font-weight="bold"><xsl:apply-templates /></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.delim ')]">
 <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.oper ')]">
 <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sep ')]">
 <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.group ')]">
<xsl:choose>
  <xsl:when test="@composite='composite'">
    <xsl:apply-templates/>
  </xsl:when>
  <xsl:otherwise>
    <fo:block xsl:use-attribute-sets="program">
      <xsl:apply-templates/>
    </fo:block>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>


<!-- Support for Scripts -->
<!-- Not Applicable for FO -->
<!-- unless there is special support for PDF links, etc. -->




<!-- NAMED TEMPLATES (call by name, only) -->

<xsl:template name="gen-att-label">
<xsl:if test="string(@title)">
  <fo:block margin-bottom="0"><fo:inline font-weight="bold"><xsl:value-of select="@title"/></fo:inline></fo:block>
</xsl:if>
</xsl:template>


<!-- named templates that can be used anywhere -->

<!-- this replaces newlines with the BR element, forcing non-concatenation even in flow contexts -->
<xsl:template name="br-replace">
  <xsl:param name="word"/>
<!-- capture an actual newline within the xsl:text element -->
  <xsl:variable name="cr"><xsl:text>
</xsl:text></xsl:variable>
  <xsl:choose>
    <xsl:when test="contains($word,$cr)">
       <xsl:value-of select="substring-before($word,$cr)"/>
       <br/>
       <xsl:call-template name="br-replace">
         <xsl:with-param name="word" select="substring-after($word,$cr)"/>
       </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$word"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- diagnostic: call this to generate a path-like view of an element's ancestry! -->
<xsl:template name="breadcrumbs">
<xsl:variable name="full-path">
  <xsl:for-each select="ancestor-or-self::*">
    <xsl:value-of select="concat('/',name())"/>
  </xsl:for-each>
</xsl:variable>
<fo:block><fo:inline text-decoration="underline"><xsl:value-of select="$full-path"/></fo:inline></fo:block>
</xsl:template>

<!-- not included:
  gen-relsect
  gen-endnotes
  gen-toc
  gen-ptoc
  renderrel
as these involved a lot of HTML to clean up for FO
-->  




<!-- Particular support for top-level and section-level qalists -->
<!-- Note: qalists in other contexts may not show; extend as necessary -->

<xsl:template name="gen-sect-qatoc">
  <xsl:if test="*/section/qalist"><!-- only in case of sectioned qalists -->
  <fo:block xsl:use-attribute-sets="p">Sections in this FAQ list</fo:block>
    <xsl:for-each select="*/section">
     <fo:block xsl:use-attribute-sets="p" font-weight="bold"><xsl:value-of select="title"/></fo:block>
       <xsl:call-template name="gen-qatoc"/>
    </xsl:for-each>
  <!--hr/-->
  </xsl:if>
</xsl:template> 

<xsl:template name="gen-body-qatoc">
  <xsl:if test="*/qalist"><!-- only in case of child qalists -->
  <fo:block xsl:use-attribute-sets="p">Questions in this FAQ list</fo:block>
    <xsl:call-template name="gen-qatoc"/>
  <!--hr/-->
  </xsl:if>
</xsl:template>
 
<xsl:template name="gen-qatoc">
<xsl:for-each select="qalist/qaentry">
  <fo:block xsl:use-attribute-sets="p" id="{generate-id()}">
       <xsl:value-of select="qitem"/>
     <!--recursive call for subtopics here"/-->
  </fo:block>
</xsl:for-each>
</xsl:template>


<!-- ALTERNATE TERM/LINK ELEMENTS -->

<xsl:template name="altterm"> <!-- call from any match="term"-like template -->
  <fo:inline text-decoration="underlined">
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>

<xsl:template name="altlink"> <!-- call from any match="link"-like template -->
    <xsl:apply-templates />
</xsl:template>





<!-- functional elements -->

<xsl:template match="*[contains(@spec,' topic.null ')]"><xsl:value-of select="name()"/></xsl:template>

<xsl:template match="include|topicref" name="do-include">
  <!-- for softcopy docs, this section can generate a link rather than fetching content -->

  <!-- using "$ifile,." causes lookup to be relative to the parent document -->
  <xsl:variable name="ifile"><xsl:value-of select="@href"/></xsl:variable>
  <xsl:variable name="extfile" select="document($ifile,.)"/>
  <xsl:apply-templates select="$extfile"/> <!-- this should do proper fall-through -->
</xsl:template>


<!-- catch undefined elements, for stylesheet maintainers -->

<xsl:template match="*">
  <fo:inline font-weight="bold" background="yellow">
     <fo:inline font-weight="bold">
       <xsl:text>[</xsl:text>
       <xsl:for-each select="ancestor-or-self::*">
        <xsl:text>/</xsl:text>
        <xsl:value-of select="name()" />
       </xsl:for-each>
       {"<xsl:value-of select="@spec"/>"}<xsl:text>) </xsl:text>
     </fo:inline>
    <xsl:apply-templates />
     <fo:inline font-weight="bold">
       <xsl:text> (</xsl:text><xsl:value-of select="name()"/><xsl:text>]</xsl:text>
     </fo:inline>
  </fo:inline>
</xsl:template>


<!-- catch required cleanup, for authors -->

<xsl:template match="required-cleanup">
  <fo:inline background="yellow" color="#CC3333">  <!-- indents won't apply here; not a block context -->
     <fo:inline font-weight="bold">Required Cleanup <xsl:if test="string(@remap)">(<xsl:value-of select="@remap"/>) </xsl:if><xsl:text>: </xsl:text></fo:inline> 
     <xsl:apply-templates />
  </fo:inline>
</xsl:template>

<!-- ============= content relocation from the prolog's elements ================== -->

<!-- PROLOG -->

<xsl:template match="prolog">
  <!-- "review" = display all metadata; "publish" = functional bits only -->
  <xsl:call-template name="publish"/>
</xsl:template>

<xsl:template name="review">
</xsl:template>

<xsl:template name="publish">
 <!-- shortdesc and longdesc may be used for "progressive disclosure" leading
      eventually to bringing up the topic itself.  The default presentation is
      to not use them directly in presented topics. -->
 <!--xsl:apply-templates select="shortdesc"/-->
 <!--xsl:apply-templates select="longdesc"/-->
</xsl:template>



<!-- skipped most metadata and prolog for now -->





<!-- things being added back in -->
<!-- ===================================================================== -->

<!-- basic document elements -->

<!-- special formatting for first paragraph in a section-->
<xsl:template match="section/p[1]|section/*[contains(@spec,' topic.p ')]" priority="2">
  <fo:block xsl:use-attribute-sets="divlike.p">
    <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
    <xsl:apply-templates />
  </fo:block>
</xsl:template>

<!-- extend with special handling of empty p elements with anchors-->
<xsl:template match="*[contains(@spec,' topic.p ')]">
   <xsl:choose>
     <xsl:when test="not(text()[normalize-space(.)] | *)">
       <fo:block/>
       <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
     </xsl:when>
     <xsl:otherwise>  
       <xsl:choose>
         <xsl:when test="position()=1">
           <fo:block xsl:use-attribute-sets="divlike.p">
             <xsl:apply-templates/>
           </fo:block>
         </xsl:when>
         <xsl:otherwise>
           <fo:block xsl:use-attribute-sets="p">
             <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
             <xsl:apply-templates/>
           </fo:block>
         </xsl:otherwise>
       </xsl:choose>
     </xsl:otherwise>
   </xsl:choose>    
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.lq ')]">
  <fo:block xsl:use-attribute-sets="lq"><xsl:apply-templates /></fo:block>
</xsl:template>

<!-- this rule is unique for FO output; HTML handles first-para mixed content implicityly -->
<xsl:template match="li/*[contains(@spec,' topic.p ')]" priority="1">
  <fo:block xsl:use-attribute-sets="top.p"><xsl:apply-templates /></fo:block>
</xsl:template>





<!-- this template rule must discern between either direct reference or include with mimetype -->
<xsl:template match="*[contains(@spec,' topic.mmobj ')]">
  <!-- build any pre break indicated by style -->
  <xsl:choose>
    <xsl:when test="@runin='runin'">
    <xsl:call-template name="unused-insert-graphic"/> <!-- no positioning or breaking props (can scale, tho)-->
    </xsl:when>
    <xsl:otherwise>
      <fo:block><!-- text-align="left"--> <!-- how to influence this setting from doc or SS?-->
        <!--fo:external-graphic src="{@href}"/-->
        <xsl:call-template name="unused-insert-graphic"/>
      </fo:block>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.textalt ')]">
  <!--xsl:apply-templates/--> <!-- NOP for now... user profile issue? -->
</xsl:template>

<xsl:template name="unused-insert-graphic">
   <fo:external-graphic> <!-- apply scaling attributes here? -->
      <xsl:attribute name="src"><!--hard force the extension to jpg -->
        <xsl:variable name="thisfn" select="@href"/>
        <xsl:value-of select="concat($img-path, substring-before($thisfn,'.'), $dflt-ext)"/> <!-- .jpg -->
      </xsl:attribute>
      <xsl:choose>
        <xsl:when test="@scale"><xsl:attribute name="content-width"><xsl:value-of select="concat(@scale*100,'%')"/></xsl:attribute></xsl:when>
        <xsl:otherwise><!--xsl:attribute name="content-width">scale-to-fit</xsl:attribute--></xsl:otherwise> <!-- no effect -->
      </xsl:choose>
    </fo:external-graphic>
</xsl:template>


<!-- SINGLE PART LISTS -->

<xsl:template name="li.name">
     <xsl:if test="string(@id)"><fo:inline id="{@id}"> </fo:inline></xsl:if>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.ul ')]|*[contains(@spec,' topic.ol ')]">
  <fo:list-block provisional-distance-between-starts="3pt"
                 provisional-label-separation="3pt">
    <xsl:apply-templates/>
  </fo:list-block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.notelist ')]">
  <fo:block xsl:use-attribute-sets="p">
    <fo:inline font-weight="bold">
    <xsl:choose>
      <xsl:when test="@title">
        <xsl:if test="not(@title='')">
          <xsl:value-of select="@title"/>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$deftxt-nota"/>
      </xsl:otherwise>
    </xsl:choose><xsl:text>: </xsl:text>
    </fo:inline>
    <!--fo:inline font-weight="bold">Notes:</fo:inline-->
  </fo:block>
  <fo:list-block provisional-distance-between-starts="3pt"
                 provisional-label-separation="3pt">
    <xsl:apply-templates/>
  </fo:list-block>
</xsl:template>


<!-- add "compact" awareness here -->
<xsl:template match="ol/*[contains(@spec,' topic.li ')]|notelist/*[contains(@spec,' topic.li ')]">
<xsl:variable name="list-level" select="count(ancestor-or-self::ul | 
                                         ancestor-or-self::dl |
                                         ancestor-or-self::qalist |
                                         ancestor-or-self::ol |
                                         ancestor-or-self::notelist )" />
  <fo:list-item space-before="6pt"> <!--.optimum="4pt", or deal with "compact=''" value -->
  <fo:list-item-label  end-indent="label-end()">
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          <xsl:value-of select="$basic-start-indent"/></xsl:when>
        <xsl:otherwise>inherit + 12pt</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block ><xsl:number format="1."/></fo:block>
  </fo:list-item-label>
  <fo:list-item-body>
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          <xsl:value-of select="$basic-start-indent"/> + 16pt</xsl:when>
        <xsl:otherwise>inherit + 24pt</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block>
      <xsl:if test="string(@id)"><fo:inline id="{@id}"> </fo:inline></xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
  </fo:list-item>
</xsl:template>


<xsl:template match="ul/*[contains(@spec,' topic.li ')]">
  <xsl:variable name="list-level" select="count(ancestor-or-self::ul | 
                                           ancestor-or-self::dl |
                                           ancestor-or-self::qalist |
                                           ancestor-or-self::ol )" />
  <fo:list-item space-before="1pt"> <!--.optimum="4pt", or deal with "compact=''" value -->
  <fo:list-item-label end-indent="label-end()">
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          <xsl:value-of select="$basic-start-indent"/>
        </xsl:when>
        <xsl:otherwise>
          inherit + 24pt
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block linefeed-treatment="ignore">
      <fo:inline font-family="Symbol">&bullet;</fo:inline>
    </fo:block>
  </fo:list-item-label>
  <fo:list-item-body>
    <xsl:attribute name="start-indent">
      <xsl:choose>
        <xsl:when test="$list-level=1">
          inherit + 12pt
        </xsl:when>
        <xsl:otherwise>
          inherit + 24pt
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <fo:block linefeed-treatment="ignore">
      <xsl:if test="string(@id)"><fo:inline id="{@id}"> </fo:inline></xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </fo:list-item-body>
  </fo:list-item>
</xsl:template>


<!-- TWO PART LISTS -->

<!-- common rule for paragraphs in all three DL-type structures -->
<xsl:template match="dd/p|aitem/p|gd/p">
 <fo:block xsl:use-attribute-sets="dd.p">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<!-- definition list -->

<xsl:template match="*[contains(@spec,' topic.dl ')]">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dlentry ')]">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dt ')]">          
 <fo:block xsl:use-attribute-sets="dt">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dd ')]">
 <fo:block xsl:use-attribute-sets="dd">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<xsl:template match="dlhead">
 <fo:block font-weight="bold" text-decoration="underline">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<!-- for HTML: if a definition list has headings, cast it as a table -->
<!-- for print: Not Applicable; all dls will be standard for print, for now -->

<xsl:template match="*[contains(@spec,' topic.dthd ')]">          
 <fo:block xsl:use-attribute-sets="dt" font-weight="bold">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.ddhd ')]">
 <fo:block xsl:use-attribute-sets="dd" font-weight="bold">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>


<!-- glossary list -->

<xsl:template match="*[contains(@spec,' topic.gl ')]">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.glentry ')]">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.gt ')]">          
 <fo:block xsl:use-attribute-sets="dt">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.gd ')]">
 <fo:block xsl:use-attribute-sets="dd">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<!-- the sort-range named template needs to be implemented here -->





<!-- QA list (FAQ end-use structure) -->

<xsl:template match="*[contains(@spec,' topic.qalist ')]">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.qaentry ')]">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.qitem ')]">          
 <fo:block xsl:use-attribute-sets="dt">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.aitem ')]">
 <fo:block xsl:use-attribute-sets="dd">
  <xsl:apply-templates />
 </fo:block>
</xsl:template>


<!-- phrase elements -->

<xsl:template match="*[contains(@spec,' topic.q ')]"> <!-- ideally, need to flipflop between " and ' -->
 <fo:inline xsl:use-attribute-sets="q">'<xsl:apply-templates />'</fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.cite ')]">
 <fo:inline font-style="italic"><xsl:apply-templates /></fo:inline>
</xsl:template>


<!-- phrase presentational styles -->

<!-- default behavior -->

<xsl:template match="*[contains(@spec,' topic.ph ')]">
<xsl:choose>
  <xsl:when test="@remap='underlined'">
    <fo:inline text-decoration="underline"><xsl:apply-templates /></fo:inline>
  </xsl:when>
  <xsl:when test="@remap='italic'">
    <fo:inline font-style="italic"><xsl:apply-templates /></fo:inline>
  </xsl:when>
  <xsl:when test="@remap='bold'">
    <fo:inline font-weight="bold"><xsl:apply-templates /></fo:inline>
  </xsl:when>
  <xsl:when test="@remap='bolditalic'">
    <fo:inline font-style="italic" font-weight="bold"><xsl:apply-templates /></fo:inline>
  </xsl:when>
  <xsl:when test="@remap='smallcaps'">
    <fo:inline text-transform="uppercase" font-size="smaller">
      <xsl:apply-templates/>
    </fo:inline>
  </xsl:when>
  <xsl:when test="@remap='monospace'">
    <fo:inline xsl:use-attribute-sets="tt"><xsl:apply-templates/></fo:inline>
  </xsl:when>
  <xsl:otherwise>
    <fo:inline color="purple"><xsl:apply-templates/></fo:inline>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>


<!-- phrase "semantic" classes -->

<xsl:template match="*[contains(@spec,' topic.xph ')]">
  <fo:inline xsl:use-attribute-sets="tt"><xsl:apply-templates/></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sup ')]">
  <fo:inline baseline-shift="super" font-size="75%">
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sub ')]">
  <fo:inline baseline-shift="sub" font-size="75%">
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.term ')]">                 
  <fo:inline>
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>


<!-- data typed phrases -->

<xsl:template match="*[contains(@spec,' topic.num ')]">
  <fo:inline color="maroon"><xsl:apply-templates/></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.char ')]">
  <fo:inline color="maroon">'<xsl:apply-templates/>'</fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.hex ')]">
  <fo:inline color="maroon">x'<xsl:apply-templates/></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dec ')]">
  <fo:inline color="maroon"><xsl:apply-templates/></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.apl ')]">
  <fo:inline color="maroon" font-face="symbol"><xsl:apply-templates/></fo:inline>
</xsl:template>

<!-- stateful data types -->

<xsl:template match="*[contains(@spec,' topic.boolean ')]">
  <fo:inline color="green"><xsl:value-of select="@state"/></fo:inline>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.state ')]">
  <fo:inline color="red"><xsl:value-of select="@value"/></fo:inline>
</xsl:template>



<!-- links -->

<xsl:template match="*[contains(@spec,' topic.zlink ')]">               
  <fo:basic-link external-destination="{@href}" color="blue">
      <xsl:apply-templates/>
  </fo:basic-link>
</xsl:template>


<!-- special handling of links
     1. If link has a @ref, is an internal book link and can ignore @href
        (@hrefs are preceded with document id, so are unique).
     2. If no @ref and @href ends with '.xml,' an internal link to beginning
        of one of "chapters."
     3. Otherwise an external link. For javadoc, design documents, and to-do list,
        link to xml.apache.org. -->
        
<xsl:template match="*[contains(@spec,' topic.link ')]">
  <xsl:choose>
    <xsl:when test="@ref">
      <fo:basic-link internal-destination="{@ref}" color="blue">
        <xsl:apply-templates/>
<!-- (<xsl:value-of select="@ref"/>)-->
      </fo:basic-link>
    </xsl:when>
    <xsl:otherwise>  
      <xsl:choose>
        <xsl:when test="starts-with(@href,'apidocs/') 
                     or starts-with(@href,'apidocs/') 
                     or starts-with(@href,'lotusxsl_apidocs/')
                     or starts-with(@href,'compat_apidocs/')
                     or starts-with(@href,'design/')
                     or starts-with(@href,'xsltc/')
                     or starts-with(@href,'todo.html')">
          <fo:basic-link external-destination="{concat('http://xml.apache.org/xalan-j/',@href)}" color="blue">
            <xsl:apply-templates/>
          </fo:basic-link>
 <!--(<xsl:value-of select="@href"/>) -->
        </xsl:when>
        <xsl:when test="starts-with(@href,'http://')
                     or starts-with(@href,'mailto:')">
          <fo:basic-link external-destination="{@href}" color="blue">
            <xsl:apply-templates/>
 <!--(<xsl:value-of select="@href"/>) -->
          </fo:basic-link>
        </xsl:when>
        <xsl:when test="contains(@href,'.xml')">
          <fo:basic-link internal-destination="{substring-before(@href,'.xml')}" color="blue">
            <xsl:apply-templates/>
 <!--(<xsl:value-of select="@href"/>) -->
          </fo:basic-link>                  
        </xsl:when>
        <xsl:otherwise>
          <fo:basic-link internal-destination="{@href}" color="blue">
            <xsl:apply-templates/>
 <!--(<xsl:value-of select="@href"/>) -->
          </fo:basic-link>
        </xsl:otherwise>  
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- set up keys based on xref's "type" attribute: %info-types;|hd|fig|table|li|fn -->
<xsl:key name="topic" match="topic" use="@id"/> <!-- uses "title" -->
<xsl:key name="fig"   match="fig"   use="@id"/> <!-- uses "title" -->
<xsl:key name="table" match="table" use="@id"/> <!-- uses "title" -->
<xsl:key name="li"    match="li"    use="@id"/> <!-- uses "?" -->
<xsl:key name="fn"    match="fn"    use="@id"/> <!-- uses "callout?" -->


<xsl:template match="*[contains(@spec,' topic.xrefx ')]">
 <!-- this needs to do an id/idref content fetch -->
   <xsl:choose>
      <xsl:when test="@href">
         <!-- nop -->
      </xsl:when>
      <xsl:when test="@refid">
         <!-- TBD: resolve a sequence number, or just get the text -->
<xsl:message>KEY SETUP: type=<xsl:value-of select="@type"/> refid=<xsl:value-of select="@refid"/></xsl:message>
         "<xsl:value-of select="key(@type,@refid)/title"/>" on page <fo:page-number-citation ref-id="{@refid}"/>
      </xsl:when>
      <xsl:otherwise>
            <xsl:value-of select="@*"/>
         <!--br/-->
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.xref ')]">
<!-- this version does not use keys -->
<!-- need to add task steps to supported types -->
<!--
<fo:inline color="red">
<xsl:choose>
  <xsl:when test="@type = 'fig'">
    <xsl:value-of select="$deftxt-fig"/>&rbl;
    <xsl:variable name="cnt"><xsl:number level="any" from="/" count="fig[@id]"/></xsl:variable>
    <xsl:value-of select="$cnt + 1"/>
  </xsl:when>
  <xsl:when test="@type = 'table'">
    <xsl:value-of select="$deftxt-tbl"/>&rbl;
    <xsl:number level="any" from="/" count="table[@id]"/>
  </xsl:when>
  <xsl:when test="@type = 'li'">
    Item&rbl;
    <xsl:number level="any" from="ul|ol"/>
  </xsl:when>
  <xsl:when test="@type = 'step'">
    Step&rbl;
    <xsl:number level="any" from="steps"/>
  </xsl:when>
  <xsl:otherwise>
    [<xsl:value-of select="@refid"/>]&rbl;
    <xsl:number level="any" from="/"/>
  </xsl:otherwise>
</xsl:choose>
</fo:inline>
-->
<!-- These support page and text snippets... 
"<xsl:value-of select="key(@type,@refid)/title"/>"
<xsl:text> on page </xsl:text>
<fo:page-number-citation ref-id="{@refid}"/>
-->

 <!-- this needs to do an id/idref content fetch -->
<fo:inline color="green">
   <xsl:variable name="xreftext">
     <xsl:choose>
       <xsl:when test="@othertype"      >Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='task'"    >Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='reftopic'">Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='concept'" >Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='fig'"     >Figure <xsl:value-of          select="id(@refid)/title"/></xsl:when>
       <xsl:when test="@type='table'"   >Table <xsl:value-of           select="id(@refid)/title"/></xsl:when>
       <xsl:when test="@type='li'"      >Item <xsl:apply-templates mode="xref" select="id(@refid)"/></xsl:when>
       <xsl:when test="@type='step'"    >Step <xsl:apply-templates mode="xref" select="id(@refid)"/></xsl:when>
       <xsl:when test="@type='fn'"      >Footnote <xsl:value-of select="id(@refid)"/></xsl:when>
       <xsl:otherwise>Here</xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="@href">
         [URL: <xsl:value-of select="@href"/>]
      </xsl:when>
      <xsl:when test="@refid">
         <xsl:value-of select="$xreftext"/>
      </xsl:when>
      <xsl:otherwise>
            <xsl:value-of select="@*"/>
      </xsl:otherwise>
   </xsl:choose>
</fo:inline>
</xsl:template>


<xsl:template match="step|li" mode="xref">
  <xsl:number/>
</xsl:template>


<!-- record end respecting data -->
 
<xsl:template match="*[contains(@spec,' topic.address ')]"> <!-- this is actually a specific data type -->
  <fo:block white-space="pre" white-space-collapse="false">
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>



<xsl:template match="*[contains(@spec,' topic.pre ')]">
<xsl:call-template name="gen-att-label"/>
  <fo:block xsl:use-attribute-sets="program">
<!--[<xsl:value-of select="name(parent::*)"/>]-->
<!--
    <xsl:if test="name(parent::*) = 'body'">
      <xsl:attribute name="start-indent">inherit</xsl:attribute>
    </xsl:if>
-->
    <xsl:choose>
      <xsl:when test="@scale">
        <xsl:attribute name="font-size"><xsl:value-of select="@scale*90"/>%</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:choose>
      <xsl:when test="name(..)='entry'">
      </xsl:when>
      <xsl:otherwise>
        <!--xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute-->
      </xsl:otherwise>
    </xsl:choose>
<!-- rules have to be applied within the scope of the PRE box; else they start from page margin! -->
    <xsl:if test="contains(@frame,'top')"><fo:block><fo:leader leader-pattern="rule" leader-length="5.65in" /></fo:block></xsl:if>
    <xsl:apply-templates/>
    <xsl:if test="contains(@frame,'bot')"><fo:block><fo:leader leader-pattern="rule" leader-length="5.65in" /></fo:block></xsl:if>
  </fo:block>
</xsl:template>
 
<xsl:template match="*[contains(@spec,' topic.lines ')]">
<xsl:call-template name="gen-att-label"/>
 <fo:block xsl:use-attribute-sets="lines">
    <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
    <xsl:apply-templates/>
<!--
  <xsl:call-template name="br-replace">
    <xsl:with-param name="word" select="."/>
  </xsl:call-template>
-->
 </fo:block>
</xsl:template>
 
<xsl:template match="*[contains(@spec,' topic.screen ')]">
<xsl:call-template name="gen-att-label"/>
 <fo:block xsl:use-attribute-sets="screen">
  <xsl:call-template name="br-replace">
    <xsl:with-param name="word" select="."/>
  </xsl:call-template>
 </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.cgraphic ')]">
<xsl:call-template name="gen-att-label"/>
  <xsl:choose>
    <xsl:when test="@remap='img'">
      <fo:block text-align="center">
        <fo:external-graphic src="{$img-path}{text()}"/>
      </fo:block>
    </xsl:when>
    <xsl:otherwise>
      <fo:block xsl:use-attribute-sets="program">
        <xsl:call-template name="br-replace">
         <xsl:with-param name="word" select="."/>
        </xsl:call-template>
       </fo:block>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- peril notices -->

<xsl:template match="*[contains(@spec,' topic.attention ')]">
  <fo:block space-after.optimum="6pt">
    <fo:wrapper font-weight="bold"><xsl:value-of select="$deftxt-attention"/> </fo:wrapper>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.caution ')]">
  <fo:block space-after.optimum="6pt">
    <fo:wrapper font-weight="bold"><xsl:value-of select="$deftxt-caution"/> </fo:wrapper>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

  <!-- special context of notices within list items -->
<xsl:template match="li/caution" priority="1">
    <fo:wrapper font-weight="bold"><xsl:value-of select="$deftxt-caution"/> </fo:wrapper>
    <xsl:apply-templates/>
</xsl:template>


<!-- note (mixed content with possibility of block within block) -->
<xsl:template match="*[contains(@spec,' topic.note ')]">
  <fo:block xsl:use-attribute-sets="p" space-after.optimum="6pt" margin-left="+2pc" start-indent="-2pc">
    <fo:wrapper font-weight="bold">
    <xsl:choose>
      <xsl:when test="@title">
        <xsl:if test="not(@title='')">
          <xsl:value-of select="@title"/>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$deftxt-nota"/>
      </xsl:otherwise>
    </xsl:choose><xsl:text>: </xsl:text>
    </fo:wrapper>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>




<!-- figure display context -->

<!-- TBD: implement frame="sides" -->
<xsl:template match="*[contains(@spec,' topic.fig ')]">
<fo:block xsl:use-attribute-sets="fig">
  <!-- attach original @id if present (for list of figures and xrefs) -->
  <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
  <!-- apply label within the block so that it floats with the fig itself -->
  <xsl:if test="$fmt-fig-lbl-loc='over'"><xsl:call-template name="place-fig-lbl"/></xsl:if>
  <!-- support "frame='all'"-->
  <!-- support "frame='top'"-->
  <xsl:if test="contains(@frame,'top')">
    <fo:block space-after="3pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="100%"/>
    </fo:block>
  </xsl:if>

  <fo:block><!-- created JUST for styling the exhibit area -->
  <xsl:if test="contains(@frame,'all')"><!-- adorn currently open block -->
    <xsl:attribute name="border-style">solid</xsl:attribute>
    <xsl:attribute name="border-color">black</xsl:attribute>
    <xsl:attribute name="border-width">thin</xsl:attribute>
    <!--xsl:attribute name="padding">6pt</xsl:attribute-->
  </xsl:if> 
  <!-- this is where the main fig rendering happens -->
  <xsl:apply-templates/>
  </fo:block><!-- created JUST for styling the exhibit area -->

  <!-- support "frame='bot'"-->
  <xsl:if test="contains(@frame,'bot')">
    <fo:block space-after="3pt">
      <fo:leader color="black" leader-pattern="rule" rule-thickness=".5pt" leader-length="100%"/>
    </fo:block>
  </xsl:if>
  <!-- apply label within the block so that it floats with the fig itself -->
  <xsl:if test="$fmt-fig-lbl-loc='under'"><xsl:call-template name="place-fig-lbl"/></xsl:if>
</fo:block>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.figbody ')]">
<fo:block><!-- content-width="scale-to-fit" scaling="auto"-->
  <xsl:apply-templates/>
</fo:block>
</xsl:template>




<!-- CALS version of table -->
<!-- adapted from sample code in the XSL FO spec; extended considerably! -->

<xsl:template match="table/title" priority="2"/>

<xsl:template match="*[contains(@spec,' topic.table ')]">
  <xsl:choose>
    <xsl:when test="xtitle"><!-- renamed for now to NOT trigger on title; testing the "otherwise" code -->
      <fo:table-and-caption xsl:use-attribute-sets="table.data frameall">
        <fo:table-caption>
          <fo:block xsl:use-attribute-sets="table.data.caption">
            <xsl:value-of select="title"/>
          </fo:block>
        </fo:table-caption>
        <fo:table xsl:use-attribute-sets="table.data frameall">
          <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
          <xsl:apply-templates/>
        </fo:table>
      </fo:table-and-caption>
    </xsl:when>
    <xsl:otherwise>
      <fo:table xsl:use-attribute-sets="table.data frameall">
        <xsl:if test="string(@id)"><xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute></xsl:if>
        <xsl:call-template name="default-colwidth"/>
        <xsl:apply-templates/>
      </fo:table>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="default-colwidth">
    <!--fo:table-column column-width="150pt"/-->
    <xsl:for-each select="row[1]/entry">
      <fo:table-column>
        <!-- compute even columns (creates valid processing, but all columns are even width) -->
        <xsl:attribute name="column-width"><xsl:value-of select="floor(@width div 72)"/>in</xsl:attribute>
        <!-- TBD: get this value directly from markup, if possible, for adjusted overrides -->
      </fo:table-column>
    </xsl:for-each>
</xsl:template>

<xsl:template match="spanspec">
<!--  <xsl:call-template name="att-align"/> -->
</xsl:template>

<xsl:template match="colspec">
  <fo:table-column>
    <xsl:call-template name="att-align"/>
    <xsl:attribute name="column-number">
      <xsl:number count="colspec"/>
    </xsl:attribute>
    <xsl:attribute name="column-width">
      <xsl:call-template name="xcalc.column.width">
        <xsl:with-param name="colwidth">
          <xsl:value-of select="@colwidth"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:attribute>
  </fo:table-column>
</xsl:template>

<xsl:template match="tgroup">
<!--
  <xsl:call-template name="att-align"/>
-->
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="thead">
  <fo:table-header>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates />
  </fo:table-header>
</xsl:template>

<xsl:template match="tfoot">
  <fo:table-footer>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates/>
  </fo:table-footer>
</xsl:template>

<xsl:template match="tbody">
  <fo:table-body>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates/>
  </fo:table-body>
</xsl:template>

<xsl:template match="tnote">
  <fo:table-row>
  <fo:table-cell xsl:use-attribute-sets="table.data.td">
    <fo:block><fo:inline font-weight="bold"><xsl:value-of select="$deftxt-nota"/><xsl:text>: </xsl:text></fo:inline><xsl:apply-templates/></fo:block>
  </fo:table-cell>
  </fo:table-row>
</xsl:template>

<xsl:template match="row">
  <fo:table-row>
    <xsl:call-template name="att-valign"/>
    <xsl:apply-templates/>
  </fo:table-row>
</xsl:template>

<xsl:template match="thead/row/entry">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
  <fo:table-cell column-number="{$colnumval}" xsl:use-attribute-sets="table.data.th">
    <xsl:call-template name="entryatts"/>
    <fo:block>
      <xsl:call-template name="fillit"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>

<xsl:template match="tfoot/row/entry">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
  <fo:table-cell column-number="{$colnumval}" xsl:use-attribute-sets="table.data.tf frameall">
    <xsl:call-template name="entryatts"/>
    <fo:block>
      <xsl:call-template name="fillit"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>


<xsl:template name="get-colnumval">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
</xsl:template>

<xsl:template match="tbody/row/entry">
<xsl:variable name="colnumval">
  <xsl:choose>
    <xsl:when test="@colname">
      <xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/>
    </xsl:when>
    <xsl:when test="@colnum">
      <xsl:value-of select="@colnum"/>
    </xsl:when>
    <xsl:otherwise><xsl:number count="entry"/></xsl:otherwise>
  </xsl:choose>
</xsl:variable>
  <fo:table-cell column-number="{$colnumval}" xsl:use-attribute-sets="table.data.td frameall">
    <xsl:call-template name="entryatts"/>
    <fo:block>
      <xsl:call-template name="fillit"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>


<!-- named templates -->
<xsl:template name="fillit">
<xsl:choose>
  <!-- test to see if the cell contains any sensible text or other element... -->
  <xsl:when test="not(text()[normalize-space(.)] | *)"><fo:inline>&rbl;'</fo:inline></xsl:when>
  <xsl:otherwise><xsl:apply-templates/></xsl:otherwise>
</xsl:choose>
</xsl:template>


<!-- table "macros" -->

<xsl:template name="entryatts">
  <xsl:call-template name="att-valign"/>
   <xsl:call-template name="att-align"/>

  <xsl:if test="string(@colsep)">
    <xsl:if test="@colsep='1'"><xsl:attribute name="cellpadding">10</xsl:attribute></xsl:if>
  </xsl:if>

<!-- IPL start -->
  <xsl:if test="string(@namest)">
    <xsl:variable name="colst" select="substring-after(@namest,'col')"/>
    <xsl:variable name="colend" select="substring-after(@nameend,'col')"/>
    <xsl:attribute name="number-columns-spanned"><xsl:value-of select="$colend - $colst + 1"/></xsl:attribute>
  </xsl:if>
<!-- IPL end -->
</xsl:template>


<xsl:template name="att-valign">
  <xsl:if test="string(@valign)">
    <xsl:choose>
      <xsl:when test="@valign='middle'">
        <xsl:attribute name="display-align">center</xsl:attribute>
      </xsl:when>
      <xsl:when test="@valign='top'">
        <xsl:attribute name="display-align">before</xsl:attribute>
      </xsl:when>
      <xsl:when test="@valign='bottom'">
        <xsl:attribute name="display-align">after</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="display-align">
          <xsl:value-of select="@valign"/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</xsl:template>


<xsl:template name="att-align">
  <xsl:if test="string(@align)">
    <xsl:attribute name="text-align">
      <xsl:value-of select="normalize-space(@align)"/>
    </xsl:attribute>
  </xsl:if>
</xsl:template>


<!-- table formatting properties -->

<xsl:attribute-set name="table.data" ><!-- use for overall table default characteristics -->
  <!--xsl:attribute name="table-layout">fixed</xsl:attribute-->
  <!--xsl:attribute name="inline-progression-dimension">auto</xsl:attribute-->
  <xsl:attribute name="space-before">10pt</xsl:attribute>
  <xsl:attribute name="space-after">10pt</xsl:attribute>
  <xsl:attribute name="background-color">white</xsl:attribute>
  <!--xsl:attribute name="start-indent">inherit</xsl:attribute-->
</xsl:attribute-set>

<xsl:attribute-set name="table.data.caption" >
  <xsl:attribute name="start-indent">inherit</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.tbody" >
  <xsl:attribute name="background-color">white</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.th" >
  <xsl:attribute name="color">black</xsl:attribute>
  <xsl:attribute name="background-color">silver</xsl:attribute>
  <xsl:attribute name="padding">2pt</xsl:attribute>
  <xsl:attribute name="text-align">center</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.tf" >
  <xsl:attribute name="color">blue</xsl:attribute>
  <xsl:attribute name="padding">2pt</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="table.data.td" >
  <xsl:attribute name="background-color">grey</xsl:attribute>
  <xsl:attribute name="padding">2pt</xsl:attribute>
</xsl:attribute-set>


<!-- separate the frame attributes into here; include them conditionally -->
<xsl:attribute-set name="frameall">
  <xsl:attribute name="border-style">solid</xsl:attribute>
  <xsl:attribute name="border-width">1pt</xsl:attribute>
  <xsl:attribute name="border-color">black</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="frametop">
  <xsl:attribute name="border-top-style">solid</xsl:attribute>
  <xsl:attribute name="border-top-width">1pt</xsl:attribute>
  <xsl:attribute name="border-top-color">black</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="framebottom">
  <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
  <xsl:attribute name="border-bottom-width">1pt</xsl:attribute>
  <xsl:attribute name="border-bottom-color">black</xsl:attribute>
</xsl:attribute-set>


<!-- table support based on examples in the XSL spec -->
  <!-- see original support comments in the XSL spec, source of this fragment -->
<xsl:template name="calc.column.width">
  <xsl:param name="colwidth">1*</xsl:param>
  <!-- Ok, the colwidth could have any one of the following forms: -->
  <!--        1*       = proportional width -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->
  <!--  1*+1unit       = proportional width + some fixed width -->
  <!--      1*+1       = proportional width + some fixed width -->

  <!-- If it has a proportional width, translate it to XSL -->
  <xsl:if test="contains($colwidth, '*')"><!-- modified to handle "*" as input -->
    <xsl:variable name="colfactor">
      <xsl:value-of select="substring-before($colwidth, '*')"/>
    </xsl:variable>
    <xsl:text>proportional-column-width(</xsl:text>
      <xsl:choose>
        <xsl:when test="not($colfactor = '')">
         <xsl:value-of select="$colfactor"/>
        </xsl:when>
        <xsl:otherwise>1</xsl:otherwise>
      </xsl:choose>
    <xsl:text>)</xsl:text>
  </xsl:if>


  <!-- Now get the non-proportional part of the specification -->
  <xsl:variable name="width-units">
    <xsl:choose>
      <xsl:when test="contains($colwidth, '*')">
        <xsl:value-of
             select="normalize-space(substring-after($colwidth, '*'))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="normalize-space($colwidth)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Now the width-units could have any one of the following forms: -->
  <!--                 = <empty string> -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->
  <!-- with an optional leading sign -->

  <!-- Get the width part by blanking out the units part and discarding -->
  <!-- white space. -->
  <xsl:variable name="width"
       select="normalize-space(translate($width-units,
                          '+-0123456789.abcdefghijklmnopqrstuvwxyz',
                          '+-0123456789.'))"/>

  <!-- Get the units part by blanking out the width part and discarding -->
  <!-- white space. -->
  <xsl:variable name="units"
       select="normalize-space(translate($width-units,
                          'abcdefghijklmnopqrstuvwxyz+-0123456789.',
                          'abcdefghijklmnopqrstuvwxyz'))"/>

  <!-- Output the width -->
  <xsl:value-of select="$width"/>

  <!-- Output the units, translated appropriately -->
  <xsl:choose>
    <xsl:when test="$units = 'pi'">pc</xsl:when>
    <xsl:when test="$units = '' and $width != ''">pt</xsl:when>
    <xsl:otherwise><xsl:value-of select="$units"/></xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="xcalc.column.width">
  <!-- see original support comments in the XSL spec, source of this fragment -->
  <xsl:param name="colwidth">1*</xsl:param>

  <!-- Ok, the colwidth could have any one of the following forms: -->
  <!--        1*       = proportional width -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->
  <!--  1*+1unit       = proportional width + some fixed width -->
  <!--      1*+1       = proportional width + some fixed width -->

  <!-- If it has a proportional width, translate it to XSL -->
  <xsl:if test="contains($colwidth, '*')">
    <xsl:variable name="colfactor">
      <xsl:value-of select="substring-before($colwidth, '*')"/>
    </xsl:variable>
    <xsl:text>proportional-column-width(</xsl:text>
      <xsl:choose>
        <xsl:when test="not($colfactor = '')">
         <xsl:value-of select="$colfactor"/>
        </xsl:when>
        <xsl:otherwise>1</xsl:otherwise>
      </xsl:choose>
    <xsl:text>)</xsl:text>
  </xsl:if>

  <!-- Now get the non-proportional part of the specification -->
  <xsl:variable name="width-units">
    <xsl:choose>
      <xsl:when test="contains($colwidth, '*')">
        <xsl:value-of
             select="normalize-space(substring-after($colwidth, '*'))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="normalize-space($colwidth)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Now the width-units could have any one of the following forms: -->
  <!--                 = <empty string> -->
  <!--     1unit       = 1.0 units wide -->
  <!--         1       = 1pt wide -->
  <!-- with an optional leading sign -->

  <!-- Get the width part by blanking out the units part and discarding -->
  <!-- whitespace. -->
  <xsl:variable name="width"
       select="normalize-space(translate($width-units,
                                         '+-0123456789.abcdefghijklmnopqrstuvwxyz',
                                         '+-0123456789.'))"/>

  <!-- Get the units part by blanking out the width part and discarding -->
  <!-- whitespace. -->
  <xsl:variable name="units"
       select="normalize-space(translate($width-units,
                                         'abcdefghijklmnopqrstuvwxyz+-0123456789.',
                                         'abcdefghijklmnopqrstuvwxyz'))"/>

  <!-- Output the width -->
  <xsl:value-of select="$width"/>

  <!-- Output the units, translated appropriately -->
  <xsl:choose>
    <xsl:when test="$units = 'pi'">pc</xsl:when>
    <xsl:when test="$units = '' and $width != ''">pt</xsl:when>
    <xsl:otherwise><xsl:value-of select="$units"/></xsl:otherwise>
  </xsl:choose>
</xsl:template>





<!-- original: special handling of galley like content within table cells; simplify! -->

<xsl:template match="entry/*[contains(@spec,' topic.p ')]">
 <fo:block><xsl:apply-templates /></fo:block>
</xsl:template>

<xsl:template match="entry/*[contains(@spec,' topic.note ')]">
  <fo:block>
    <fo:wrapper font-weight="bold">
    <xsl:choose>
      <xsl:when test="@title">
        <xsl:if test="not(@title='')">
          <xsl:value-of select="@title"/>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$deftxt-nota"/>
      </xsl:otherwise>
    </xsl:choose><xsl:text>: </xsl:text>
    </fo:wrapper>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>


<!-- end of table section -->




<!-- "FORMAT" GLOBAL DECLARATIONS -->
<xsl:variable name="fmt-fig-lbl-loc">under</xsl:variable><!-- values: over, under -->
<xsl:variable name="fmt-tbl-lbl-loc">over</xsl:variable><!-- values: over, under -->
<xsl:variable name="link-top-section">no</xsl:variable><!-- values: yes, no (or any not "yes") -->
<xsl:variable name="do-place-ing">no</xsl:variable><!-- values: yes, no (or any not "yes") -->

<xsl:variable name="fig-pfx-tx1t"></xsl:variable>
<xsl:variable name="fig-pfx-txt2">
 <xsl:value-of select="$deftxt-fig"/> <xsl:number level="any"/>.
 <!--of <xsl:value-of select="count(//fig/title)"/>.--> 
</xsl:variable><!-- values: '' or custom design -->


<!-- "FORMAT" MACROS -->
<!-- 
 | These macros support globally-defined formatting constants for
 | document content.  Some elements have attributes that permit local
 | control of formatting; such logic is part of the pertinent template rule.
 +-->

<xsl:template name="place-tbl-width">
<xsl:variable name="twidth-fixed"><xsl:attribute name="width">540</xsl:attribute></xsl:variable>
  <xsl:if test="$twidth-fixed != ''">
    <xsl:value-of select="$twidth-fixed"/>
  </xsl:if>
</xsl:template>

<xsl:template name="place-tbl-lbl">
  <xsl:if test="*[contains(@spec,' topic.title ')]">
    <fo:block>
      <fo:inline>
        <xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute>
        <xsl:value-of select="*[contains(@spec,' topic.title ')]"/> <!-- this specialization is not right for tbl context-->
      </fo:inline>
    </fo:block>
  </xsl:if>
</xsl:template>


<xsl:template name="place-fig-lbl">
  <xsl:if test="*[contains(@spec,' topic.title ')]">
    <fo:block xsl:use-attribute-sets="fig">
      <fo:inline font-weight="bold">
        <xsl:value-of select="$deftxt-fig"/>&nbsp;
        <xsl:number level="any" />.
<!-- of <xsl:value-of select="count(//fig/title)"/>. --> <!-- TBD: convention for numbering figures-->
        <xsl:value-of select="*[contains(@spec,' topic.title ')]"/>
      </fo:inline>
    <xsl:if test="shortdesc">
      <xsl:text>. </xsl:text><xsl:value-of select="*[contains(@spec,' topic.shortdesc ')]"/>
    </xsl:if>
    </fo:block>
  </xsl:if>
</xsl:template>



<!-- other named templates -->

<xsl:template name="gen-ptoc">
<xsl:if test="/*/topic">
<xsl:comment> Begin Table of Contents </xsl:comment>
<!-- Begin Table of Contents -->
  <fo:block>
    <fo:inline line-height="5" background="#0033CC" text-align="center" font-weight="bold" color="#FFFFFF">Contents:</fo:inline>

    <xsl:for-each select="/*/*/*[contains(@spec,' topic.title ')]">
     <fo:block> &nbsp;
     <!-- link href="#{generate-id()}"-->
     <xsl:value-of select="."/>
     <!--recursive call for subtopics here"/-->
     </fo:block>
    </xsl:for-each>
  </fo:block>
<xsl:comment> End Table of Contents </xsl:comment>
</xsl:if>
</xsl:template>



<!-- sample embedded SVG from Elliotte Rusty Harold's XML Bible, Chapter 15 -->
<!-- Once this capability becomes more common in FO processors, we can use it for
     character graphics and syntax diagrams. -->

<!--xsl:call-template name="try-svg"/--> <!-- insert this somewhere to call it for rendering -->

<xsl:template name="try-svg">
  <fo:instream-foreign-object>
    <svg xmlns="http://www.w3.org/2000/svg"
         width="1.5cm" height="1cm">
      <polygon style="fill:#FFCCCC" points="0,31 18,0 36,31" />
    </svg>
  </fo:instream-foreign-object>
</xsl:template>




<!-- semantic tables -->

<xsl:template match="*[contains(@spec,' topic.semtable ')]">
  <fo:table xsl:use-attribute-sets="table.data frameall">
    <xsl:call-template name="semtbl-colwidth"/>
    <fo:table-body>
      <xsl:apply-templates/>
    </fo:table-body>
  </fo:table>
</xsl:template>

<xsl:template name="semtbl-colwidth">
    <!--fo:table-column column-width="150pt"/-->
    <xsl:variable name="numcols" select="count(semrow[1]/*)"/>
    <xsl:for-each select="semrow[1]/*">
      <fo:table-column>
        <!-- compute even columns (creates valid processing, but all columns are even width) -->
        <xsl:attribute name="column-width"><xsl:value-of select="floor($numcols * 132 div 72)"/>in</xsl:attribute>
        <!-- TBD: get this value directly from markup, if possible, for adjusted overrides -->
      </fo:table-column>
    </xsl:for-each>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.semhdr ')]">
  <fo:table-row>
    <xsl:apply-templates/>
  </fo:table-row>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.semrow ')]">
  <fo:table-row>
    <xsl:apply-templates/>
  </fo:table-row>
</xsl:template>

<!-- in the following two rules, the xsl:number needs to give the
  sequence number of either the current semdh or sementry in the current
  semrow OR semhdr. -->

<xsl:template match="*[contains(@spec,' topic.semhd ')]">
  <fo:table-cell background-color="silver" padding="2pt" text-align=
"center" font-weight="bold" xsl:use-attribute-sets="frameall">
    <xsl:attribute name="column-number"><xsl:number count="*"/></xsl:attribute>
    <fo:block>
    <xsl:call-template name="get-title"/>
    </fo:block>
  </fo:table-cell>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sementry ')]">
  <fo:table-cell background-color="white" padding="2pt" xsl:use-attribute-sets="frameall">
    <xsl:attribute name="column-number"><xsl:number count="*"/></xsl:attribute>
    <fo:block>
    <xsl:apply-templates/>
    </fo:block>
  </fo:table-cell>
</xsl:template>







<!-- ===================== FORMATTER DECLARATIONS AND GLOBALS ================ -->


<!-- named templates with page models -->



<!-- set up CSS-like attribute sets -->
<!-- tempted to use the shorthand form? if so, beware of "silent overrides" -->

<xsl:attribute-set name="topic" >
  <xsl:attribute name="font-family">Helvetica</xsl:attribute>
  <xsl:attribute name="font-size">10pt</xsl:attribute>
</xsl:attribute-set>

<!-- add line-height for multi-line headings? -->

<xsl:attribute-set name="h0.title" >
  <xsl:attribute name="break-before">page</xsl:attribute>
  <xsl:attribute name="margin-top">0pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">1.4pc</xsl:attribute>
  <xsl:attribute name="font-size">16pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle1" >
  <xsl:attribute name="break-before">page</xsl:attribute>
  <xsl:attribute name="margin-top">0pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">1.4pc</xsl:attribute>
  <xsl:attribute name="font-size">16pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle2" >
  <xsl:attribute name="margin-top">1pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">5pt</xsl:attribute>
  <xsl:attribute name="font-size">14pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle3" >
  <xsl:attribute name="margin-top">1pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">2pt</xsl:attribute>
  <xsl:attribute name="font-size">14pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle4" >
  <xsl:attribute name="font-size">9pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle5" >
  <xsl:attribute name="font-size">11pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="topictitle6" >
  <xsl:attribute name="font-size">11pt</xsl:attribute>
  <xsl:attribute name="font-style">italic</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="section.title" >
  <xsl:attribute name="space-before">1em</xsl:attribute>
  <xsl:attribute name="color">black</xsl:attribute>
  <xsl:attribute name="font-size">11pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="unused.section.title">
  <xsl:attribute name="space-before">1em</xsl:attribute>
  <xsl:attribute name="space-before.precedence">1</xsl:attribute>
  <xsl:attribute name="space-after">1em</xsl:attribute>
  <xsl:attribute name="keep-with-next.within-column">always</xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>


<!-- p-->
<xsl:attribute-set name="p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <!--xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute-->
  <!--xsl:attribute name="text-align">justify</xsl:attribute-->
</xsl:attribute-set>

<!-- p-->
<xsl:attribute-set name="top.p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <!--xsl:attribute name="text-align">justify</xsl:attribute-->
</xsl:attribute-set>

<!-- p-->
<xsl:attribute-set name="divlike.p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <!--xsl:attribute name="text-align">justify</xsl:attribute-->
</xsl:attribute-set>

<xsl:attribute-set name="lq" >
  <!--xsl:attribute name="background-color">antiquewhite</xsl:attribute-->
  <!--xsl:attribute name="font-style">italic</xsl:attribute-->
  <xsl:attribute name="space-before">10pt</xsl:attribute>
  <xsl:attribute name="space-after">10pt</xsl:attribute>
  <xsl:attribute name="padding-left">2pt</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
  <xsl:attribute name="text-align">left</xsl:attribute>
  <xsl:attribute name="border-style">solid</xsl:attribute>
  <xsl:attribute name="border-color">black</xsl:attribute>
  <xsl:attribute name="border-width">thin</xsl:attribute>
</xsl:attribute-set>


<!-- footnote-->
<xsl:attribute-set name="footnote">
  <xsl:attribute name="font-family">Helvetica</xsl:attribute>
  <xsl:attribute name="font-size">8pt</xsl:attribute>
  <xsl:attribute name="line-height">10pt</xsl:attribute>
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="start-indent">0in</xsl:attribute>
  <!--xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute-->
  <xsl:attribute name="text-align">justify</xsl:attribute>
</xsl:attribute-set>


<!-- note -->
<xsl:attribute-set name="note" >
  <xsl:attribute name="background-color">antiquewhite</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>


<!-- ul-->
<xsl:attribute-set name="ul">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="dd.p">
  <xsl:attribute name="text-indent">0em</xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="dt" >
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="dd" >
  <xsl:attribute name="space-before">0.3em</xsl:attribute>
  <xsl:attribute name="space-after">0.5em</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/> + 4em</xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="program">
  <!--xsl:attribute name="start-indent">inherit</xsl:attribute-->
  <xsl:attribute name="padding-left">2pt</xsl:attribute>
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="white-space">pre</xsl:attribute>
  <xsl:attribute name="white-space-collapse">false</xsl:attribute>
  <xsl:attribute name="wrap-option">wrap</xsl:attribute>
  <xsl:attribute name="background-color">#e0e0e0</xsl:attribute>
  <xsl:attribute name="font-family">Courier</xsl:attribute>
  <xsl:attribute name="line-height">106%</xsl:attribute>
  <xsl:attribute name="font-size">9pt</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="screen">
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="white-space">pre</xsl:attribute>
  <xsl:attribute name="white-space-collapse">false</xsl:attribute>
  <xsl:attribute name="font-family">Courier</xsl:attribute>
  <xsl:attribute name="font-size">8pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="border-style">solid</xsl:attribute>
  <xsl:attribute name="border-color">black</xsl:attribute>
  <xsl:attribute name="border-width">thin</xsl:attribute>
  <xsl:attribute name="color">green</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="lines">
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="white-space">pre</xsl:attribute>
  <xsl:attribute name="white-space-collapse">false</xsl:attribute>
  <xsl:attribute name="wrap-option">wrap</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="fig">
  <xsl:attribute name="space-before">0.6em</xsl:attribute>
  <xsl:attribute name="space-after">0.6em</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="figure.title" > <!-- unused-->
  <xsl:attribute name="text-align">center</xsl:attribute>
  <xsl:attribute name="space-before">3pt</xsl:attribute>
  <xsl:attribute name="start-indent"><xsl:value-of select="$basic-start-indent"/></xsl:attribute>
  <xsl:attribute name="end-indent"><xsl:value-of select="$basic-end-indent"/></xsl:attribute>
</xsl:attribute-set>


<!-- inline-->
<xsl:attribute-set name="strong" >
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="em" >
  <xsl:attribute name="font-style">italic</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="q" >
  <xsl:attribute name="font-style">italic</xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="dfn" > <!-- unused -->
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="font-style">normal</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="code" >
  <xsl:attribute name="font-family">Helvetica</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="tt" >
  <xsl:attribute name="font-family">Courier</xsl:attribute>
</xsl:attribute-set>



</xsl:stylesheet>
