<?xml version="1.0" encoding="UTF-8" ?>

<!--  bkbook8x11.xsl 1.0; 25 June 2001
 | Book "delivery context" for DITA topics in (based on bkbook.dtd)
 |
 | Copyright IBM Corporation, 2001
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +======================================
 | 
 *-->

<!DOCTYPE xsl:stylesheet [
<!-- entities for use in the generated output (Unicode typographic glyphs) -->
  <!ENTITY gt            "&gt;"> 
  <!ENTITY lt            "&lt;"> 
  <!ENTITY rbl           "&#160;">
  <!ENTITY nbsp          "&#160;">
  <!ENTITY quot          "&#34;">
  <!ENTITY quotedblleft  "&#x201C;">
  <!ENTITY quotedblright "&#x201D;">
  <!ENTITY sqbull        "[]">
  <!ENTITY middot        "&#x00A5;">

  <!ENTITY section       "&#xA7;">
  <!ENTITY endash        "&#x2013;">
  <!ENTITY emdash        "&#x2014;">

  <!ENTITY copyr         "&#xA9;">
  <!ENTITY trademark     "&#x2122;">
  <!ENTITY registered    "&#xAE;">
<!-- create some fixed values for now for customizing later -->
  <!ENTITY copyrowner    "Sample Company">
  <!ENTITY copyrdate     "2001">
  <!ENTITY headerstub    "(stub for header content)">

]>


<xsl:stylesheet version="1.0"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">


<!-- ====================== Processing Prolog ==================== 
 | Copy these parts into any overriding stylesheet for customized settings, etc..
 *-->
<xsl:import href="dit2fo.xsl"/>

<!-- include other document vocabulary support here.  Use xsl:include if their vocab is unique -->
<!--xsl:include href="elementrefprint.xsl"/-->

<xsl:output method="xml"
 version="1.0" indent="yes"/>

<!-- SETTINGS: -->
<xsl:variable name="trace">no</xsl:variable> <!--set string to 'yes' to turn on trace -->

<!-- set up common attributes for all page definitions -->
<xsl:attribute-set name="common-grid">
  <xsl:attribute name="page-width">51pc</xsl:attribute> <!-- A4: 210mm -->
  <xsl:attribute name="page-height">66pc</xsl:attribute> <!-- A4: 297mm -->
  <xsl:attribute name="margin-top">3pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">3pc</xsl:attribute>
  <xsl:attribute name="margin-left">6pc</xsl:attribute>
  <xsl:attribute name="margin-right">6pc</xsl:attribute>
</xsl:attribute-set>

<!-- CONTROL PARAMETERS: -->
<!-- offset -->
<xsl:param name="basic-start-indent">72pt</xsl:param>
<xsl:param name="basic-end-indent">24pt</xsl:param>


<!-- GLOBALS: -->
<xsl:variable name="dflt-ext">.jpg</xsl:variable> <!-- For Antenna House, set to ".jpg" -->




<!-- ====================== template rules for "bkbook" dtd content ==================== -->

<xsl:template match="/"> <!-- no special root level procs at this time -->
  <xsl:comment>Sample DITA to XSL-FO application by IBM Corp., 2001</xsl:comment>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkbook">
<!-- this is an "setup" context for book context documents -->
  <xsl:call-template name="book-setup"/>
</xsl:template>

<xsl:template match="bknotices|bkspecialnotices|bkednotices"/>



<!-- this content is not usually directly printed, but used in publishing/distribution services -->
<xsl:template match="*[contains(@spec,' bkbook.bkabstract ')]" mode="review">
  <fo:block xsl:use-attribute-sets="p" space-after.optimum="6pt" margin-left="+2pc" start-indent="-2pc">
    <fo:wrapper font-weight="bold">
      <xsl:value-of select="$deftxt-abstract"/><xsl:text>: </xsl:text>
    </fo:wrapper>
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>


<xsl:template match="bkfrontm">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkbody">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkappendixes">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkbackm">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkchapter|bkappendix|bkglossary">
  <fo:inline id="{generate-id()}"> </fo:inline>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkchapter/title">
    <fo:block color="magenta"><xsl:apply-templates/></fo:block>
</xsl:template>

<xsl:template match="bksection|bksubsection">
  <fo:inline id="{generate-id()}"> </fo:inline>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksubsect1|bksubsect2|bksubsect3|bksubsect4">
  <fo:inline id="{generate-id()}"> </fo:inline>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkpre|bkpost">
  <fo:inline id="{generate-id()}"> </fo:inline>
  <xsl:apply-templates/>
</xsl:template>

<!-- trap unwanted metadata -->
<xsl:template match="bkprolog"/>


<!-- level specific overrides (process included topic as if child topic, not as standalone)-->

<!-- h1 -->
<xsl:template name="bk-topic-intercept" match="bkchapter/*[contains(@spec,' topic.topic ')] | bkappendix/*[contains(@spec,' topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bk-topic-intercept' }</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkchapter/*/*[contains(@spec,' topic.title ')]" priority="5">
<xsl:if test="$trace='yes'">
    <fo:block color="magenta">{arrived at bk h1 heading context FOR CHAPTER-CONTEXT TOPICS!}</fo:block></xsl:if>
  <!-- TBD: prepare and pass the context-appropriate prefix text here: Chapter, Appendix, nothing, etc. -->
  <xsl:call-template name="h1heading"/>
</xsl:template>


<xsl:template match="bkappendix/*/*[contains(@spec,' topic.title ')]" priority="5">
<xsl:if test="$trace='yes'">
    <fo:block color="magenta">{arrived at bk h1 heading context FOR CHAPTER-CONTEXT TOPICS!}</fo:block></xsl:if>
  <!-- TBD: prepare and pass the context-appropriate prefix text here: Chapter, Appendix, nothing, etc. -->
  <xsl:call-template name="h1heading"/>
</xsl:template>


<!-- h2 -->
<xsl:template name="bkh2div" match="bksection/*[contains(@spec, 'topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh2div' }</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksection/*/*[contains(@spec,' topic.title ')]">
  <xsl:call-template name="h2heading"/>
</xsl:template>

<!-- h3 -->
<xsl:template name="h3div" match="bksubsection/*[contains(@spec, 'topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh3div' }</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksubsection/*/*[contains(@spec,' topic.title ')]">
  <xsl:call-template name="h3heading"/>
</xsl:template>

<!-- h4 -->
<xsl:template name="h4div" match="bksubsect1/*[contains(@spec, 'topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh4div' }</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksubsect1/*/*[contains(@spec,' topic.title ')]">
  <xsl:call-template  name="h4heading"/>
</xsl:template>

<!-- h5 -->
<xsl:template name="h5div" match="bksubsect2/*[contains(@spec,' topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh5div' }</fo:block></xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksubsect2/*/*[contains(@spec, 'topic.title ')]">
  <xsl:call-template  name="h5heading"/>
</xsl:template>

<!-- h6 -->
<xsl:template name="h6div" match="bksubsect3/*[contains(@spec,' topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh6div' }</fo:block></xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksubsect3/*/*[contains(@spec, 'topic.title ')]">
  <xsl:call-template  name="h6heading"/>
</xsl:template>

<!-- h7 -->
<xsl:template name="h7div" match="bksubsect4/*/*[contains(@spec,' topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh7div' }</fo:block></xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksubsect4/*/*[contains(@spec, 'topic.title ')]">
  <xsl:call-template  name="h7heading"/>
</xsl:template>



<!-- additional traps for directly nested topics; maybe we can undo this during merge and 
     eliminate the need for these traps. -->

<!-- h3 (where the problem first occurs) -->
<xsl:template match="bksection/*/*[contains(@spec, 'topic.topic ')]">
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh3div'--directlly imbedded! }</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksection/*/*/*[contains(@spec,' topic.title ')]">
  <xsl:call-template name="h3heading"/>
</xsl:template>

<!-- h4 -->
<xsl:template match="bksubsection/*/*[contains(@spec, 'topic.topic ')]"> <!-- | bksection/*[contains(@spec, 'topic.topic ')]/*[contains(@spec, 'topic.topic ')]-->
<xsl:if test="$trace='yes'"><fo:block color="magenta">{arrived at 'bkh3div'--directlly imbedded! }</fo:block></xsl:if>
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bksubsection/*/*/*[contains(@spec,' topic.title ')]"> <!-- | bksection/*[contains(@spec, 'topic.topic ')]/*[contains(@spec, 'topic.topic ')]/*[contains(@spec, 'topic.topic ')]-->
  <xsl:call-template name="h3heading"/>
</xsl:template>

<!-- h5 -->
<!-- the permutations go nuts at this point.  Caveat emptor! -->


<!-- =========================== overall book organization ========================= -->

<!-- this template rule defines the overall book organization -->
<xsl:template name="book-setup">
<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
  <!-- get the overall master page defs here -->
  <xsl:call-template name="define-page-masters"/>
  <!-- place generated content -->
  <xsl:call-template name="front-covers"/>
  <xsl:call-template name="titlepage-ednotice"/>
  <xsl:call-template name="generated-frontmatter"/>
    <!-- place main content (fall through occurs here) -->
    <xsl:call-template name="main-doc2"/>
  <!-- return to place closing generated content -->
  <!--xsl:call-template name="index-chapter"/-->
  <xsl:call-template name="back-covers"/>
</fo:root>
</xsl:template>



<xsl:template name="define-page-masters">
  <fo:layout-master-set>
    <!-- master set for chapter pages, first page is the title page -->
    <fo:page-sequence-master master-name="chapter-master">
      <fo:repeatable-page-master-alternatives>
        <fo:conditional-page-master-reference page-position="first" 
          odd-or-even="odd"
          master-name="chapter-first-odd"/>
        <fo:conditional-page-master-reference page-position="first" 
          odd-or-even="even"
          master-name="chapter-first-even"/>
        <fo:conditional-page-master-reference page-position="rest" 
          odd-or-even="odd"
          master-name="chapter-rest-odd"/>
        <fo:conditional-page-master-reference page-position="rest" 
          odd-or-even="even"
          master-name="chapter-rest-even"/>
      </fo:repeatable-page-master-alternatives>
    </fo:page-sequence-master>

    <!-- simple page masters -->
    <fo:simple-page-master master-name="chapter-first-odd"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-bottom="12pt"/>
      <fo:region-after extent="24pt"
       region-name="first-region-after-odd"/> <!-- added this to allow for special copyright content -->
    </fo:simple-page-master>

    <fo:simple-page-master master-name="chapter-first-even"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-bottom="12pt"/>
      <fo:region-after extent="24pt"
       region-name="first-region-after-even"/> <!-- added this to allow for special copyright content -->
    </fo:simple-page-master>

    <fo:simple-page-master master-name="chapter-rest-odd"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-bottom="36pt" margin-top="12pt" />
      <fo:region-before extent="12pt" region-name="rest-region-before-odd"/>
      <fo:region-after extent="24pt"  region-name="rest-region-after-odd"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="chapter-rest-even"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-bottom="36pt" margin-top="12pt" />
      <fo:region-before extent="12pt" region-name="rest-region-before-even"/>
      <fo:region-after extent="24pt"  region-name="rest-region-after-even"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="cover"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-top="72pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="notices"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-top="36pt"/>
      <fo:region-after extent="16pc" region-name="copyr-section"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="common-page"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-bottom="36pt" margin-top="12pt" />
      <fo:region-before extent="12pt"/>
      <fo:region-after  extent="24pt"/>
    </fo:simple-page-master>

  </fo:layout-master-set>
</xsl:template>


<xsl:template name="front-covers">
  <!-- generate an "outside front cover" page (right side) (sheet 1) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block text-align="right" font-family="Helvetica">
        <!-- set the title -->
        <fo:block font-size="30pt" font-weight="bold" line-height="140%">
          <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
        </fo:block>
        <!-- set the subtitle -->
        <fo:block font-size="24pt" font-weight="bold" line-height="140%" margin-bottom="1in">
          <xsl:value-of select="/bkbook/bkprolog/bksubtitle"/>
        </fo:block>
        <!-- place authors as a vertical list -->
        <xsl:for-each select="/bkbook/bkprolog/bkauthor">
          <fo:block font-size="11pt" font-weight="bold" line-height="100%">
            <xsl:value-of select="."/>
          </fo:block>
        </xsl:for-each>
        <!-- set the brief copyright notice -->
        <fo:block font-size="11pt" font-weight="bold" line-height="100%">
          &copyr; &nbsp;&nbsp;<xsl:value-of select="/bkbook/bkprolog/bkcopyr"/>
        </fo:block>
      </fo:block>
      <!-- Custom cover art/text goes here -->
      <xsl:call-template name="place-cover-art"/>
      <!-- End of custom art section -->
      <!--fo:block xsl:use-attribute-sets="p" color="purple" text-align="center">[TBD: check cover margin spec]</fo:block-->
    </fo:flow>
  </fo:page-sequence>

  <!-- generate an "inside front cover" page (left side) (sheet 2) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block xsl:use-attribute-sets="p" color="purple" text-align="center">[This page intentionally left blank.]</fo:block>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>

<xsl:template name="place-cover-art">
<!-- product specific art, etc. -->
<fo:block margin-top="2pc" font-family="Helvetica" border-style="dashed" border-color="black" border-width="thin" padding="6pt">
  <fo:block font-size="12pt" line-height="100%" margin-top="12pc" margin-bottom="12pc" text-align="center">
    <fo:inline color="purple" font-weight="bold">[cover art/text goes here]</fo:inline>
    <!-- one might imbed SVG directly here for use with FOP, for instance -->
  </fo:block>
</fo:block>
</xsl:template>


<xsl:template name="titlepage-ednotice">
  <!-- generate an "internal title page" page (right side) (sheet 3) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block text-align="right" font-family="Helvetica">
        <fo:block font-size="24pt" font-weight="bold" line-height="140%">
          <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
        </fo:block>
        <fo:block font-size="20pt" font-weight="bold" line-height="140%" margin-bottom="1in">
          <xsl:value-of select="/bkbook/bkprolog/bksubtitle"/>
        </fo:block>
        <xsl:for-each select="/bkbook/bkprolog/bkauthor">
          <fo:block font-size="11pt" font-weight="bold" line-height="100%">
            <xsl:value-of select="."/>
          </fo:block>
        </xsl:for-each>
     </fo:block>
    </fo:flow>
  </fo:page-sequence>

  <!-- generate an edition notices page (left side) (sheet 4) -->
  <fo:page-sequence master-name="notices">
  <!-- float down to static area at end of page: copyright and version notices section -->
    <fo:static-content flow-name="copyr-section" >
      <fo:block text-align="left" font-family="Helvetica" display-align="after"><!-- content-width="80%"-->

<!-- "arbitrary boilerplate": document notice data -->
        <fo:block font-size="9pt" line-height="100%" margin-top="6pt">
          <fo:block font-weight="bold">
            <xsl:value-of select="/bkbook/bknotices/bkednotices/title"/>
          </fo:block>
          <fo:block margin-top="9pt">
            <xsl:apply-templates select="/bkbook/bknotices/bkednotices/*[name() != 'title']"/>
          </fo:block>
        </fo:block>

<!-- fixed boilerplate: grant of usage -->
        <fo:block font-size="9pt" line-height="100%" margin-top="6pt">
When you send information to &copyrowner;, you grant &copyrowner; a non-exclusive right to use or distribute the
information in any way it believes appropriate without incurring and obligation to you.
        </fo:block>

<!-- fixed boilerplate: copyright info -->
        <fo:block font-size="7pt" line-height="100%" margin-top="6pt">
&copyr; Copyright &copyrowner; &copyrdate;. All rights reserved.
        </fo:block>

<!-- fixed boilerplate: disclaimers -->
        <fo:block font-size="7pt" line-height="100%" margin-top="6pt">
Note to U.S. Government Users -- Documentation related to restricted rights -- Use, duplication or
disclosure is subject to restrictions set forth in GSA ADP Schedule Contract with &copyrowner;.
        </fo:block>
      </fo:block>
    </fo:static-content>

  <!-- Flow from top of page: redirect to notices page here (this area builds downward from top of page) -->
    <fo:flow flow-name="xsl-region-body" >
      <fo:block text-align="left" font-family="Helvetica" border-style="solid" border-color="black" border-width="thin" padding="6pt">

        <fo:block font-size="9pt" line-height="100%" margin-top="6pt">
          <fo:block font-weight="bold">
            <xsl:value-of select="/bkbook/bknotices/bkspecialnotices/title"/>
          </fo:block>
          <fo:block margin-top="9pt">
            <xsl:apply-templates select="/bkbook/bknotices/bkspecialnotices/*[name() != 'title']"/>
          </fo:block>
        </fo:block>

      </fo:block>
    </fo:flow>

  </fo:page-sequence>
</xsl:template>


<!-- definitions for placement of Front Matter content -->

<xsl:template name="generated-frontmatter">
  <fo:page-sequence master-name="common-page" format="i" initial-page-number="1">
  <!-- Static setup for the generated pages -->
    <!-- header -->
    <fo:static-content flow-name="xsl-region-before">
      <fo:block font-size="8pt" line-height="8pt">&headerstub;</fo:block>
    </fo:static-content>
    <!-- footer -->
    <fo:static-content flow-name="xsl-region-after">
    <fo:block text-align-last="justify">
       <fo:inline text-align="start" font-size="10pt" font-family="Helvetica">
      </fo:inline>
      <fo:leader leader-pattern="use-content"> </fo:leader>
      <fo:inline text-align="end" font-size="10pt" font-weight="bold"  font-family="Helvetica">
        <!--xsl:text>Page </xsl:text--><fo:page-number/>
      </fo:inline>
    </fo:block>
    </fo:static-content>
    <!-- Flow setup for the Front Matter "body" (new "chapters" start on odd pages) -->
    <fo:flow flow-name="xsl-region-body" >
      <!-- first, generate a compulsory Table of Contents -->
      <fo:block line-height="12pt" font-size="10pt"
                font-family="Helvetica"
                id="page1-1">
        <fo:block text-align="left" font-family="Helvetica">
    <fo:block><fo:leader color="black" leader-pattern="rule" rule-thickness="3pt" leader-length="2in"/></fo:block>
          <fo:block font-size="20pt" font-weight="bold" line-height="140%">
            Contents
          </fo:block>
          <xsl:call-template name="gen-toc"/>
      <!--fo:block xsl:use-attribute-sets="p" color="purple" text-align="center">[TBD: list index? how, if style-driven? (not queryable)]</fo:block-->
        </fo:block>
        <!-- genereate the List of Figures -->
        <fo:block text-align="left" font-family="Helvetica" break-before="page">
    <fo:block><fo:leader color="black" leader-pattern="rule" rule-thickness="3pt" leader-length="2in"/></fo:block>
          <fo:block font-size="20pt" font-weight="bold" line-height="140%">
            Figures
          </fo:block>
          <xsl:call-template name="gen-figlist"/>
        </fo:block>
        <!-- generate the List of Tables -->
        <fo:block text-align="left" font-family="Helvetica" break-before="page">
    <fo:block><fo:leader color="black" leader-pattern="rule" rule-thickness="3pt" leader-length="2in"/></fo:block>
          <fo:block font-size="20pt" font-weight="bold" line-height="140%">
            Tables
          </fo:block>
          <xsl:call-template name="gen-tlist"/>
        </fo:block>
      <!-- What else, while still in Roman numbering?  all the bkfrontm content, surely -->
      </fo:block>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>


<!-- initiate main content processing within basic page "shell" -->

<xsl:template name="main-doc2">
  <fo:page-sequence master-name="chapter-master">

    <!-- header: odd page -->
    <fo:static-content flow-name="rest-region-before-odd">
      <fo:block text-align="right" font-size="8pt" line-height="8pt">stub for header content</fo:block>
    </fo:static-content>

    <!-- header: even page -->
    <fo:static-content flow-name="rest-region-before-even">
      <fo:block text-align="left" font-size="8pt" line-height="8pt">stub for header content</fo:block>
    </fo:static-content>

    <!-- footer: odd page -->
    <fo:static-content flow-name="rest-region-after-odd">
      <fo:block text-align-last="justify">
        <fo:inline font-weight="bold">
          [access chapter title via marker here]
          <fo:retrieve-marker retrieve-class-name="chaptitle"/>
        </fo:inline>
        <fo:leader leader-pattern="use-content"> </fo:leader>
        <fo:inline font-weight="bold">
          Page <fo:page-number/> of <fo:page-number-citation ref-id="endofdoc"/>
        </fo:inline>
      </fo:block>
    </fo:static-content>

    <!-- footer: even page -->
    <fo:static-content flow-name="rest-region-after-even">
      <fo:block text-align-last="justify">
        <fo:inline font-weight="bold">
          Page <fo:page-number/> of <fo:page-number-citation ref-id="endofdoc"/>
        </fo:inline>
        <fo:leader leader-pattern="use-content"> </fo:leader>
        <fo:inline font-weight="bold">
          <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
       </fo:inline>
      </fo:block>
    </fo:static-content>


 <!-- special footers for first page of new chapter -->

    <!-- footer: odd page when a new chapter occurs on it -->
    <fo:static-content flow-name="first-region-after-odd">
      <fo:block text-align-last="justify">
        <fo:inline color="purple">
          [copyright statement here]
        </fo:inline>
        <fo:leader leader-pattern="use-content"> </fo:leader>
        <fo:inline font-weight="bold">
          Page <fo:page-number/> of <fo:page-number-citation ref-id="endofdoc"/>
        </fo:inline>
      </fo:block>
    </fo:static-content>

    <!-- footer: even page -->
    <fo:static-content flow-name="first-region-after-even">
      <fo:block text-align-last="justify">
        <fo:inline font-weight="bold">
          Page <fo:page-number/> of <fo:page-number-citation ref-id="endofdoc"/>
        </fo:inline>
        <fo:leader leader-pattern="use-content"> </fo:leader>
        <fo:inline font-weight="bold">
          <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
       </fo:inline>
      </fo:block>
    </fo:static-content>



    <!-- Flow setup for the main content (frontm, body, backm) (new "chapters" start on odd pages) -->
    <fo:flow flow-name="xsl-region-body">
      <!-- chapter body content here -->
      <fo:block text-align="left" font-size="10pt" font-family="Helvetica" break-before="page">
        <xsl:apply-templates />
      </fo:block>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>




<xsl:template name="main-doc">
  <fo:page-sequence master-name="common-page" format="1" initial-page-number="1">
  <!-- Static setup for the generated pages -->
    <!-- header -->
    <fo:static-content flow-name="xsl-region-before">
      <fo:block font-size="8pt" line-height="8pt">stub for header content</fo:block>
      <!--fo:block><fo:leader leader-pattern="rule" space-after.optimum="12pt" leader-length="100%" /></fo:block--> 
    </fo:static-content>
    <!-- footer -->
    <fo:static-content flow-name="xsl-region-after">
    <fo:block text-align-last="justify">
      <fo:inline font-weight="bold">
       <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
         <fo:retrieve-marker retrieve-class-name="chaptitle"/>
       </fo:inline>
      <fo:leader leader-pattern="use-content"> </fo:leader>
      <fo:inline font-weight="bold">
        Page <fo:page-number/> of <fo:page-number-citation ref-id="endofdoc"/>
      </fo:inline>
    </fo:block>
    </fo:static-content>
    <!-- Flow setup for the main content (frontm, body, backm) (new "chapters" start on odd pages) -->
    <fo:flow flow-name="xsl-region-body" >
      <!-- chapter body content here -->
      <fo:block line-height="12pt" font-size="10pt" font-family="Helvetica" id="page1-1">
        <xsl:apply-templates />
      </fo:block>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>



<xsl:template name="main-doc3">
  <fo:page-sequence master-name="chapter-master">
    <!-- header: odd page -->
    <fo:static-content flow-name="rest-region-before-odd">
      <fo:block text-align="end" font-weight="bold">
        <!-- book title here -->
        <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
      </fo:block>
      <fo:block>
        <fo:leader leader-pattern="rule" space-after.optimum="12pt" leader-length="100%"/>
      </fo:block>
    </fo:static-content>
    <!-- header: even page -->
    <fo:static-content flow-name="rest-region-before-even">
      <fo:block text-align="start" font-weight="bold">
        <!-- chapter title here (what is the current context?)-->
        <xsl:value-of select="descendant::bkchapter/*/*[contains(@spec,' topic.title ')]"/>
      </fo:block>
      <fo:block>
        <fo:leader leader-pattern="rule" space-after.optimum="12pt" leader-length="100%"/>
      </fo:block>
    </fo:static-content>
    <!-- footer static stuff -->
    <fo:static-content flow-name="xsl-region-after">
      <fo:block text-align="center">
        <fo:page-number/>
      </fo:block>
    </fo:static-content>
    <!-- Flow setup for the main content (frontm, body, backm) (new "chapters" start on odd pages) -->
    <fo:flow flow-name="xsl-region-body">
      <!-- chapter body content here -->
      <xsl:apply-templates/>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>

<!-- this needs to actually be "generated back matter" based on the same for front matter -->
<xsl:template name="index-chapter">
<!-- Flow setup for the Front Matter "body" (new "chapters" start on odd pages) -->
<fo:flow flow-name="xsl-region-body" >
  <fo:block text-align="left" font-family="Helvetica" break-before="page">
    <fo:block><fo:leader color="black" leader-pattern="rule" rule-thickness="3pt" leader-length="2in"/></fo:block>
    <fo:block font-size="20pt" font-weight="bold" line-height="140%">
      Index
    </fo:block>
    <!-- nothing here yet -->
    <xsl:call-template name="gen-index"/>
  </fo:block>
</fo:flow>
</xsl:template>


<xsl:template name="back-covers">
  <!-- generate an "inside back cover" page (right side) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block xsl:use-attribute-sets="p" text-align="center">[this page intentionally left blank.]</fo:block>
      <fo:block id="endofdoc"></fo:block> <!-- use this to help generate "page n of x" page numbers -->
    </fo:flow>
  </fo:page-sequence>

  <!-- generate an "outside back cover" page (left side) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block text-align="left" font-family="Helvetica">
        <xsl:call-template name="place-logo-backcover"/>
        <fo:block font-size="10pt">
          <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
        </fo:block>
        <fo:block font-weight="bold" color="green">(/_\)</fo:block>
        <fo:block font-size="8pt" white-space="pre">
          Printed in <xsl:value-of select="/bkbook/bkprolog/bkprintloc"/>
          on recycled paper containing 10% recovered post-consumer fiber.
        </fo:block>
      </fo:block>
    </fo:flow>
  </fo:page-sequence>

</xsl:template>

<!-- customize this rule for company-specific logoization of back cover -->
<xsl:template name="place-logo-backcover">
        <fo:block color="purple"
          margin-bottom="1in"
          margin-top="3pc">
          <fo:inline font-size="20pt" font-weight="bold">&copyrowner;</fo:inline>
          <fo:inline baseline-shift="sub" font-size="9pt">&copyr;</fo:inline>
        </fo:block>
</xsl:template>




<!-- original: special handling of galley like content within boxed contexts; simplify! -->

<xsl:template match="bkednotices/p|bkspecialnotices/p" priority="2">
 <fo:block><xsl:apply-templates /></fo:block>
</xsl:template>


<!--==========================================================================
 |
 | Main support for generators: toc, index, figlist, etc..
 |
 +-->

<!-- main index generator -->
<xsl:template name="gen-index">
  <!-- nothing here yet -->
</xsl:template>

<!-- main tlist (tables) generator -->
<xsl:template name="gen-tlist">
<xsl:for-each select="//table[title]">
  <fo:block margin-left="+4.9pc" text-align-last="justify">
    <xsl:number level="any" from="bkbody"/>. <xsl:value-of select="title"/>
    <fo:leader leader-pattern="dots"/>
    <fo:page-number-citation ref-id="{@id}"/>
  </fo:block>
</xsl:for-each>
</xsl:template>

<!-- main figlist generator -->
<xsl:template name="gen-figlist">
<!--fo:block xsl:use-attribute-sets="p" color="purple">[TBD: numbers for frontm figs? backm figs?]</fo:block-->
<xsl:for-each select="//fig[@id]">
  <fo:block margin-left="+4.9pc" text-align-last="justify">
    <xsl:number level="any" from="bkbook"/>. <xsl:value-of select="label"/>
    <fo:leader leader-pattern="dots"/>
    <fo:page-number-citation ref-id="{@id}"/>
  </fo:block>
</xsl:for-each>
</xsl:template>


<!-- main toc generator -->
<xsl:template name="gen-toc">
<!-- get by main part: frontm -->
  <xsl:for-each select="//bkfrontm/bkchapter">
    <fo:block text-align-last="justify" margin-top="6pt" margin-left="+4.9pc">
      <fo:inline font-weight="bold"><xsl:value-of select="*/title"/></fo:inline>
      <fo:leader leader-pattern="dots"/>
      <fo:page-number-citation ref-id="{generate-id()}"/>
    </fo:block>
    <xsl:call-template name="get-tce2-section"/>
  </xsl:for-each>
<!-- get by main part: body -->
  <xsl:for-each select="//bkbody/bkchapter">
    <fo:block text-align-last="justify" margin-top="6pt" margin-left="+4.9pc">
      <fo:inline font-weight="bold">Chapter <xsl:number level="any" from="bkbody"/>. <xsl:value-of select="*/title"/></fo:inline>
      <fo:leader leader-pattern="dots"/>
      <fo:page-number-citation ref-id="{generate-id()}"/>
    </fo:block>
    <xsl:call-template name="get-tce2-section"/>
  </xsl:for-each>
<!-- get by main part: appendix -->
  <xsl:for-each select="//bkappendixes/bkappendix">
    <fo:block text-align-last="justify" margin-top="6pt" margin-left="+4.9pc">
      <fo:inline font-weight="bold">Appendix <xsl:number from="bkappendixes" format="A"/>. <xsl:value-of select="*/title"/></fo:inline>
      <fo:leader leader-pattern="dots"/>
      <fo:page-number-citation ref-id="{generate-id()}"/>
    </fo:block>
    <xsl:call-template name="get-tce2-section"/>
  </xsl:for-each>
<!-- get by main part: backm -->
  <xsl:for-each select="//bkbackm/bkglossary">
    <fo:block text-align-last="justify" margin-top="6pt" margin-left="+4.9pc">
      <fo:inline font-weight="bold"><xsl:value-of select="*/title"/></fo:inline>
      <fo:leader leader-pattern="dots"/>
      <fo:page-number-citation ref-id="{generate-id()}"/>
    </fo:block>
    <xsl:call-template name="get-tce2-section"/>
  </xsl:for-each>
</xsl:template>

<!-- 2nd level header -->
<xsl:template name="get-tce2-section">
<xsl:for-each select="bksection">
  <fo:block text-align-last="justify" margin-left="+4.9pc">
    <fo:inline font-weight="bolder"><xsl:value-of select="*/title"/></fo:inline>
    <fo:leader leader-pattern="dots"/>
    <fo:page-number-citation ref-id="{generate-id()}"/>
  </fo:block>
  <xsl:call-template name="get-tce3-section"/>
</xsl:for-each>
</xsl:template>

<!-- 3nd level header -->
<xsl:template name="get-tce3-section">
<xsl:for-each select="bksubsection">
  <fo:block text-align-last="justify" margin-left="+5.9pc">
    <xsl:value-of select="*/title"/>
    <fo:leader leader-pattern="dots"/>
    <fo:page-number-citation ref-id="{generate-id()}"/>
  </fo:block>
  <xsl:call-template name="get-tce4-section"/>
</xsl:for-each>
</xsl:template>

<!-- 4th level header -->
<xsl:template name="get-tce4-section">
<xsl:for-each select="bksubsect1">
  <fo:block text-align-last="justify" margin-left="+5.9pc">
    <xsl:value-of select="*/title"/>
    <fo:leader leader-pattern="dots"/>
    <fo:page-number-citation ref-id="{generate-id()}"/>
  </fo:block>
  <xsl:call-template name="get-tce5-section"/>
</xsl:for-each>
</xsl:template>

<!-- 5th level header -->
<xsl:template name="get-tce5-section">
<xsl:for-each select="bksubsect2">
  <fo:block text-align-last="justify" margin-left="+5.9pc">
    <xsl:value-of select="*/title"/>
    <fo:leader leader-pattern="dots"/>
    <fo:page-number-citation ref-id="{generate-id()}"/>
  </fo:block>
  <!--xsl:call-template name="get-tce6-section"/-->
</xsl:for-each>
</xsl:template>



</xsl:stylesheet>