<?xml version="1.0" encoding="UTF-8" ?>

<!--  dit2htm.xsl 1.02; 06 Sept 2001
 | DITA topic to HTML; "single topic to single web page"-level view
 |
 | Copyright IBM Corporation, 2001
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +======================================
 | Note that the localization method is observed to work okay for Saxon
 | (for example, C:\pkg\dita>saxon DITA-readme.xml ss\dit2htm.xsl Lang=sp > DITA-readme.htm
 | will produce a version with Spanish generated text), but msxml seems to pick whatever
 | was last defined in the strings.xml file.
 |
 *-->

<!DOCTYPE xsl:stylesheet [
<!-- entities for use in the generated output (must produce correctly in HTML) -->
<!ENTITY gt     "&gt;"> 
<!ENTITY lt     "&lt;"> 
<!ENTITY copyr  "&#169;">
<!ENTITY rbl    "&#160;">
<!ENTITY nbsp   "&#160;">
<!ENTITY quot   "&#34;">
]>

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">



<!-- stylesheet imports -->


<!-- set output method here.  base format is standard html -->
<xsl:output method="html" encoding="iso-8859-1"
            indent="yes" />



<!-- null out some things whose content will be built elsewhere or excluded -->
<!--xsl:template match="fig/desc"/-->
<xsl:template match="*[contains(@spec,' topic.desc ')]"/>
<!--xsl:template match="*[contains(@spec,' topic.title ')]"/-->
<xsl:template match="*[contains(@spec,' topic.section ')]/*[contains(@spec,' topic.title ')]" priority="100"/>
<xsl:template match="*[contains(@spec,' topic.fig ')]/*[contains(@spec,' topic.title ')]" priority="100"/>
<xsl:template match="*[contains(@spec,' topic.table ')]/*[contains(@spec,' topic.title ')]" priority="100"/>



<!-- footnotes can be collected as endnotes or cast as popups -->
<xsl:template match="*[contains(@spec,' topic.fn ')]">
<xsl:variable name="fnid"><xsl:number from="/" level="any"/></xsl:variable>
    <xsl:variable name="fnbody"><xsl:apply-templates/></xsl:variable>
<a name="fnsrc_{$fnid}" href="#fntarg_{$fnid}" OnClick='alert ("{$fnbody}")'>
  <sup><xsl:number from="/" level="any"/></sup>
</a>
</xsl:template>


<!-- "GLOBAL" DECLARATIONS -->

<xsl:variable name="img-path"></xsl:variable> <!-- this will be specific to your delivery tree! -->


<!-- TBD: move localizable rules/defs to external, language-controlled imports -->
<!-- ITEMS FOR TRANSLATION/GLOBALIZATION -->

<!-- default language parameter -->
<xsl:param name="Lang" select="'en-US'"/>
<xsl:variable name="StringFile" select="document('strings.xml')"/>
<xsl:variable name="PrimaryLang" select="substring-before($Lang,'-')"/>


<!-- Localizable rules -->

<xsl:template match="*[contains(@spec,' topic.date ')]" priority="1">
  <!-- apply lexical checking and locale-specific rendering for this content -->
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.time ')]" priority="1">
  <!-- apply lexical checking and locale-specific rendering for this content -->
  <!-- for example, if the lang/country code is en-US, produce a $ sign -->
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.currency ')]" priority="1">
  <!-- apply lexical checking and locale-specific rendering for this content -->
  <xsl:apply-templates/>
</xsl:template>



<!-- ==== START OF DITA COMPLIANT TRANSFORMS ==== -->

<!-- For this stylesheet, the document element is 
     presumed to be either a topic or a specialized topic. -->



<!-- TYPED TOPIC MAIN STRUCTURES -->

<!-- root rule... just fall through; no side effects for new delivery contexts -->

<xsl:template match="/">
  <xsl:comment>Sample DITA to HTML application by IBM Corp., 2001</xsl:comment>
  <xsl:apply-templates/>
</xsl:template>



<!-- these template rules generate the outer-level shell for a delivery context -->

<xsl:template match="/*[contains(@spec,' topic.topic ')]">
<html>
  <head>
    <link rel="stylesheet" type="text/css" href="../ss/common.css" />
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
    <meta http-equiv="generator" content="IBM Darwin Information Typing Architecture dit2htm transform"/>
    <title>
      <xsl:call-template name="gen-user-panel-title-pfx"/>
<xsl:value-of select="*[contains(@spec, 'topic.title')]"/>
    </title>
    <!-- include scripts here -->
    <xsl:call-template name="gen-user-scripts"/>
    <!--style type="text/css"/-->
  </head>
  <body class="base">

  <xsl:call-template name="gen-user-header"/>
    <!-- here, you can generate a toc based on what's a child of topic -->
    <xsl:call-template name="gen-user-sidetoc"/>

    <xsl:apply-templates/> <!-- this will include all things within topic; therefore, -->
    <!-- title content will appear here by fall-through -->
    <!-- followed by prolog, if fall-through is permitted on it -->
    <!-- followed by body content, again by fall-through in document order -->
    <!-- finally, and child topics and includes will imbed here by fall-through -->

  <xsl:call-template name="gen-relsect"/>
  <xsl:call-template name="gen-endnotes"/>
  <xsl:call-template name="gen-user-footer"/>

  </body>
</html>
</xsl:template>


<!-- includes and child topics get a div wrapper and fall through -->

<xsl:template match="*/*[contains(@spec, 'topic.topic ')]" priority="3">
  <div class="nested1">
  <xsl:if test="@id"><a name="{@id}"></a></xsl:if>
  <a><xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute> </a>
    <xsl:apply-templates/>
  </div>
</xsl:template>

<xsl:template match="*/*/*[contains(@spec, 'topic.topic ')]" priority="4">
  <div class="nested2">
  <xsl:if test="@id"><a name="{@id}"></a></xsl:if>
    <xsl:apply-templates/>
  </div>
</xsl:template>

<xsl:template match="*/*/*/*[contains(@spec, 'topic.topic ')]" priority="5">
  <div class="nested3">
  <xsl:if test="@id"><a name="{@id}"></a></xsl:if>
    <xsl:apply-templates/>
  </div>
</xsl:template>

<xsl:template match="*/*/*/*/*[contains(@spec, 'topic.topic ')]" priority="6">
  <div class="nested4">
  <xsl:if test="@id"><a name="{@id}"></a></xsl:if>
    <xsl:apply-templates/>
  </div>
</xsl:template>

<xsl:template match="*/*/*/*/*/*[contains(@spec, 'topic.topic ')]" priority="7">
  <div class="nested5">
  <xsl:if test="@id"><a name="{@id}"></a></xsl:if>
    <xsl:apply-templates/>
  </div>
</xsl:template>


<!-- titles get level appropriate treatment, but are still processed for contained markup -->

<xsl:template match="/*[contains(@spec,' topic.topic ')]/*[contains(@spec,' topic.title ')]">
  <a><xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute> </a>
  <h1 class="topictitle1"><xsl:call-template name="get-title"/></h1>
  <xsl:call-template name="gen-ptoc"/>
</xsl:template>

<xsl:template match="/*/*/*[contains(@spec,' topic.title ')]">
  <a><xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute> </a>
  <h2 class="topictitle2"><xsl:call-template name="get-title"/></h2>
</xsl:template>

<xsl:template match="/*/*/*/*[contains(@spec,' topic.title ')]">
  <a><xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute> </a>
  <h3 class="topictitle3"><xsl:call-template name="get-title"/></h3>
</xsl:template>

<xsl:template match="/*/*/*/*/*[contains(@spec,' topic.title ')]">
  <a><xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute> </a>
  <h4 class="topictitle4"><xsl:call-template name="get-title"/></h4>
</xsl:template>

<xsl:template match="/*/*/*/*/*/*[contains(@spec,' topic.title ')]">
  <a><xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute> </a>
  <h5 class="topictitle5"><xsl:call-template name="get-title"/></h5>
</xsl:template>

<xsl:template match="/*/*/*/*/*/*/*/*[contains(@spec,' topic.title ')]">
  <a><xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute> </a>
  <h6 class="topictitle6"><xsl:call-template name="get-title"/></h6>
</xsl:template>



<!-- the following named templates generate inline content for the delivery context -->


<!-- named templates for labels and titles related to topic structures -->

<xsl:template name="get-title"><!-- get fully-processed title content by whatever mechanism -->
  <xsl:choose>
   <xsl:when test="@classif">
     <xsl:value-of select="//classdef[@classname=current()/@classif]/title"/>
   </xsl:when>
   <xsl:when test="@title">
     <xsl:value-of select="@title"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:apply-templates/> <!-- select="title|*[contains(@spec,' topic.title ')]"/-->
   </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="get-sect-heading">
     <xsl:choose>
      <xsl:when test="@classif">
        <xsl:value-of select="//classdef[@classname=current()/@classif]/title"/>
      </xsl:when>
      <xsl:when test="@title">
        <xsl:value-of select="@title"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="title"/>
      </xsl:otherwise>
     </xsl:choose>
</xsl:template>

<xsl:template name="sect-heading">
  <xsl:param name="deftitle" select="."/> <!-- get param by reference -->
  <xsl:variable name="heading">
     <xsl:choose>
      <xsl:when test="@classif">
        <xsl:value-of select="//classdef[@classname=current()/@classif]/*[contains(@spec,' topic.title ')]"/>
      </xsl:when>
      <xsl:when test="@title">
        <xsl:value-of select="@title"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="*[contains(@spec,' topic.title ')]"/>
      </xsl:otherwise>
     </xsl:choose>
  </xsl:variable>

  <!-- based on graceful defaults, build an appropriate section-level heading -->
  <xsl:choose>
    <xsl:when test="not($heading='')">
      <xsl:if test="normalize-space($heading)=''">
        <!-- hack: a title with whitespace ALWAYS overrides as null -->
        <xsl:comment>no heading</xsl:comment>
      </xsl:if>
      <p class="sectiontitle"><b><xsl:call-template name="proc-ing"/><xsl:value-of select="$heading"/></b></p>
    </xsl:when>
    <xsl:when test="$deftitle">
      <p class="sectiontitle"><b><xsl:value-of select="$deftitle"/></b></p>
    </xsl:when>
    <xsl:otherwise><!-- no output --></xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="proc-ing">
  <xsl:if test="$do-place-ing = 'yes'"> <!-- set in a global variable, as with label placement, etc. -->
    <img src="tip-ing.jpg"/> <!-- this should be an xsl:choose with the approved list and a selection method-->
    <!-- add any other required positioning controls, if needed, but must be valid in the location
         from which the call to this template was made -->
    &nbsp;
  </xsl:if>
</xsl:template>



<!-- template rules not sensitive to nesting depth -->

<xsl:template match="*[contains(@spec,' topic.body ')]">
<!-- here, you can generate a toc based on what's a child of body -->
    <xsl:call-template name="gen-body-qatoc"/> <!-- only one of these will usually "take" -->
    <xsl:call-template name="gen-sect-qatoc"/>
    <!--xsl:call-template  name="gen-sect-ptoc"/--><!--works! not always wanted, tho-->
  <xsl:apply-templates/>
</xsl:template>



<!-- Pass through things whose content will be used out of apply=templates sequence -->
<!-- (this includes titles of all section-like exhibits) -->



  <!-- == TOPIC UNIQUE SUBSTRUCTURES == -->

<!-- title excluded from apply-templates call: produced dup -->
<xsl:template match="*[contains(@spec,' topic.section ')]">
  <!-- place locally defined IDs for use by xref -->
  <xsl:if test="@id"><a name="{@id}"> </a></xsl:if>
  <xsl:call-template name="sect-heading">
     <xsl:with-param name="deftitle"></xsl:with-param>
  </xsl:call-template>

  <xsl:apply-templates/>

  <xsl:if test="$link-top-section='yes'">
    <p align="LEFT"><a href="#TOP">
      <!--xsl:value-of select="$deftxt-linktop"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Return to Top'"/>
      </xsl:call-template>
    </a></p>
  </xsl:if>
</xsl:template>



<xsl:template match="*[contains(@spec,' topic.xmp ')]">
  <xsl:call-template name="sect-heading">
    <xsl:with-param name="deftitle">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Example'"/>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:apply-templates/>
  <xsl:if test="$link-top-section='yes'">
    <p align="LEFT"><a href="#TOP">
      <!--xsl:value-of select="$deftxt-linktop"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Return to Top'"/>
      </xsl:call-template>
    </a></p>
  </xsl:if>
</xsl:template>



  <!-- == REFTOPIC UNIQUE SUBSTRUCTURES == -->
  <!-- when a specialized rule is imbedded with the topic defaults, there will be
       a priority conflict unless you set the priority of the contained specialied rule
       so that it overrides the topic fallback -->

<xsl:template match="*[contains(@spec,' reftopic.refsect ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <blockquote>
    <xsl:apply-templates/>
  </blockquote>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.refsyn ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle">
      <!--xsl:value-of select="$deftxt-refsyn"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Syntax'"/>
      </xsl:call-template>
      </xsl:with-param></xsl:call-template>
  <blockquote>
    <xsl:apply-templates/>
  </blockquote>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.refxmp ')]" priority="1">
  <xsl:call-template name="sect-heading">
  <xsl:with-param name="deftitle">
    <!--xsl:value-of select="$deftxt-refxmp"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Examples'"/>
      </xsl:call-template>
  </xsl:with-param></xsl:call-template>
  <blockquote>
    <xsl:apply-templates/>
  </blockquote>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.properties ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <dl><xsl:apply-templates /></dl>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.property ')]" priority="1">
  <!--dlentry-->
    <xsl:apply-templates />
  <!--/dlentry-->
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.proptype ')]" priority="1">        
  <dt><xsl:apply-templates /></dt>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.propvalue ')]" priority="1">
  <dd><xsl:apply-templates /></dd>
</xsl:template>


<!-- parameter lists (specialized DL within a refsect or refbody)  -->
<xsl:template match="*[contains(@spec,' reftopic.parml ')]" priority="1">
  <dl>
  <xsl:apply-templates/>
  </dl>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.plentry ')]" priority="1">
  <dlentry><xsl:apply-templates/></dlentry>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.pt ')]" priority="1">
  <dt><a name="pl_{.}"><xsl:apply-templates/></a></dt>
</xsl:template>

<xsl:template match="*[contains(@spec,' reftopic.pd ')]" priority="1">
  <dd><xsl:apply-templates/></dd>
</xsl:template>


  <!-- == TASK UNIQUE SUBSTRUCTURES == -->

<xsl:template match="*[contains(@spec,' task.steps ')]" priority="1">
  <ol><xsl:apply-templates/></ol>
</xsl:template>

<!-- optreq support implemented here -->
<xsl:template match="*[contains(@spec,' task.step ')]" priority="1">
  <li>
    <xsl:if test="@importance='optional' or @optreq='opt'">
      <strong>
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Optional'"/>
      </xsl:call-template><xsl:text>: </xsl:text>
      </strong>
    </xsl:if>
    <xsl:call-template name="li.name"/>
    <xsl:apply-templates/>
  </li>
</xsl:template>


<xsl:template match="*[contains(@spec,' task.cmd ')]" priority="1">
  <span><xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.context ')]" priority="1">
  <xsl:call-template name="sect-heading">
    <xsl:with-param name="deftitle">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Context'"/>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:apply-templates/>
</xsl:template> 

<xsl:template match="*[contains(@spec,' task.prereq ')]" priority="1">
  <xsl:call-template name="sect-heading">
    <xsl:with-param name="deftitle">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Prerequisites'"/>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:apply-templates/>
</xsl:template> 

<xsl:template match="*[contains(@spec,' task.postreq ')]" priority="1">
  <xsl:call-template name="sect-heading">
    <xsl:with-param name="deftitle">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Postrequisites'"/>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:apply-templates/>
</xsl:template> 

<xsl:template match="*[contains(@spec,' task.result ')]" priority="1">
  <xsl:call-template name="sect-heading">
    <xsl:with-param name="deftitle">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Results'"/>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:apply-templates/>
</xsl:template> 

<!-- custom version of example within a task (No return to top behavior) -->
<xsl:template match="*[contains(@spec,' task.taskbody ')]/*[contains(@spec,' topic.xmp ')]" priority="5">
  <xsl:call-template name="sect-heading">
    <xsl:with-param name="deftitle">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Example'"/>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:apply-templates/>
</xsl:template>


<xsl:template match="*[contains(@spec,' task.substeps ')]" priority="1">
  <ol type="a"><xsl:apply-templates/></ol>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.substep ')]" priority="1">
  <li>
    <xsl:if test="@importance='optional' or @optreq='opt'">
      <strong>
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Optional'"/>
      </xsl:call-template><xsl:text>: </xsl:text>
      </strong>
    </xsl:if>
    <xsl:call-template name="li.name"/>
    <xsl:apply-templates/>
  </li>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.choices ')]" priority="1">
  <ul><xsl:apply-templates/></ul>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.choice ')]" priority="1">
  <li>
    <xsl:call-template name="li.name"/>
    <xsl:apply-templates/>
  </li>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.ifcond ')]" priority="1">
  <li>
    <xsl:call-template name="li.name"/>
    <xsl:apply-templates/>
  </li>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.stepresult ')]" priority="1">
  <p><xsl:apply-templates/></p>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.info ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <div class="liblock"><xsl:apply-templates/></div>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.tutorialinfo ')]" priority="1">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <div class="liblock"><xsl:apply-templates/></div>
</xsl:template>

<xsl:template match="*[contains(@spec,' task.taskxmp ')]" priority="1">
  <xsl:call-template name="sect-heading">
    <xsl:with-param name="deftitle">
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Examples'"/>
      </xsl:call-template>
    </xsl:with-param>
  </xsl:call-template>
  <div class="liblock"><xsl:apply-templates/></div>
</xsl:template>


<!-- ========= regular body content |*[contains(@spec,' topic.')]============== -->


<!-- list item block like li/lq but without presentation (indent) -->
<xsl:template match="*[contains(@spec,' topic.liblock ')]">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <div class="liblock"><xsl:apply-templates/></div>
</xsl:template>


<!-- newly added "XHTML compatibility subset" extras -->

<xsl:template match="*[contains(@spec,' topic.b ')]">
<b><xsl:apply-templates/></b>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.i ')]">
<i><xsl:apply-templates/></i>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.strong ')]">
<strong><xsl:apply-templates/></strong>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.em ')]">
<em><xsl:apply-templates/></em>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.tt ')]">
<tt><xsl:apply-templates/></tt>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.code ')]">
<code><xsl:apply-templates/></code>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.samp ')]">
<samp><xsl:apply-templates/></samp>
</xsl:template> 


<!-- TYPED CONTENT (not likely to be further specialized at this granularity) -->


<xsl:template match="*[contains(@spec,' topic.synph ')]">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.comment ')]">
  <span style="color:purple; font-style:italic;"><xsl:apply-templates/></span>
</xsl:template>


<!-- syntax diagram markup language -->

<!-- implemented @id support -->
<xsl:template match="*[contains(@spec,' topic.syntax ')]">
  <div>
   <xsl:if test="@id"><a name="{@id}" ></a></xsl:if>
   <xsl:apply-templates/></div>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.kbd ')]">
  <kbd><xsl:apply-templates/></kbd>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.kwd ')]">
  <kwd><b><xsl:apply-templates/></b></kwd>
</xsl:template>

<!-- match="var|*[contains(@spec,' topic.var ')]" -->

<xsl:template match="*[contains(@spec,' topic.flag ')]">
  <span style="color: red"><xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.parm ')]">
  <span style="color: navy; font-weight: bold;"><xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.repsep ')]">
  <span style="color: magenta"><xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.delim ')]">
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.oper ')]">
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sep ')]">
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.literal ')]">
"<xsl:apply-templates/>"
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.group ')]">
<xsl:choose>
  <xsl:when test="@composite='composite'">
    <xsl:apply-templates/>
  </xsl:when>
  <xsl:otherwise><pre>
    <xsl:apply-templates/>
    </pre>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>



<!-- ALTERNATE TERM/LINK ELEMENTS -->

<xsl:template name="altterm"> <!-- call from any match="term"-like template -->
  <span class="underlined">
    <xsl:apply-templates/>
  </span>
</xsl:template>

<xsl:template name="altlink"> <!-- call from any match="link"-like template -->
  <span class="unresolved">
    <xsl:apply-templates />
  </span>
</xsl:template>





<!-- functional elements -->

<xsl:template match="*[contains(@spec,' topic.null ')]"><xsl:value-of select="name()"/></xsl:template>

<xsl:template match="include|topicref">
  <xsl:if test="@ method='link'">
    <!-- create a link to the designated file -->
    <xsl:variable name="lfile"><xsl:value-of select="@href"/></xsl:variable>
    <p><a href="{$lfile}"><xsl:apply-templates/><xsl:value-of select="$lfile"/></a></p>
  </xsl:if>

  <!-- using "$ifile,." causes lookup to be relative to the parent document -->
  <xsl:variable name="ifile"><xsl:value-of select="@href"/></xsl:variable>
  <xsl:variable name="extfile" select="document($ifile,.)"/>
<hr size="10"/>
  <xsl:apply-templates select="$extfile"/> <!-- this should do proper fall-through -->
</xsl:template>


<!-- catch undefined elements, for stylesheet maintainers -->

<xsl:template match="*">
   <span style="background-color: yellow;">
     <span style="font-weight: bold">
       <xsl:text>[</xsl:text>
       <xsl:for-each select="ancestor-or-self::*">
        <xsl:text>/</xsl:text>
        <xsl:value-of select="name()" />
       </xsl:for-each>
       {"<xsl:value-of select="@spec"/>"}<xsl:text>) </xsl:text>
     </span>
     <xsl:apply-templates/>
     <span style="font-weight: bold">
       <xsl:text> (</xsl:text><xsl:value-of select="name()"/><xsl:text>]</xsl:text>
     </span>
   </span>
 </xsl:template>


<!-- catch required cleanup, for authors -->

<xsl:template match="required-cleanup|Required_Cleanup">
  <span style="background-color: #FFFF99; color:#CC3333">
     <b>Required Cleanup <xsl:if test="@remap">(<xsl:value-of select="@remap"/>) </xsl:if><xsl:text>: </xsl:text></b> 
     <xsl:apply-templates/>
  </span>
</xsl:template>


<!-- ============= content relocation from the prolog's elements ================== -->

<!-- PROLOG -->

<xsl:template match="*[contains(@spec,' topic.prolog ')]">
  <!-- "review" = display all metadata; "publish" = functional bits only -->
  <xsl:call-template name="publish"/>
</xsl:template>

<xsl:template name="review">
<table border="1">
<tr><td><p><b>Metadata and properties/descriptions of the following topic content:</b></p></td></tr>
<tr><td bgcolor="lime"><xsl:apply-templates/></td></tr>
<tr><td bgcolor="silver"><xsl:apply-templates select="classdef"/></td></tr>
</table>
</xsl:template>

<xsl:template name="publish">
<xsl:if test="docmeta/author">
 <strong>Author(s)<xsl:text>: </xsl:text></strong>
 <xsl:for-each select="docmeta/author">
   <xsl:value-of select="name"/><br/>
 </xsl:for-each>
</xsl:if>
 <!-- shortdesc and longdesc may be used for "progressive disclosure" leading
      eventually to bringing up the topic itself.  The default presentation is
      to not use them directly in presented topics. -->
 <!--xsl:apply-templates select="shortdesc"/-->
 <!--xsl:apply-templates select="longdesc"/-->
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.shortdesc ')]">
<br/>
<br/>
<blockquote>
    <xsl:apply-templates/>
</blockquote>
<br/>
<br/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.longdesc ')]">
<br/>
<br/>
<blockquote>
    <xsl:apply-templates/>
</blockquote>
<br/>
<br/>
</xsl:template>



<!-- THIS IS DATA DRIVEN FOR NOW; TYPICALLY, METADATA WILL BE USED BY REFERENCE -->
<xsl:template match="*[contains(@spec,' topic.docmeta ')]">
  <table border="1" cellpadding="4" width="95%">
  <tr><td bgcolor="#CCCCCC">
  <xsl:apply-templates/>
  </td></tr>
  </table>
</xsl:template>

<xsl:template match="docmeta/title">
  <strong>Title<xsl:text>: </xsl:text></strong><span style="color:purple"><xsl:value-of select="."/></span><br/>
</xsl:template>

<xsl:template match="metadata/date">
  <strong>Date<xsl:text>: </xsl:text></strong><span style="color:purple"><xsl:apply-templates select="date"/></span><br/>
</xsl:template>

<xsl:template match="docmeta/author"/>

<xsl:template match="docmeta/category">
  <strong>Category<xsl:text>: </xsl:text></strong><span style="color:purple"><xsl:value-of select="."/></span><br/>
</xsl:template>

<xsl:template match="docmeta/subject">  <!-- one ancestor -->
  <strong>Subject<xsl:text>: </xsl:text></strong><span style="color:purple"><xsl:value-of select="."/></span><br/>
</xsl:template>


<xsl:template match="classdef">
<table border="1"><tr><td bgcolor="lime">
 <b>Class Definition for classname<xsl:text>: </xsl:text><xsl:value-of select="@classname"/></b>
 <ul compact="compact">
   <li><b>Title<xsl:text>: </xsl:text></b><xsl:value-of select="title"/></li>
   <li><b>Semantic<xsl:text>: </xsl:text></b><xsl:value-of select="sem"/></li>
   <li><b>Description<xsl:text>: </xsl:text></b><xsl:value-of select="desc"/></li>
   <li>assertion grammar<xsl:text>: </xsl:text>equate <i>classdef/@<xsl:value-of select="@classname"/>/title</i> to its matching section</li>
 </ul>
</td></tr></table>
</xsl:template>

<xsl:template match="copyright">
 <p>
  <b>Copyright &#169; <xsl:value-of select="copyrdate"/>, <xsl:value-of select="copyrholder"/></b>
 </p>
</xsl:template>

<xsl:template match="copyrdate | copyrholder | xmeta | prodinfo">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="prodinfo/prodname">  <!-- one ancestor -->
  <b>Product name<xsl:text>: </xsl:text></b><span style="color:maroon"><xsl:value-of select="."/></span><br/>
</xsl:template>

<xsl:template match="xmeta/audience">
 <br/><b>Audience<xsl:text>: </xsl:text></b>
  <xsl:apply-templates/>
</xsl:template>



<!-- =========== description processors ============ -->

<xsl:template name="get-linkdescinfo">
<!-- we are in a link element context; test to see what we can get -->
<!-- TBD: more than just copy href info into the returned string -->
  <xsl:value-of select="//linkdesc[@linkname=current()/@linkref]/@href"/>
</xsl:template>

<xsl:template name="get-objdescinfo">
<!-- we are in a mmobj element context; test to see what we can get -->
<!-- TBD: more than just copy href info into the returned string -->
  <xsl:value-of select="//objdesc[@objname=current()/@objref]/@href"/>
</xsl:template>

<xsl:template name="get-bibdescinfo">
<!-- we are in a cite element context; test to see what we can get -->
<!-- TBD: copy href info into the returned string -->
<xsl:choose>
  <xsl:when test="normalize-space(.)">
    <xsl:value-of select="."/> <!-- this probably ought to be the fallthrough action -->
  </xsl:when>
  <xsl:otherwise>
    <xsl:value-of select="//bibdesc[@bibname=current()/@bibref]/title"/>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>


<!-- note that if links need to be converted from "*.xml" to "*.html", this is where to do it-->
<xsl:template name="renderrel">       <!-- DOES NOT REFLECT NESTING -->
      <xsl:choose>
       <xsl:when test="name(.)='cite'"><xsl:value-of select="."/>: <xsl:call-template name="get-bibdescinfo"/></xsl:when>
       <xsl:when test="name(.)='link'"><a href="{@href}"><xsl:value-of select="."/></a></xsl:when>
       <xsl:when test="name(.)='rel'"><a href="{@address}"><xsl:value-of select="."/></a></xsl:when>
       <xsl:when test="name(.)='xref'"><a href="#{@refid}"><xsl:value-of select="@refid"/></a></xsl:when>
       <xsl:when test="name(.)='desc'"><xsl:apply-templates/></xsl:when>
       <xsl:otherwise><xsl:value-of select="name(.)"/></xsl:otherwise>
      </xsl:choose> 

      <xsl:if test="@role='parent'"><b>
          <!--xsl:value-of select="$deftxt-parentlink" /-->
          <xsl:call-template name="getString">
            <xsl:with-param name="stringName" select="'Parent Link'"/>
          </xsl:call-template>
        <xsl:text>: </xsl:text></b>
      </xsl:if>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.relgroup ')]/*[contains(@spec,' topic.xtitle ')]"> <!-- similar to See Also -->
  <h3 class="topictitle3"><xsl:value-of select="."/></h3>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.relgroup ')]"> <!-- for direct rendering within the prolog, if that is enabled. -->
 <b>
  <xsl:choose>
    <xsl:when test="xtitle">
      <xsl:value-of select="*[contains(@spec,' topic.title ')]"/>
    </xsl:when>
    <xsl:otherwise>
<!--xsl:value-of select="$deftxt-rellinks"/-->
       <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Link Relations'"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
 </b>
 <ol>
  <xsl:for-each select="rel|link|xref|cite|desc">
   <li>
     <xsl:choose>
      <xsl:when test="name(.)='cite'"><xsl:value-of select="."/>: <xsl:call-template name="get-bibdescinfo"/></xsl:when>
      <xsl:when test="name(.)='link'"><a href="{@href}"><xsl:value-of select="."/></a></xsl:when>
      <xsl:when test="name(.)='rel'"><xsl:value-of select="."/> [<xsl:value-of select="@type"/>] <xsl:value-of select="@address"/></xsl:when>
      <xsl:when test="name(.)='xref'"><a href="#{@refid}"><xsl:value-of select="@refid"/></a></xsl:when>
      <xsl:when test="name(.)='desc'"><xsl:apply-templates/></xsl:when>
      <xsl:otherwise><xsl:value-of select="name(.)"/></xsl:otherwise>
     </xsl:choose>
   </li>
  </xsl:for-each>
 </ol>
</xsl:template>





<!-- things being added back in -->
<!-- ===================================================================== -->

<!-- basic document elements -->

<!-- special formatting for first paragraph in a section-->
<xsl:template match="*[contains(@spec,' topic.section ')]/*[1][contains(@spec,' topic.p ')]" priority="2">
  <xsl:if test="@id"><a name="{@id}"> </a></xsl:if>
       <div class="parafirst" style="margin-bottom: 6pt">
         <xsl:apply-templates />
       </div>
</xsl:template>

<!-- extended with special handling of empty p elements with anchors-->
<xsl:template match="*[contains(@spec,' topic.p ')]" name="parastd">
<!--xsl:if test="name(preceding-sibling::*)='p'"><p><br/></p></xsl:if-->
   <xsl:choose>
     <xsl:when test="not(text()[normalize-space(.)] | *)">
       <p/>
         <xsl:if test="@id"><a name="{@id}"> </a></xsl:if>
     </xsl:when>
     <xsl:otherwise>  
       <div style="margin-bottom: 6pt">
         <xsl:if test="position()=1"><xsl:attribute name="class">parafirst</xsl:attribute></xsl:if>
         <xsl:if test="@id"><a name="{@id}"> </a></xsl:if>
         <xsl:apply-templates />
       </div>
     </xsl:otherwise>
   </xsl:choose>    
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.lq ')]">
   <blockquote><xsl:apply-templates/></blockquote>
</xsl:template>










<!-- this template rule must discern between either direct reference or include with mimetype -->
<xsl:template match="*[contains(@spec,' topic.mmobj ')]">
  <!-- build any pre break indicated by style -->
  <xsl:choose>
    <xsl:when test="parent::fig[contains(@frame,'top ')]">
      <!-- NOP if there is already a break implied by a parent property -->
    </xsl:when>
    <xsl:otherwise>
      <xsl:if test="not(@runin='runin')"><br clear="all"/></xsl:if>
    </xsl:otherwise>
  </xsl:choose>
  <!-- now invoke the actual content and its alt text -->
  <xsl:element name="img">
    <xsl:attribute name="src">
     <xsl:value-of select="$img-path"/>
       <xsl:choose>
         <xsl:when test="@objname">
           <xsl:call-template name="get-objdescinfo"/> <!-- TBD: need to parameterize to get just the href -->
         </xsl:when>
         <xsl:when test="@href">
           <xsl:value-of select="@href"/>
         </xsl:when>
         <xsl:otherwise>
           <!-- no action -->
         </xsl:otherwise>
       </xsl:choose>
     </xsl:attribute>
    <xsl:if test="@height"><xsl:attribute name="height"><xsl:value-of select="@height"/></xsl:attribute></xsl:if>
    <xsl:if test="@width"><xsl:attribute name="width"><xsl:value-of select="@width"/></xsl:attribute></xsl:if>
    <xsl:choose>
      <xsl:when test="@alt">
        <xsl:attribute name="alt"><xsl:value-of select="@alt"/></xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="alt"><xsl:value-of select="textalt|*[contains(@spec,' topic.textalt')]"/></xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@align"><xsl:attribute name="align"><xsl:value-of select="@align"/></xsl:attribute></xsl:if>    
  </xsl:element>
  <!-- build any post break indicated by style -->
  <xsl:if test="not(@runin='runin')"><br clear="all"/></xsl:if>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.textalt ')]">
  <!--xsl:apply-templates/--> <!-- NOP for now... user profile issue? -->
</xsl:template>


<!-- SINGLE PART LISTS -->

<xsl:template name="li.name">
   <xsl:if test="@id">
      <a>
      <xsl:attribute name="name">
         <xsl:value-of select="@id"/>
      </xsl:attribute>
      *</a>
   </xsl:if>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.ul ')]">
  <ul><xsl:copy-of select="@*"/>
    <xsl:apply-templates/>
  </ul>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.ol ')]">
  <ol><xsl:copy-of select="@*"/>
    <xsl:apply-templates/>
  </ol>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.notelist ')]">
  <p><b>
    <xsl:choose>
      <xsl:when test="@title">
        <xsl:if test="not(@title='')">
          <xsl:value-of select="@title"/>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
         <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Notes'"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose><xsl:text>: </xsl:text>
  </b></p>
  <ol>
    <xsl:apply-templates/>
  </ol>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.li ')]">
   <li>
      <xsl:call-template name="li.name"/>
      <xsl:apply-templates/>
   </li>
</xsl:template>


<!-- TWO PART LISTS -->

<xsl:template match="*[contains(@spec,' topic.dl ')]">
  <dl>
    <xsl:apply-templates/>
  </dl>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dlentry ')]">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dt ')]">          
  <dt> <!-- allow selectable view; is this data dependent? -->
    <xsl:apply-templates/>
  </dt>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dd ')]">
  <dd>
    <xsl:apply-templates/>
  </dd>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dthd ')]">
  <dt class="defListHead">
    <xsl:apply-templates/>
  </dt>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.ddhd ')]">
  <dd class="defListHead">
    <xsl:apply-templates/>
  </dd>
</xsl:template>


<!-- if a definition list has headings, cast it as a table -->

<xsl:template match="*[contains(@spec,' topic.dt')]" mode="semtbl">
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dd')]" mode="semtbl">
    <xsl:apply-templates/>
</xsl:template>

<xsl:template match="dl[name(child::*)='dlhead']" priority="1">
  <table border="0">
  <tr>
   <th align="left" bgcolor="#d0d0d0"><i><xsl:value-of select="dlhead/dthd"/></i></th>
   <th align="left" bgcolor="#d0d0d0" cellpadding="4"><i><xsl:value-of select="dlhead/ddhd"/></i></th>
  </tr>
  <xsl:for-each select="*[contains(@spec,' topic.dlentry ')]">
    <tr>
      <td valign="top" bgcolor="#f0f0f0"><xsl:apply-templates select="*[contains(@spec,' topic.dt')]" mode="semtbl"/></td>
      <td valign="top" bgcolor="#f0f0f0" cellpadding="4"><xsl:apply-templates select="*[contains(@spec,' topic.dd')]" mode="semtbl"/></td>
    </tr>
  </xsl:for-each>
</table>
</xsl:template>


<!-- glossary list -->

<xsl:template match="*[contains(@spec,' topic.glentry ')]">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="xgl|*[contains(@spec,' topic.xgl ')]"> <!-- simple, non-sorted version --><!-- not currently in use -->
  <dl>
  <xsl:for-each select="glentry|*[contains(@spec,' topic.glentry ')]">
      <dt><u><xsl:value-of select="gt|*[contains(@spec,' topic.gt')]"/>:</u></dt>
      <dd><xsl:value-of select="gd|*[contains(@spec,' topic.gd')]"/></dd>
  </xsl:for-each>
  </dl>
</xsl:template>


<!-- the next two templates set up a procedure and a set of parameterized calls
     for doing a Latin-based alphabetic sort within a single glossary list. -->

<xsl:template name="sort-range">
  <xsl:param name="letter"/> <!-- get param by reference -->
  <xsl:variable name="caps"> <!-- and cast it to uppercase -->
    <xsl:value-of select="translate($letter,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <a name="{concat('start_0f_',$caps)}"> </a>
  <h3 align="center">--=[ <xsl:value-of select="$caps"/> ]=--</h3>
  <dl>
  <xsl:for-each select="glentry[starts-with(gt,$letter) or starts-with(gt,$caps)]">
    <xsl:sort select="gt"/>
    <xsl:apply-templates/>
  </xsl:for-each>
  </dl>
  <p><a href="#top_of_gl">Back to top</a></p>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.gl ')]">
  <a name="top_of_gl"> </a>
  <!-- In this nav model, the anchor is bound to the top of the nav structure -->
  <center>
  <h3> 
  <a href="#start_0f_A"> A </a>  <a href="#start_0f_B"> B </a>  <a href="#start_0f_C"> C </a>
  <a href="#start_0f_D"> D </a>  <a href="#start_0f_E"> E </a>  <a href="#start_0f_F"> F </a>
  <a href="#start_0f_G"> G </a>  <a href="#start_0f_H"> H </a>  <a href="#start_0f_I"> I </a>
  <a href="#start_0f_J"> J </a>  <a href="#start_0f_K"> K </a>  <a href="#start_0f_L"> L </a>
  <a href="#start_0f_M"> M </a>  <a href="#start_0f_N"> N </a>  <a href="#start_0f_O"> O </a>
  <a href="#start_0f_P"> P </a>  <a href="#start_0f_Q"> Q </a>  <a href="#start_0f_R"> R </a>
  <a href="#start_0f_S"> S </a>  <a href="#start_0f_T"> T </a>  <a href="#start_0f_U"> U </a>
  <a href="#start_0f_V"> V </a>  <a href="#start_0f_W"> W </a>  <a href="#start_0f_X"> X </a>
  <a href="#start_0f_Y"> Y </a>  <a href="#start_0f_Z"> Z </a>
  </h3><br/><br/>
  </center>


  <h3 align="center">--=[ Numeric ]=--</h3>
  <dl>
  <xsl:for-each select="glentry[string(number(substring(gt,1,1)))!='NaN']">
    <xsl:sort select="gt"/>
    <xsl:apply-templates/>
  </xsl:for-each>
  </dl><p><a href="#top_of_gl">Back to top</a></p>

  <h3 align="center">--=[ Special ]=--</h3>
  <dl>
  <xsl:for-each select="glentry[starts-with(gt,'@ ')]">
    <xsl:sort select="gt"/>
    <xsl:apply-templates/>
  </xsl:for-each>
  </dl><p><a href="#top_of_gl">Back to top</a></p>

  <xsl:call-template name="sort-range"><xsl:with-param name="letter">a</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">b</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">c</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">d</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">e</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">f</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">g</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">h</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">i</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">j</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">k</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">l</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">m</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">n</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">o</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">p</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">q</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">r</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">s</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">t</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">u</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">v</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">w</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">x</xsl:with-param></xsl:call-template>
  <xsl:call-template name="sort-range"><xsl:with-param name="letter">y</xsl:with-param></xsl:call-template>

  <xsl:call-template name="sort-range"><xsl:with-param name="letter">z</xsl:with-param></xsl:call-template>

</xsl:template>

<xsl:template match="*[contains(@spec,' topic.gt ')]">
  <a name="term_{.}"> </a>
  <dt><b><xsl:apply-templates/></b></dt>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.gd ')]">
  <dd><xsl:apply-templates /></dd>
</xsl:template>





<!-- QA list (FAQ end-use structure) -->

<xsl:template match="*[contains(@spec,' topic.qalist ')]">
  <xsl:apply-templates/>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.qaentry ')]">
<blockquote>
  <a>
     <xsl:attribute name="name"><xsl:value-of select="generate-id()"/></xsl:attribute>
     <xsl:text> </xsl:text>
  </a>
  <xsl:apply-templates/>
  <p align="LEFT">
    <a href="#TOP">
    <!--xsl:value-of select="$deftxt-linktop"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Return to Top'"/>
      </xsl:call-template>
    </a></p>
</blockquote>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.aitem ')]">
<p><b>A: </b>
  <xsl:apply-templates/>
</p>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.qitem ')]">
<p><b>Q: </b>
  <b><xsl:apply-templates/></b>
</p>
</xsl:template> 

<xsl:template match="*[contains(@spec,' topic.aitem ')]/p[position()='1']">
  <xsl:apply-templates/>
</xsl:template> 

<xsl:template match="aitem/ul[position()='1']">
  <ul><xsl:apply-templates/></ul>
</xsl:template> 


<!-- dialog elements -->
<xsl:template match="*[contains(@spec,' topic.dialog ')]">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dialogdef ')]">
 <div style="padding: 3pt; border: thin black solid;">
  <xsl:apply-templates/>
 </div>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dialogbody ')]">
  <br/>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sys ')]">
  <b>SYS: </b> <xsl:apply-templates/><br/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.user ')]">
  <b>USER: </b> <xsl:apply-templates/><br/>
</xsl:template>



<!-- phrase elements -->

<xsl:template match="*[contains(@spec,' topic.q ')]"> 
  <!-- ideally, need to flipflop between " and ' -->
  <xsl:text>"</xsl:text><xsl:apply-templates/><xsl:text>"</xsl:text>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.cite ')]">
<cite>
  <!-- add some associative linking to literal title text, if no bibdesc -->
  <xsl:call-template name="get-bibdescinfo"/>
</cite>
</xsl:template>


<!-- phrase presentational styles -->

<!-- default behavior -->

<xsl:template match="*[contains(@spec,' topic.ph ')]">
<xsl:choose>
  <xsl:when test="@remap='underlined'">
    <u><xsl:apply-templates/></u>
  </xsl:when>
  <xsl:when test="@remap='italic'">
    <em><xsl:apply-templates/></em>
  </xsl:when>
  <xsl:when test="@remap='bold'">
    <strong><xsl:apply-templates/></strong>
  </xsl:when>
  <xsl:when test="@remap='bolditalic'">
    <b><i><xsl:apply-templates/></i></b>
  </xsl:when>
  <xsl:when test="@remap='smallcaps'">
    <span style="text-transform: uppercase; font-size: smaller;">
      <xsl:apply-templates/>
    </span>
  </xsl:when>
  <xsl:when test="@remap='monospace'">
    <tt><xsl:apply-templates/></tt>
  </xsl:when>
  <xsl:otherwise>
    <span style="color: purple;"><xsl:apply-templates/></span>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>


<!-- phrase "semantic" classes -->

<xsl:template match="*[contains(@spec,' topic.xph ')]">
  <tt><xsl:apply-templates/></tt>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sup ')]">
  <sup><xsl:apply-templates/></sup>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sub ')]">
  <sub><xsl:apply-templates/></sub>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.var ')]">
  <var><xsl:apply-templates/></var>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.term ')]">                 
  <span>
    <!--add 'if exists' test for target prior to building link -->
    <a href="#term_{.}"><xsl:apply-templates/></a>
  </span>
</xsl:template>


<!-- data typed phrases -->

<xsl:template match="*[contains(@spec,' topic.num ')]">
  <span style="color:maroon"><xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.char ')]">
  <span style="color:maroon">'<xsl:apply-templates/>'</span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.hex ')]">
  <span style="color:maroon">x'<xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.dec ')]">
  <span style="color:maroon"><xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.apl ')]">
  <span style="color:maroon; font-face:symbol;"><xsl:apply-templates/></span>
</xsl:template>

<!-- stateful data types -->

<xsl:template match="*[contains(@spec,' topic.boolean ')]">
  <span style="color:green"><xsl:value-of select="@state"/></span>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.state ')]">
  <span style="color:red"><xsl:value-of select="@value"/></span>
</xsl:template>


<!-- links -->

<xsl:template match="*[contains(@spec,' topic.link ')]">               
  <xsl:choose>                        
    <xsl:when test="@href">
         <a href="{@href}"><xsl:apply-templates/></a>
    </xsl:when>
    <xsl:otherwise>
       <xsl:call-template name="get-linkdescinfo"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.xref ')]">
 <!-- this needs to do an id/idref content fetch -->
   <xsl:variable name="xreftext">
     <xsl:choose>
       <xsl:when test="@othertype"      >Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='task'"    >Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='reftopic'">Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='concept'" >Topic "<xsl:value-of          select="id(@refid)/title"/>"</xsl:when>
       <xsl:when test="@type='fig'"     >Figure <xsl:value-of   select="id(@refid)/label"/></xsl:when>
       <xsl:when test="@type='table'"   >Table <xsl:value-of    select="id(@refid)/title"/></xsl:when>
       <xsl:when test="@type='li'"      >Item <xsl:apply-templates mode="xref" select="id(@refid)"/></xsl:when>
       <xsl:when test="@type='step'"    >Step <xsl:apply-templates mode="xref" select="id(@refid)"/></xsl:when>
       <xsl:when test="@type='fn'"      >Footnote <xsl:value-of select="id(@refid)"/></xsl:when>
       <xsl:otherwise>Here</xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="@href">
         <a href="{@href}"><xsl:value-of select="@href"/></a>
      </xsl:when>
      <xsl:when test="@refid">
         <a href="#{@refid}"><xsl:value-of select="$xreftext"/></a>
      </xsl:when>
      <xsl:otherwise>
         <span class="unresolved">
            <xsl:value-of select="@*"/>
         </span>
         <!--br/-->
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>



<xsl:template match="*[contains(@spec,' topic.li ')]" mode="xref">
  <xsl:number/>
</xsl:template>


<!-- record end respecting data -->
 
<xsl:template match="*[contains(@spec,' topic.address ')]"> <!-- this is actually a specific data type -->
  <blockquote>
<!-- quick and dirty workaround for the inconsistent br-replace named template -->
    <pre class="program">
      <font style="font-family: arial">
      <xsl:apply-templates/>
      </font>
    </pre>
<!-- TBD: debug why this routine chops content at word length instead of line length:
  <xsl:call-template name="br-replace">
    <xsl:with-param name="word" select="."/>
  </xsl:call-template>
-->
  </blockquote>
</xsl:template>


<xsl:template match="*[contains(@spec,' topic.pre ')]">
<xsl:if test="contains(@frame,'top')"><hr/></xsl:if>
<xsl:call-template name="gen-att-label"/>
<xsl:choose>
  <xsl:when test="@scale">
    <xsl:variable name="thissize"><xsl:value-of select="@scale*90"/>%</xsl:variable>
    <pre class="program" style="font-size: {$thissize};" >
      <xsl:apply-templates/>
    </pre>
  </xsl:when>
  <xsl:otherwise>
    <pre class="program">
      <xsl:apply-templates/>
    </pre>
  </xsl:otherwise>
</xsl:choose>
<xsl:if test="contains(@frame,'bot')"><hr/></xsl:if>
</xsl:template>
 
<xsl:template match="*[contains(@spec,' topic.lines ')]">
<xsl:call-template name="gen-att-label"/>
<!-- quick and dirty workaround for the inconsistent br-replace named template -->
    <pre class="program">
      <font style="font-family: arial">
      <xsl:apply-templates/>
      </font>
    </pre>
<!-- TBD: debug why this routine chops content at word length instead of line length:
 <p class="lines">
  <xsl:call-template name="br-replace">
    <xsl:with-param name="word" select="."/>
  </xsl:call-template>
 </p>
-->
</xsl:template>
 
<xsl:template match="*[contains(@spec,' topic.screen ')]">
<xsl:call-template name="gen-att-label"/>
<table border="1" cellpadding="6"><tr><td>
  <pre>
    <xsl:apply-templates/>
  </pre></td></tr>
</table>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.cgraphic ')]">
<xsl:call-template name="gen-att-label"/>
  <xsl:choose>
    <xsl:when test="@remap='img'">
      <img src="{$img-path}{text()}.gif"/>
    </xsl:when>
    <xsl:otherwise>
      <pre><xsl:apply-templates/></pre>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- peril notices -->

<xsl:template match="*[contains(@spec,' topic.attention ')]">
  <p><strong>
    <!--xsl:value-of select="$deftxt-attention"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Attention'"/>
      </xsl:call-template><xsl:text>: </xsl:text>
  </strong><br/>
    <xsl:apply-templates/>
  </p>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.caution ')]">
  <p><b>
    <!--xsl:value-of select="$deftxt-caution"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Caution'"/>
      </xsl:call-template><xsl:text>: </xsl:text><br/>
    <xsl:apply-templates/></b>
  </p>
</xsl:template>

  <!-- special context of notices within list items -->
<xsl:template match="*[contains(@spec,' topic.li ')]/*[contains(@spec,' topic.caution ')]" priority="1">
  <b>
    <!--xsl:value-of select="$deftxt-caution"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Caution'"/>
      </xsl:call-template><xsl:text>: </xsl:text><br/>
    <xsl:apply-templates/></b>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.danger ')]">
  <p><blockquote><hr/><strong>
    <!--xsl:value-of select="$deftxt-danger"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Danger'"/>
      </xsl:call-template><xsl:text>: </xsl:text>
    <br/>
    <xsl:apply-templates/>
  </strong><hr/></blockquote></p>
</xsl:template>


<!-- note (mixed content with possibility of block within block) -->
<xsl:template match="*[contains(@spec,' topic.note ')]">
<p>
  <b>
    <xsl:choose>
      <xsl:when test="@title">
        <xsl:if test="not(@title='')">
          <xsl:value-of select="@title"/>
        </xsl:if>
      </xsl:when>
      <xsl:when test="@classif='tip'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Tip'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@classif='fastpath'">
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Fastpath'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="getString">
          <xsl:with-param name="stringName" select="'Note'"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose><xsl:text>: </xsl:text>
  </b>
  <xsl:apply-templates />
</p>
</xsl:template>




<!-- figure display context -->

<xsl:template match="*[contains(@spec,' topic.fig ')]">
  <xsl:if test="contains(@frame,'top')"><hr/></xsl:if>
  <xsl:choose>
    <xsl:when test="@frame='all'">
      <div class="figborder">
      <xsl:call-template name="fig-common"/>
      </div>
    </xsl:when>
    <xsl:when test="@frame='sides'">
      <xsl:call-template name="fig-common"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="fig-common"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:if test="contains(@frame,'bot')"><hr/></xsl:if>
</xsl:template>


<xsl:template name="fig-common">
  <xsl:if test="@id"><a name="{@id}"> </a></xsl:if>
  <xsl:if test="$fmt-fig-lbl-loc='over'"><xsl:call-template name="place-fig-lbl"/></xsl:if>
  <xsl:apply-templates/>
  <xsl:if test="$fmt-fig-lbl-loc='under'"><xsl:call-template name="place-fig-lbl"/></xsl:if>
</xsl:template>


<xsl:template match="*[contains(@spec,' topic.figbody ')]">
  <xsl:apply-templates/>
</xsl:template>




<!-- CALS version of table -->

<xsl:template match="*[contains(@spec,' topic.tgroup ')]">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.spanspec ')]"/>

<xsl:template match="*[contains(@spec,' topic.colspec ')]"></xsl:template>




<xsl:template match="*[contains(@spec,' topic.table ')]">
  <xsl:if test="@id"><a name="{@id}"> </a></xsl:if>
  <xsl:if test="$fmt-tbl-lbl-loc='over'"><xsl:call-template name="place-tbl-lbl"/></xsl:if>
  <xsl:if test="contains(@frame,'top')"><hr/></xsl:if>

  <table>
    <xsl:if test="contains(@spec,' elementref.attlist ')">
      <xsl:attribute name="border">1</xsl:attribute>
    </xsl:if>
    <xsl:if test="desc">
      <xsl:attribute name="longdesc"><xsl:value-of select="desc"/></xsl:attribute>
    </xsl:if>
    <xsl:if test="@frame">
      <xsl:variable name="frameval" select="@frame"/>
      <xsl:choose>
        <xsl:when test="$frameval='none'">
          <xsl:attribute name="border">0</xsl:attribute>
        </xsl:when>
        <xsl:when test="contains($frameval,'top')">
          <!-- any HTML table frame controls for this condition? -->
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="border">1</xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    <xsl:call-template name="tblatts"/>
    <xsl:call-template name="place-tbl-width"/> <!-- pick up global definition -->
    <xsl:apply-templates/>
  </table>
  <br clear="all"/>
  <br clear="all"/>
  <xsl:if test="contains(@frame,'bot')"><hr/></xsl:if>
  <xsl:if test="$fmt-tbl-lbl-loc='under'"><xsl:call-template name="place-tbl-lbl"/></xsl:if>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.row ')]">
  <tr><xsl:call-template name="tblatts"/>
  <xsl:apply-templates/></tr>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.thead ')]">
  <xsl:apply-templates/>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.tfoot ')]">
  <xsl:apply-templates/>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.tbody ')]">
  <xsl:apply-templates/>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.thead ')]/*/*[contains(@spec,' topic.entry ')]" name="thdr">
  <th><xsl:call-template name="tblatts"/>
   <xsl:call-template name="fillit"/>
  </th>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.tfoot ')]/*/*[contains(@spec,' topic.entry ')]" name="tftr">
  <th><xsl:call-template name="tblatts"/>
    <xsl:value-of select="./tft/c"/>
  </th>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.tbody ')]/*/*[contains(@spec,' topic.entry ')]" name="tbod">
  <td><xsl:call-template name="tblatts"/>
   <xsl:call-template name="fillit"/>
  </td>
 </xsl:template>

<xsl:template match="*[contains(@spec,' topic.tnote ')]">
  <tr><xsl:call-template name="tblatts"/>
  <td><xsl:call-template name="tblatts"/>
  <p><b>
  <!--xsl:value-of select="$deftxt-nota"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Note'"/>
      </xsl:call-template><xsl:text>: </xsl:text>
  </b><xsl:apply-templates/></p></td></tr>
 </xsl:template>

<!-- named templates -->
<xsl:template name="fillit">
<xsl:choose>
  <xsl:when test="not(text()[normalize-space(.)] | *)">&nbsp;</xsl:when>
  <xsl:otherwise><xsl:apply-templates/></xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="tblatts">
  <xsl:choose>
    <xsl:when test="@valign"><xsl:attribute name="valign"><xsl:value-of select="@valign"/></xsl:attribute></xsl:when>
    <xsl:otherwise><xsl:attribute name="valign">top</xsl:attribute></xsl:otherwise>
  </xsl:choose>
  <xsl:choose>
    <xsl:when test="@align"><xsl:attribute name="align"><xsl:value-of select="@align"/></xsl:attribute></xsl:when>
    <xsl:otherwise><xsl:attribute name="align">left</xsl:attribute></xsl:otherwise>
  </xsl:choose>
  <xsl:if test="@colsep">
    <xsl:if test="@colsep='1'"><xsl:attribute name="cellpadding">10</xsl:attribute></xsl:if>
  </xsl:if>
  <xsl:if test="@morerows">
    <xsl:attribute name="rowspan"><xsl:value-of select="@morerows+1"/></xsl:attribute>
  </xsl:if>
  <xsl:if test="@colname">
    <xsl:attribute name="colstart"><xsl:value-of select="ancestor::tgroup/colspec[@colname=current()/@colname]/@colnum"/></xsl:attribute>
  </xsl:if>
  <xsl:if test="@namest">
    <xsl:variable name="colst" select="substring-after(@namest,'col')"/>
    <xsl:variable name="colend" select="substring-after(@nameend,'col')"/>
    <xsl:attribute name="colspan"><xsl:value-of select="$colend - $colst + 1"/></xsl:attribute>
  </xsl:if>
</xsl:template>

<!-- end of table section -->




<!-- "FORMAT" GLOBAL DECLARATIONS -->
<xsl:variable name="fmt-fig-lbl-loc">over</xsl:variable><!-- values: over, under -->
<xsl:variable name="fmt-tbl-lbl-loc">over</xsl:variable><!-- values: over, under -->
<xsl:variable name="link-top-section">yes</xsl:variable><!-- values: yes, no (or any not "yes") -->
<xsl:variable name="do-place-ing">no</xsl:variable><!-- values: yes, no (or any not "yes") -->

<!-- this value should be created in a named template since it needs to increment per call -->
<!-- for now, the static value is null until we can redo this intent -->
<xsl:variable name="fig-pfx-txt"></xsl:variable>
<xsl:variable name="xfig-pfx-txt">
 <!--xsl:value-of select="$deftxt-fig"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Figure'"/>
      </xsl:call-template> <xsl:number level="any" count="label" from="/" />
 of <xsl:value-of select="count(//fig/label)"/>. </xsl:variable><!-- values: '' or custom design -->
<xsl:variable name="tbl-pfx-txt"></xsl:variable>


<!-- "FORMAT" MACROS -->
<!-- 
 | These macros support globally-defined formatting constants for
 | document content.  Some elements have attributes that permit local
 | control of formatting; such logic is part of the pertinent template rule.
 +-->

<xsl:template name="place-tbl-width">
<xsl:variable name="twidth-fixed">540</xsl:variable>
  <xsl:if test="$twidth-fixed != ''">
    <xsl:attribute name="width"><xsl:value-of select="$twidth-fixed"/></xsl:attribute>
  </xsl:if>
</xsl:template>

<xsl:template name="place-tbl-lbl">
  <xsl:if test="*[contains(@spec,' topic.title ')]">
    <br/><div>
    <strong>
      <xsl:value-of select="$tbl-pfx-txt"/>
      <xsl:value-of select="*[contains(@spec,' topic.title ')]"/>
    </strong>
    <!-- description text in table captions? -->
    </div>
  </xsl:if>
</xsl:template>


<xsl:template name="place-fig-lbl">
  <xsl:if test="*[contains(@spec,' topic.title ')]">
    <br/><div>
    <strong>
      <xsl:value-of select="$fig-pfx-txt"/>
      <xsl:value-of select="*[contains(@spec,' topic.title ')]"/>
    </strong>
    <xsl:if test="desc">
      <xsl:text>. </xsl:text><xsl:value-of select="*[contains(@spec,' topic.desc ')]"/>
    </xsl:if>
    </div>
  </xsl:if>
</xsl:template>






<!-- Support for Scripts -->
<xsl:template name="script-sample">
<script language="JavaScript">
<xsl:comment>
  // define JavaScript within here; use CDATA sections as appropriate
  self.focus();
  function popwin(popupitem)
  {
         var PopUpWin=window.open(popupitem,"PopUpWin","toolbar=no,directories=no,menubar=no,status=no,scrollbars=yes,resizable=yes,width=500,height=200");
         PopUpWin.focus();
  }
//</xsl:comment>
</script>
</xsl:template>



<!-- NAMED TEMPLATES (call by name, only) -->

<xsl:template name="gen-att-label">
<xsl:if test="@title"><div style="margin-bottom: 0;"><b><xsl:value-of select="@title"/></b></div></xsl:if>
</xsl:template>


<!-- named templates that can be used anywhere -->

<!-- this replaces newlines with the BR element, forcing non-concatenation even in flow contexts -->
<xsl:template name="br-replace">
  <xsl:param name="word"/>
<!-- capture an actual newline within the xsl:text element -->
  <xsl:variable name="cr"><xsl:text>
</xsl:text></xsl:variable>
  <xsl:choose>
    <xsl:when test="contains($word,$cr)">
       <xsl:value-of select="substring-before($word,$cr)"/>
       <br/>
       <xsl:call-template name="br-replace">
         <xsl:with-param name="word" select="substring-after($word,$cr)"/>
       </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$word"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- diagnostic: call this to generate a path-like view of an element's ancestry! -->
<xsl:template name="breadcrumbs">
<xsl:variable name="full-path">
  <xsl:for-each select="ancestor-or-self::*">
    <xsl:value-of select="concat('/',name())"/>
  </xsl:for-each>
</xsl:variable>
<p><b><xsl:value-of select="$full-path"/></b></p>
</xsl:template>

<!-- section-like generated stuff -->

<xsl:template name="gen-relsect"> <!-- maintain formatting consistent with other "sections" -->
  <xsl:if test="./prolog/relgroup">
   <div>
   <h3 class="topictitle3">
    <!--xsl:value-of select="$deftxt-relsect"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Related Information'"/>
      </xsl:call-template>
    </h3>
    <xsl:for-each select="./prolog/relgroup/link | ./prolog/relgroup/rel | ./prolog/relgroup/cite">
      <xsl:call-template name="renderrel"/><br/>
    </xsl:for-each>
    <xsl:if test="./prolog/relgroup/relgroup">
       <b><xsl:value-of select="./prolog/relgroup/relgroup/title"/></b><br/>
     <blockquote>
       <xsl:for-each select="./prolog/relgroup/relgroup/rel">
       <xsl:call-template name="renderrel"/><br/>
       </xsl:for-each>
     </blockquote>
    </xsl:if>
  </div>
  </xsl:if>
</xsl:template>


<xsl:template name="xgen-relsect"> <!-- maintain formatting consistent with other "sections" -->
  <xsl:if test="./prolog/relgroup">
    <div>
      <h3 class="topictitle3">
       <!--xsl:value-of select="$deftxt-relsect"/-->
         <xsl:call-template name="getString">
           <xsl:with-param name="stringName" select="'Related Information'"/>
        </xsl:call-template>
      </h3>
      <xsl:for-each select="./prolog/relgroup/link | ./prolog/relgroup/rel | ./prolog/relgroup/cite">
        <xsl:call-template name="renderrel"/><br/>
      </xsl:for-each>
    <!-- call for gen-sub-relsect for nested relgroups -->
    </div>
  </xsl:if>
</xsl:template>

<xsl:template name="gen-sub-resect">
  <!-- similar to above, but visually nested -->
</xsl:template>

<!-- render any contained footnotes as endnotes.  Links back to reference point -->
<xsl:template name="gen-endnotes">
  <xsl:if test="//fn">
   <hr/>
   <div>
   <h3 class="topictitle3">
      <!--xsl:value-of select="$deftxt-endnotes"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'End Notes'"/>
      </xsl:call-template>
    </h3>
    <xsl:for-each select="//fn">
     <p>
       <xsl:variable name="fnid"><xsl:number from="/" level="any"/></xsl:variable>
       <a name="fntarg_{$fnid}" href="#fnsrc_{$fnid}">
         <sup><xsl:number from="/" level="any"/></sup>
       </a><xsl:text>  </xsl:text>
       <xsl:value-of select="."/>
     </p>
    </xsl:for-each>
  </div>
  </xsl:if>
</xsl:template>


<!-- provide stubs for user provided override extensions -->
<xsl:template name="gen-user-header">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
</xsl:template>

<xsl:template name="gen-user-footer">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
</xsl:template>

<xsl:template name="gen-user-sidetoc">
  <xsl:call-template name="sidetoc"/> <!-- provide as a default freebie -->
</xsl:template>

<xsl:template name="gen-user-scripts">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
  <!-- see (or enable) the named template "script-sample" for an example -->
  <!--xsl:call-template name="script-sample"/-->
</xsl:template>


<xsl:template name="gen-user-panel-title-pfx">
  <!-- to customize: copy this to your override transform, add whatever content you want! -->
</xsl:template>


<!-- listing of topics from calling context only; can be expanded for nesting -->
<xsl:template name="gen-toc">
  <div>
  <h3 class="topictitle3">
    <!--xsl:value-of select="$deftxt-toc"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Contents'"/>
      </xsl:call-template>
  </h3>
   <ul>
    <xsl:for-each select="//topic/title">
     <li>
<!-- this directive provides a "depth" indicator without doing recursive nesting -->
<xsl:value-of select="substring('------',1,count(ancestor::*))"/>
     <a>
       <xsl:attribute name="href">#<xsl:value-of select="generate-id()"/></xsl:attribute>
       <xsl:value-of select="."/>
     </a>
     <!--recursive call for subtopics here"/-->
     </li>
    </xsl:for-each>
   </ul>
  </div>
</xsl:template>

<!-- listing of immediate child topics; can be expanded for nesting -->
<xsl:template name="gen-ptoc">
<xsl:if test="*[contains(@spec,' topic.topic ')]">
  <div>
  <h3 class="topictitle3">
     <!--xsl:value-of select="$deftxt-ptoc"/-->
      <xsl:call-template name="getString">
        <xsl:with-param name="stringName" select="'Local topics'"/>
      </xsl:call-template><xsl:text>: </xsl:text>
  </h3>
   <ol>
    <xsl:for-each select="*[contains(@spec,' topic.topic ')]">
     <li><a>
       <xsl:attribute name="href">#<xsl:value-of select="generate-id()"/></xsl:attribute>
       <xsl:value-of select="title"/>
     </a>
     <!--recursive call for subtopics here"/-->
     </li>
    </xsl:for-each>
   </ol>
  </div>
</xsl:if>
</xsl:template>

<xsl:template name="gen-sect-ptoc">
  <xsl:if test="section"><!-- only in case of child sections of body -->
  <div>
  <h3>Sections in this topic:</h3>
    <ol>
    <xsl:for-each select="section">
     <li><xsl:call-template name="get-sect-heading"/></li>
    </xsl:for-each>
    </ol>
  <hr/>
  </div>
  </xsl:if>
</xsl:template> 



<!-- Particular support for top-level and section-level qalists -->
<!-- Note: qalists in other contexts may not show; extend as necessary -->

<xsl:template name="gen-sect-qatoc">
  <xsl:if test="*[contains(@spec,' topic.section ')]/*[contains(@spec,' topic.qalist ')]"><!-- only in case of child qalists of section -->
  <div>
  <h3>Sections in this FAQ list:</h3>
    <xsl:for-each select="*[contains(@spec,' topic.section ')]">
     <!-- this may need to use an adaptation of get-sect-heading -->
     <p><b><xsl:value-of select="*[contains(@spec,' topic.title ')]"/></b></p>
       <xsl:call-template name="gen-qatoc"/>
    </xsl:for-each>
  <hr/>
  </div>
  </xsl:if>
</xsl:template> 

<xsl:template name="gen-body-qatoc">
  <xsl:if test="*[contains(@spec,' topic.qalist ')]"><!-- only in case of child qalists of body -->
  <p>Questions in this FAQ list</p>
    <xsl:call-template name="gen-qatoc"/>
  <hr/>
  </xsl:if>
</xsl:template>
 
<xsl:template name="gen-qatoc">
   <ol>
    <xsl:for-each select="*[contains(@spec,' topic.qalist ')]/*[contains(@spec,' topic.qaentry ')]">
     <li><a>
       <xsl:attribute name="href">#<xsl:value-of select="generate-id()"/></xsl:attribute>
       <xsl:value-of select="*[contains(@spec,' topic.qitem ')]"/>
     </a>
     <!--recursive call for subtopics here"/-->
     </li>
    </xsl:for-each>
   </ol>
</xsl:template>


<!-- language support (from an algorithm described in XSL FAQ) -->
   <!--
           given the info needed to produce a set of candidates ($str),
           pick the best of the bunch:
           1. $str[lang($Lang)][1]
           2. $str[lang($PrimaryLang)][1]
           3. $str[1]
           4. if not($str) then issue warning to STDERR
   - -->
   <xsl:template name="getString">
     <xsl:param name="stringName"/>
     <xsl:variable name="str" 
        select="$StringFile/strings/str[@name=$stringName]"/>
     <xsl:choose>
       <xsl:when test="$str[lang($Lang)]">
         <xsl:value-of select="$str[lang($Lang)][1]"/>
       </xsl:when>
       <xsl:when test="$str[lang($PrimaryLang)]">
         <xsl:value-of select="$str[lang($PrimaryLang)][1]"/>
       </xsl:when>
       <xsl:when test="$str">
         <xsl:value-of select="$str[1]"/>
       </xsl:when>
       <xsl:otherwise>
         <xsl:message terminate="no">
           <xsl:text>Warning: no string named '</xsl:text>
           <xsl:value-of select="$stringName"/>
           <xsl:text>' found.</xsl:text>
         </xsl:message>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

<!-- other named templates -->

<xsl:template name="sidetoc">
<xsl:if test="/*/*[contains(@spec,' topic.topic ')]"><!-- if there are nested topics... -->
<xsl:comment> Begin Table of Contents </xsl:comment>
<!-- Begin Table of Contents -->
<p>
  <table width="150" align="right" border="0">
    <tr><td height="5" bgcolor="#0033CC" align="center"><b><font color="#FFFFFF">Contents:</font></b></td></tr>
      <xsl:for-each select="/*/*/*[contains(@spec,' topic.title ')]">
        <tr><td class="toc">&nbsp;<a href="#{generate-id()}"><xsl:value-of select="."/></a>
        <!--recursive call for subtopics here"/-->
        </td></tr>
      </xsl:for-each>
  </table>
</p>
<xsl:comment> End Table of Contents </xsl:comment>
</xsl:if>
</xsl:template>

<!-- section with special processing based on a presumed classif -->
<xsl:template match="*[contains(@spec,' topic.section ')][@classif='sidebar']" priority="1">
<xsl:comment>A sample sidebar:</xsl:comment>
<xsl:comment> SIDEBAR (OPTIONAL) </xsl:comment>
  <table align="RIGHT" border="0" cellpadding="3" width="35%" class="sidebar">
    <tr>
      <td bgcolor="yellow"><a name="Sidebar: {title}"></a>
        <b><xsl:value-of select="*[contains(@spec,' topic.title ')]"/></b>
        <br />
        <xsl:apply-templates/>
      </td>
    </tr>
  </table>
<xsl:comment> END OF SIDEBAR </xsl:comment>
</xsl:template>


<!-- semantic tables -->

<xsl:template match="*[contains(@spec,' topic.semtable ')]">
  <table border="1">
    <xsl:apply-templates/>
  </table>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.semhdr ')]">
  <tr>
    <xsl:apply-templates/>
  </tr>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.semhd ')]">
  <th align="left" valign="top">
    <xsl:call-template name="get-title"/>
  </th>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.semrow ')]">
  <tr>
    <xsl:apply-templates/>
  </tr>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.sementry ')]">
  <td align="left" valign="top">
    <xsl:apply-templates/>
  </td>
</xsl:template>


<!-- interactive elements; basically just pass through for HTML -->
<!-- To provide attaching the logic for clientside processing, -->
<!-- make an override of the "script-sample" named template -->

<xsl:template match="*[contains(@spec,' topic.select ')]">
  <xsl:call-template name="sect-heading"><xsl:with-param name="deftitle"></xsl:with-param></xsl:call-template>
  <select>
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates/>
  </select>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.option ')]">
  <option>
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates/>
  </option>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.input ')]">
  <input>
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates/>
  </input>
</xsl:template>






<!-- ===================== FORMATTER DECLARATIONS AND GLOBALS ================ -->

<!-- set up CSS-like attribute sets -->
<!-- tempted to use the shorthand form? if so, beware of "silent overrides" -->

<xsl:attribute-set name="topic" >
  <xsl:attribute name="font-family">helvetica, sans-serif</xsl:attribute>
  <xsl:attribute name="font-size">10pt</xsl:attribute>
</xsl:attribute-set>



</xsl:stylesheet>
