<?xml version="1.0" encoding="UTF-8" ?>

<!--  dtdfix1.xsl 1.0; 08 October 2001
 | Update "dita00" topics to "dita01" level of DTD update
 |
 | Copyright IBM Corporation, 2001
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +======================================
 | NOTE: revise the "doctype-system" line below to produce the desired result DTD
 | for the specific topic type you are converting.  "ditabase.dtd" is a generic
 | type for authoring but it would not apply to specialized topics, for example.
 |
 *-->

<xsl:stylesheet version="1.0" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- customize: change the doctype value below to match your required setup -->
<xsl:output method="xml" indent="yes"
            doctype-system="../dtd/ditabase.dtd"
/>

<!-- change the DTD designation above to match what is needed for your non-topic-dtd content -->
<!-- when creating a new element, must nop out the @spec when copying local attrs -->
<!-- need to raise priority to disambiguate from the generic passthru template -->


<!-- warn if there is a cite with the deprecated @href -->
<xsl:template match="cite" priority="2">
  <xsl:if test="@href">
  <xsl:comment>possible bibref value:
    <xsl:value-of select="@href"/>
  </xsl:comment>
  <xsl:message>Deprecated @href found for citation: <xsl:value-of select="@href"/></xsl:message>
  </xsl:if>
</xsl:template>


<!-- patch the ldesc to linkdesc name change -->
<xsl:template match="ldesc" priority="2">
  <linkdesc>
    <xsl:copy-of select="@*[local-name() != 'spec']" />
    <xsl:apply-templates />
  </linkdesc>
</xsl:template>


<!-- patch the xph to code name change -->
<xsl:template match="xph" priority="2">
  <code>
    <xsl:apply-templates />
  </code>
</xsl:template>

<!-- patch any possible apl to ph remap name change -->
<xsl:template match="apl" priority="2">
  <ph remap="underscored">
    <xsl:apply-templates />
  </ph>
</xsl:template>



<!-- rename a changed element -->
<xsl:template match="fig/label" priority="2">
<xsl:message>Got to fig/label</xsl:message>
  <title>
    <xsl:copy-of select="@*[local-name() != 'spec']" />
    <xsl:apply-templates />
  </title>
</xsl:template>

<!-- rename a changed element -->
<xsl:template match="table/label" priority="2">
  <title>
    <xsl:copy-of select="@*[local-name() != 'spec']" />
    <xsl:apply-templates />
  </title>
</xsl:template>

<!-- rename a changed attribute -->
<xsl:template match="section|refsect|fig|table|note" priority="2">
<xsl:copy>
  <xsl:if test="@label">
    <xsl:message>In default handler: <xsl:value-of select="@label"/></xsl:message>
    <xsl:attribute name="title">
      <xsl:value-of select="@label"/>
    </xsl:attribute>
  </xsl:if>
  <xsl:copy-of select="@id|@classif" />
  <!--xsl:copy-of select="@*[local-name() != 'spec' or local-name() != 'label']"/--> 
  <xsl:apply-templates/>
</xsl:copy>
</xsl:template>




<!-- patch the link direct reference name change -->
<xsl:template match="link" priority="2">
<xsl:copy>
  <xsl:if test="@url">
    <xsl:attribute name="href">
      <xsl:value-of select="@url"/>
    </xsl:attribute>
  </xsl:if>
  <xsl:if test="@ref">
    <xsl:attribute name="linkref">
      <xsl:value-of select="@ref"/>
    </xsl:attribute>
  </xsl:if>
  <xsl:copy-of select="@* [not(name() = 'url' or name() = 'ref')]" />
  <xsl:apply-templates/>
</xsl:copy>
</xsl:template>

<!-- patch the mmobj direct reference name change -->
<xsl:template match="mmobj" priority="2">
<xsl:copy>
  <xsl:if test="@name">
    <xsl:attribute name="href">
      <xsl:value-of select="@name"/>
    </xsl:attribute>
  </xsl:if>
  <xsl:copy-of select="@* [not(name() = 'name')]" />
  <xsl:apply-templates/>
</xsl:copy>
</xsl:template>

<!-- copy the rel/@address content into new rel/@href -->
<xsl:template match="rel" priority="2">
<xsl:copy>
  <xsl:if test="@address">
    <xsl:message>In default handler: <xsl:value-of select="@address"/></xsl:message>
    <xsl:attribute name="href">
      <xsl:value-of select="@address"/>
    </xsl:attribute>
  </xsl:if>
  <!--xsl:copy-of select="@* [not(name() = 'address')]" /--> 
  <!-- above line: allow the old to come through; it is still valid -->
  <xsl:apply-templates/>
</xsl:copy>
</xsl:template>

<!-- patch the link indirect reference name change -->
<xsl:template match="flink" priority="2">
<xsl:copy>
  <xsl:if test="@ref">
    <xsl:message>In default handler: <xsl:value-of select="@ref"/></xsl:message>
    <xsl:attribute name="linkref">
      <xsl:value-of select="@ref"/>
    </xsl:attribute>
  </xsl:if>
  <xsl:copy-of select="@* [not(name() = 'ref')]" />
  <xsl:apply-templates/>
</xsl:copy>
</xsl:template>

<!-- patch the bibdesc indirect reference name change -->
<xsl:template match="bibdesc" priority="2">
<xsl:copy>
  <xsl:if test="@citename">
    <xsl:message>In default handler: <xsl:value-of select="@citename"/></xsl:message>
    <xsl:attribute name="bibname">
      <xsl:value-of select="@citename"/>
    </xsl:attribute>
  </xsl:if>
  <xsl:copy-of select="@* [not(name() = 'citename')]" />
  <xsl:apply-templates/>
</xsl:copy>
</xsl:template>


<!-- updates to the elementref dtd (instanced within ditabase) -->
<xsl:template match="elementname">
<xsl:variable name="all"><xsl:value-of select="normalize-space(.)"/></xsl:variable>
<elementname>
  <element><xsl:value-of select="substring-before($all, ' ')"/></element> <!-- get the first token -->
  <longname><xsl:value-of select="substring-after($all, ' ')"/></longname> <!-- get everything else that follows -->
</elementname>
</xsl:template>




<!-- copy through all other content unchanged -->


<xsl:template match="*[@spec]">
	<xsl:call-template name="specialize">			
		<xsl:with-param name="spec" select="@spec"/>
	</xsl:call-template>
</xsl:template>


<xsl:template name="specialize">
  <xsl:param name="spec"/>
  <xsl:choose> 
    <xsl:when test="contains(normalize-space($spec),' ')">
      <xsl:call-template name="specialize">
        <xsl:with-param name="spec" select="substring-after($spec,' ')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="element_name" select="substring-after(normalize-space($spec),'.')" />	
      <xsl:element name="{$element_name}">
        <xsl:copy-of select="@*[local-name() != 'spec']"/> 
        <xsl:apply-templates/>
      </xsl:element>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>	

<xsl:template match="*|@*|comment()|processing-instruction()|text()">
  <xsl:copy>
    <xsl:apply-templates select="*|@*|comment()|processing-instruction()|text()"/>
  </xsl:copy>
</xsl:template>

</xsl:stylesheet>
