<?xml version="1.0" encoding="UTF-8" ?>

<!--  fld2htm.xsl 1.0; 3 May 2001
 | folder of DITA topics to FO
 |
 | Copyright IBM Corporation, 2001
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +======================================
 | This stylesheet performs a simple aggregation of sequenced topics
 | into a printable "folder" of topics.
 | TBD: add a "Table of Contents" for the collection.
 *-->

<!DOCTYPE xsl:stylesheet [
<!-- entities for use in the generated output (Unicode typographic glyphs) -->
  <!ENTITY gt            "&gt;"> 
  <!ENTITY lt            "&lt;"> 
  <!ENTITY rbl           "&#160;">
  <!ENTITY nbsp          "&#160;">
  <!ENTITY quot          "&#34;">
  <!ENTITY quotedblleft  "&#x201C;">
  <!ENTITY quotedblright "&#x201D;">
  <!ENTITY sqbull        "[]">
  <!ENTITY middot        "&#x00A5;">

  <!ENTITY section       "&#xA7;">
  <!ENTITY endash        "&#x2013;">
  <!ENTITY emdash        "&#x2014;">

  <!ENTITY copyr         "&#xA9;">
  <!ENTITY trademark     "&#x2122;">
  <!ENTITY registered    "&#xAE;">
<!-- create some fixed values for now for customizing later -->
  <!ENTITY copyrowner    "Sample Company">
  <!ENTITY copyrdate     "2001">
  <!ENTITY headerstub    "(stub for header content)">

]>

<xsl:stylesheet version="1.0"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">


<xsl:import href="dit2fo.xsl"/>

<xsl:output method="xml"
 version="1.0" indent="yes"/>

<!-- SETTINGS: -->
<xsl:variable name="trace">no</xsl:variable> <!--set string to 'yes' to turn on trace -->

<!-- CONTROL PARAMETERS: -->
<!-- offset -->
<xsl:param name="basic-start-indent">72pt</xsl:param>
<xsl:param name="basic-end-indent">24pt</xsl:param>


<!-- GLOBALS: -->
<xsl:variable name="dflt-ext">.jpg</xsl:variable> <!-- For Antenna House, set to ".jpg" -->

<xsl:variable name="imgdir">images/</xsl:variable>

<xsl:variable name="thistitle"><xsl:value-of select="//*/fldtitle"/></xsl:variable>



<!-- ====================== template rules for "bkbook" dtd content ==================== -->

<xsl:template match="/"> <!-- no special root level procs at this time -->
  <xsl:comment>Sample DITA to XSL-FO application by IBM Corp., 2001</xsl:comment>
  <xsl:apply-templates/>
</xsl:template>


<xsl:template match="folder">
<!-- this is an "setup" context for book context documents -->
  <xsl:call-template name="folder-setup"/>
</xsl:template>


<xsl:template match="fldabstract">
<fo:block xsl:use-attribute-sets="fldabstract">
  <xsl:apply-templates/>
</fo:block>
</xsl:template>


<xsl:template match="fldtitle">
<fo:block xsl:use-attribute-sets="fldtitle">
  <xsl:apply-templates/>
</fo:block>
</xsl:template>


<!-- =========================== overall book organization ========================= -->

<!-- this template rule defines the overall book organization -->
<xsl:template name="folder-setup">
<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
  <!-- get the overall master page defs here -->
    <xsl:call-template name="define-page-masters"/>
  <!-- place generated content -->
    <xsl:call-template name="front-covers"/>
  <!-- place main content (fall through occurs here) -->
    <xsl:call-template name="main-doc3"/>
  <!-- return to place closing generated content -->
</fo:root>
</xsl:template>



<xsl:template name="define-page-masters">
  <fo:layout-master-set>
    <!-- master set for chapter pages, first page is the title page -->
    <fo:page-sequence-master master-name="chapter-master">
      <fo:repeatable-page-master-alternatives>
        <fo:conditional-page-master-reference page-position="first" 
          odd-or-even="odd"
          master-name="common-page"/><!-- chapter-first-odd -->
        <fo:conditional-page-master-reference page-position="first" 
          odd-or-even="even"
          master-name="common-page"/><!--chapter-first-even"/-->
        <fo:conditional-page-master-reference page-position="rest" 
          odd-or-even="odd"
          master-name="common-page"/><!--chapter-rest-odd"/-->
        <fo:conditional-page-master-reference page-position="rest" 
          odd-or-even="even"
          master-name="common-page"/><!--chapter-rest-even"/-->
      </fo:repeatable-page-master-alternatives>
    </fo:page-sequence-master>

    <fo:simple-page-master master-name="cover"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-top="72pt"/>
    </fo:simple-page-master>

    <fo:simple-page-master master-name="common-page"
      xsl:use-attribute-sets="common-grid">
      <fo:region-body margin-bottom="36pt" margin-top="12pt" />
      <fo:region-before extent="12pt"/>
      <fo:region-after  extent="24pt"/>
    </fo:simple-page-master>

  </fo:layout-master-set>
</xsl:template>



<xsl:template name="front-covers">
  <!-- generate an "outside front cover" page (right side) (sheet 1) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block text-align="right" font-family="Helvetica">
        <!-- set the title -->
        <fo:block font-size="30pt" font-weight="bold" line-height="140%">
          <xsl:value-of select="/bkbook/bkprolog/bktitle"/>
        </fo:block>
        <!-- set the subtitle -->
        <fo:block font-size="24pt" font-weight="bold" line-height="140%" margin-bottom="1in">
          <xsl:value-of select="/bkbook/bkprolog/bksubtitle"/>
        </fo:block>
        <!-- place authors as a vertical list -->
        <xsl:for-each select="/bkbook/bkprolog/bkauthor">
          <fo:block font-size="11pt" font-weight="bold" line-height="100%">
            <xsl:value-of select="."/>
          </fo:block>
        </xsl:for-each>
        <!-- set the brief copyright notice -->
        <fo:block font-size="11pt" font-weight="bold" line-height="100%">
          &copyr; &nbsp;&nbsp;<xsl:value-of select="/bkbook/bkprolog/bkcopyr"/>
        </fo:block>
      </fo:block>
      <!-- Custom cover art/text goes here -->
      <xsl:call-template name="place-cover-art"/>
      <!-- End of custom art section -->
      <fo:block xsl:use-attribute-sets="p" color="purple" text-align="center">[TBD: check cover margin spec]</fo:block>
    </fo:flow>
  </fo:page-sequence>

  <!-- generate an "inside front cover" page (left side) (sheet 2) -->
  <fo:page-sequence master-name="cover">
    <fo:flow flow-name="xsl-region-body" >
      <fo:block xsl:use-attribute-sets="p" color="purple" text-align="center">[TBD: this page intentionally left blank.]</fo:block>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>


<xsl:template name="place-cover-art">
<!-- product specific art, etc. -->
<fo:block margin-top="2pc" font-family="Helvetica" border-style="dashed" border-color="black" border-width="thin" padding="6pt">
  <fo:block font-size="12pt" line-height="100%" margin-top="12pc" margin-bottom="12pc" text-align="center">
    <fo:inline color="purple" font-weight="bold">[cover art/text goes here]</fo:inline>
    <!-- one might imbed SVG directly here for use with FOP, for instance -->
  </fo:block>
</fo:block>
</xsl:template>


<xsl:template name="main-doc3">
  <fo:page-sequence master-name="chapter-master">
    <!-- header: odd page -->
    <fo:static-content flow-name="rest-region-before-odd">
      <fo:block text-align="end" xsl:use-attribute-sets="fldtitle">
        <!-- book title here -->
        <xsl:value-of select="$thistitle"/>
      </fo:block>
      <fo:block>
        <fo:leader leader-pattern="rule" space-after.optimum="12pt" leader-length="100%"/>
      </fo:block>
    </fo:static-content>
    <!-- header: even page -->
    <fo:static-content flow-name="rest-region-before-even">
      <fo:block text-align="start" font-weight="bold">
        <!-- chapter title here (what is the current context?)-->
        <xsl:value-of select="descendant::*/*[contains(@spec,' topic.title ')]"/>
      </fo:block>
      <fo:block>
        <fo:leader leader-pattern="rule" space-after.optimum="12pt" leader-length="100%"/>
      </fo:block>
    </fo:static-content>
    <!-- footer static stuff -->
    <fo:static-content flow-name="xsl-region-after">
      <fo:block text-align="center">
        <fo:page-number/>
      </fo:block>
    </fo:static-content>
    <!-- Flow setup for the main content (frontm, body, backm) (new "chapters" start on odd pages) -->
    <fo:flow flow-name="xsl-region-body">
      <!-- chapter body content here -->
      <xsl:apply-templates/>
    </fo:flow>
  </fo:page-sequence>
</xsl:template>



<!-- set up common attributes for all page definitions -->
<xsl:attribute-set name="common-grid">
  <xsl:attribute name="page-width">51pc</xsl:attribute> <!-- A4: 210mm -->
  <xsl:attribute name="page-height">66pc</xsl:attribute> <!-- A4: 297mm -->
  <xsl:attribute name="margin-top">3pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">3pc</xsl:attribute>
  <xsl:attribute name="margin-left">6pc</xsl:attribute>
  <xsl:attribute name="margin-right">6pc</xsl:attribute>
</xsl:attribute-set>

<!-- set up common attributes for all page definitions -->
<xsl:attribute-set name="fldtitle">
  <xsl:attribute name="font-size">16pt</xsl:attribute>
  <xsl:attribute name="font-weight">bold</xsl:attribute>
</xsl:attribute-set>

<!-- set up common attributes for all page definitions -->
<xsl:attribute-set name="fldabstract">
  <xsl:attribute name="margin-top">3pc</xsl:attribute>
  <xsl:attribute name="margin-bottom">3pc</xsl:attribute>
  <xsl:attribute name="margin-left">6pc</xsl:attribute>
  <xsl:attribute name="margin-right">6pc</xsl:attribute>
</xsl:attribute-set>



</xsl:stylesheet>

