<?xml version="1.0" encoding="UTF-8" ?>

<!-- dit2jh.xsl 1.0; 3 July 2001
 | DITA topic to JavaHelp stylesheet
 |
 | Copyright IBM Corporation, 2001
 | This file is part of the DITA package on IBM's developerWorks site.
 | See license.txt for disclaimers.
 +======================================
 |
 *-->
<!-- This sample stylesheet makes use of a "multiple output files" capability
  that, for XSLT 1.0 engines, was supported only as a proprietary interface.
  Wherever you see an "xt:" namespaced element in this stylesheet, please
  substitute whatever mechanism is used by the XSLT engine of your choice.
  The present support for multiple output documents is an unsupported function
  in DITA, provided solely for demonstrating proof of concept.
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xt="http://www.jclark.com/xt"
                extension-element-prefixes="xt">

<xsl:import href="dit2htm.xsl"/>

<xsl:output method="html" encoding="iso-8859-1"
            indent="no" />

<!-- globals -->
<xsl:variable name="file">out</xsl:variable>
<xsl:variable name="gnt">DUMMY TITLE</xsl:variable>

<!-- ====================================================== -->

<!-- Root rule -->

<xsl:template match="/">  <!--Within the scope of the entire source file, -->

<xt:document method="xml" href="MyHelp.hs">
 <xsl:comment>Sample DITA to HTML application by IBM Corp., 2001</xsl:comment>
 <helpset version="1.0"> 
  <title>JavaHelp Sample</title> 
<xsl:text>
</xsl:text> 
  <maps> 
   <homeID>about</homeID> 
   <mapref location="MyMap.map" /> 
  </maps> 
<xsl:text>
</xsl:text> 
  <view> 
   <type>javax.help.TOCView</type> 
   <data>MyTOC.xml</data> 
   <name>TOC</name> 
   <label>Table Of Contents</label> 
  </view> 
<xsl:text>
</xsl:text> 
 </helpset>
</xt:document>

<xt:document method="xml" href="MyMap.map">
 <xsl:comment>Sample DITA to HTML application by IBM Corp., 2001</xsl:comment>
 <map version="1.0">
<xsl:text>
</xsl:text> 
  <!-- pull any first or second-level sections in as independent chunks -->
  <xsl:for-each select="//bkchapter/*[contains(@spec,' topic.topic ')]|//bksection/*[contains(@spec,' topic.topic ')]">   
   <mapID>
    <xsl:attribute name="target">target<xsl:number level="any" format="1"/></xsl:attribute>
    <xsl:attribute name="url">site<xsl:number level="any" format="1"/>.html</xsl:attribute>
   </mapID>
<xsl:text>
</xsl:text> 
  </xsl:for-each>
 </map>
</xt:document>

<xt:document method="xml" href="MyTOC.xml">
 <xsl:comment>Sample DITA to HTML application by IBM Corp., 2001</xsl:comment>
 <toc version="1.0"> 
<xsl:text>
</xsl:text> 
  <xsl:for-each select="//bkchapter/*[contains(@spec,' topic.topic ')]">
   <tocitem>
    <xsl:attribute name="target">target<xsl:number level="any" format="1"/></xsl:attribute>
    <xsl:attribute name="text"><xsl:value-of select="normalize-space(title)"/></xsl:attribute>
    <xsl:for-each select="bksection/topic">
      <tocitem>
        <xsl:attribute name="target">target<xsl:number level="any" format="1.1"/></xsl:attribute>
        <xsl:attribute name="text"><xsl:value-of select="normalize-space(title)"/></xsl:attribute>
      </tocitem><xsl:text>
   </xsl:text>
    </xsl:for-each>
   </tocitem>
<xsl:text>
</xsl:text> 
  </xsl:for-each> 
 </toc>
</xt:document>

<xsl:apply-templates/> <!-- process the rest of the content here -->

</xsl:template>

<!-- book specific parts -->

<xsl:template match="bkbook">
  <xsl:apply-templates/>
</xsl:template>


<!-- major book divisions -->
<xsl:template match="bkfrontm">
  <h1>Front Matter</h1>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkbody">
  <h1>Body</h1>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkappendixes">
  <h1>Appendixes</h1>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkbackm">
  <h1>Back Matter</h1>
  <xsl:apply-templates/>
</xsl:template>


<!-- major H1 level collections -->
<xsl:template match="bkchapter">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkappendix">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="bkglossary">
  <xsl:apply-templates/>
</xsl:template>


<!-- section level elements -->
<xsl:template match="bksection">
  <xsl:apply-templates/>
</xsl:template>

<!-- pre and post elements -->
<xsl:template match="bkpre|bkpost">
  <xsl:apply-templates/>
</xsl:template>


<!-- special handling -->

<xsl:template match="bkprolog|bknotices"/>

<xsl:template match="*[contains(@spec,' topic.topic ')]/*[contains(@spec,' topic.title ')]" mode="localtopictitle">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.section ')]/*[contains(@spec,' topic.title ')]" mode="localsectitle">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*[contains(@spec,' topic.topic ')]/*[contains(@spec,' topic.title ')]"/>
<xsl:template match="*[contains(@spec,' topic.section ')]/*[contains(@spec,' topic.title ')]"/>

<xsl:template match="*[contains(@spec,' topic.topic ')]/*[contains(@spec,' topic.body ')]">
<xsl:apply-templates/>
</xsl:template>

<!-- topicref cannot be used directly in a document processed by this stylesheet-->
<xsl:template match="topicref">
  <xsl:comment>topicref may not be used directly by this stylesheet.  Merge the document first!</xsl:comment>
  <xsl:message>topicref may not be used directly by this stylesheet.  Merge the document first!</xsl:message>
  <xsl:apply-templates/>
</xsl:template>



<!-- ===== boilerplate named templates ===== -->

<xsl:template name="stdheader">
<a name="_top"> </a>
</xsl:template>


<!-- modify this as suitable for your publication requirements -->
<xsl:template name="stdfooter">
<hr/>
<p align="center"><b>[Home | Copyright | Help]</b></p>
</xsl:template>




<!-- ===== templates to specifically create new output chunks ===== -->



<!-- this rule handles reftopics slightly differently -->
<xsl:template match="*[contains(@spec,' reftopic.reftopic ')]">
<xsl:variable name="outname">site<xsl:number level="any" format="1"/>.html</xsl:variable>

<p>
 <xsl:element name="A">
  <xsl:attribute name="HREF"><xsl:value-of select="$outname"/></xsl:attribute>
  <xsl:value-of select="title"/> [<xsl:value-of select="$outname"/>]
 </xsl:element>
</p>

<xt:document method="html" href="{$outname}">
 <html>
  <head><title><xsl:value-of select="//bktitle"/>: <xsl:value-of select="title"/></title></head>
  <body>
   <xsl:call-template name="stdheader"/>
     <xsl:apply-templates/>
   <xsl:call-template name="stdfooter"/>
  </body>
 </html>
</xt:document>

</xsl:template>



<!-- first level topics in chapters -->

<xsl:template match="bkchapter/*[contains(@spec,' topic.topic ')]"> 
<xsl:variable name="outname">site<xsl:number level="any" format="1"/>.html</xsl:variable>

<p>
 <xsl:element name="A">
  <xsl:attribute name="HREF"><xsl:value-of select="$outname"/></xsl:attribute>
  Chapter: <xsl:value-of select="title"/> [<xsl:value-of select="$outname"/>]
 </xsl:element>
</p>


<xt:document method="html" href="{$outname}">
 <html>
  <head><title><xsl:value-of select="//bktitle"/>: <xsl:value-of select="title"/></title></head>
  <body>
   <xsl:call-template name="stdheader"/>
   <h1>Chapter: <xsl:apply-templates select="title" mode="localtopictitle"/></h1>
   <xsl:apply-templates/>
   <xsl:call-template name="stdfooter"/>
  </body>
 </html>
</xt:document>

</xsl:template>





<!-- first level topics in appendixes -->

<xsl:template match="bkappendix/*[contains(@spec,' topic.topic ')]"> 
<xsl:variable name="outname">site<xsl:number level="any" format="1"/>.html</xsl:variable>

<p>
 <xsl:element name="A">
  <xsl:attribute name="HREF"><xsl:value-of select="$outname"/></xsl:attribute>
  Appendix: <xsl:value-of select="title"/> [<xsl:value-of select="$outname"/>]
 </xsl:element>
</p>


<xt:document method="html" href="{$outname}">
 <html>
  <head><title><xsl:value-of select="//bktitle"/>: <xsl:value-of select="title"/></title></head>
  <body>
   <xsl:call-template name="stdheader"/>
   <h1>Appendix: <xsl:apply-templates select="title" mode="localtopictitle"/></h1>
   <xsl:apply-templates/>
   <xsl:call-template name="stdfooter"/>
  </body>
 </html>
</xt:document>

</xsl:template>




<!-- first level topics in glossaries -->

<xsl:template match="bkglossary/*[contains(@spec,' topic.topic ')]"> 
<xsl:variable name="outname">site<xsl:number level="any" format="1"/>.html</xsl:variable>

<p>
 <xsl:element name="A">
  <xsl:attribute name="HREF"><xsl:value-of select="$outname"/></xsl:attribute>
  Glossary: <xsl:value-of select="title"/> [<xsl:value-of select="$outname"/>]
 </xsl:element>
</p>

<xt:document method="html" href="{$outname}">
 <html>
  <head><title><xsl:value-of select="//bktitle"/>: <xsl:value-of select="title"/></title></head>
  <body>
   <xsl:call-template name="stdheader"/>
   <h1>Glossary: <xsl:apply-templates select="title" mode="localtopictitle"/></h1>
   <xsl:apply-templates/>
   <xsl:call-template name="stdfooter"/>
  </body>
 </html>
</xt:document>

</xsl:template>





<!-- second level topics in chapters (sections) -->

<xsl:template match="bksection/*[contains(@spec,' topic.topic ')]"> 
<xsl:variable name="outname">site0<xsl:number level="any" format="1"/>.html</xsl:variable>

<blockquote>
 <xsl:element name="A">
  <xsl:attribute name="HREF"><xsl:value-of select="$outname"/></xsl:attribute>
  <xsl:value-of select="title"/> [<xsl:value-of select="$outname"/>]
 </xsl:element>
</blockquote>


<xt:document method="html" href="{$outname}">
 <html>
  <head><title><xsl:value-of select="//bktitle"/>: <xsl:value-of select="title"/></title></head>
  <body>
   <xsl:call-template name="stdheader"/>
   <h2><xsl:apply-templates select="title" mode="localsectitle"/></h2>
   <xsl:apply-templates/>
   <xsl:call-template name="stdfooter"/>
  </body>
 </html>
</xt:document>

</xsl:template>




</xsl:stylesheet>